/*
 * Cache Manager
 * 
 * V2.0.7
 */

/* global WebOSManager, SOAPClient, XPathResult, utils, base64js, LibraryManager, LayoutManager, ScheduleManager, ERRORS, XMLDocument, application, logic, Logger */

"use strict";

/**
 * @classdesc Provide cache functionality.
 * @type CacheManager
 * @version 2.0.7
 * @since 2.0.7
 */
var CacheManager = new function () {
  // <editor-fold defaultstate="collapsed" desc="Private variables">
  var v_LayoutDurations = {};
  var cycleSubList = [];
  /** Holds the layouts that are in a campaign ...*/
  var campaignOrder = {};
  /** Holds the campaig that played before ...*/
  var currentGroupKey = -1;
  /**
   * Keep track of how many plays a layout did per hour.
   * @type {Object} Object with layout ids as keys and plays per hour and last (re)set time.
   */
  var maxPlaysPerHour = {};
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Private functions">
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Public functions">
  
  /**
   * Set the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} duration The duration of the layout in seconds.
   * @returns {undefined} 
   */
  this.setLayoutDuration = function(id , duration){
    v_LayoutDurations[id] = duration;
  };
  
  /**
   * Get the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} defaultDuration The duration of the layout in seconds if not previously set.
   * @returns {String|Number}  The duration of the layout in seconds.
   */
  this.getLayoutDuration = function(id , defaultDuration){
    if (Object.keys(v_LayoutDurations).indexOf(id)!==-1){
        return v_LayoutDurations[id];
    }
    return defaultDuration;
  };
  this.setCycleListSequence  = function (media) {
    var j;
    var pcycleref ;
    var mediaflag = 0;// 1 if media exits !

    // if (Object.keys(cycleSubList).indexOf(media.parentWidgetId)===-1){
    if (!cycleSubList[media.parentWidgetId]){
      cycleSubList[media.parentWidgetId] = {
        count: media.countplay,
        Israndom: media.IsRandom,
        counter: media.countplay,
        mediaIdx: -1,
        mediaId: []
      }
    }
    for (j = 0; j < cycleSubList[media.parentWidgetId].mediaId.length; j++) {
      if (cycleSubList[media.parentWidgetId].mediaId[j].mid === media.id) {
        mediaflag = 1;
        break;
      }
    }
    if (!mediaflag) {
      cycleSubList[media.parentWidgetId].mediaId.push({
        mid: media.id,
        media: media,
      })
    }
  }
  /**
   * Get the next widget from the cycle
   * @param {String} media The duration of the layout in seconds if not previously set.
   * @returns {String|Number}  The duration of the layout in seconds.
   */
  this.counter = function (media) {
    var parentWid = cycleSubList[media.parentWidgetId];
    if (parentWid.count == parentWid.counter) {
      parentWid.counter = 0;
    } else {
      parentWid.counter += 1;
    }
    return parentWid.counter;
  }
  this.getNextWidgetFromCycle = function (media) {
    var c;
    var current_mediaIdx = cycleSubList[media.parentWidgetId].mediaIdx;
    var allMedia = cycleSubList[media.parentWidgetId].mediaId;

    c = CacheManager.counter(media);

    if (c === 0 ){
      CacheManager.counter(media);
      if (cycleSubList[media.parentWidgetId].Israndom == "1") {
        current_mediaIdx = Math.floor(Math.random() * allMedia.length);
      } else {
        current_mediaIdx += 1;
        if (current_mediaIdx >= allMedia.length) {
          current_mediaIdx = 0;
        }
      }
    }
    cycleSubList[media.parentWidgetId].mediaIdx = current_mediaIdx;
    media = allMedia[current_mediaIdx].media;
    return media;
  };

  /**
   * Add a play count for the specific layout.
   * @param layoutId The id of the layout.
   */
  this.mpph_addPlay = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      CacheManager.mpph_getPlays(layoutId);
    }
    maxPlaysPerHour[layoutId].plays += 1;

  };
  /**
   * Get the number of plays in the current hour.
   * @param layoutId The id of the layout.
   * @returns {Object} An object with the plays and the time started/reset.
   */
  this.mpph_getPlays = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      maxPlaysPerHour[layoutId] = {
        plays : 0,
        time : new Date()
      }
    }

    return maxPlaysPerHour[layoutId];
  };

  /**
   * Reset the plays per hour and the time.
   * @param layoutId The id of the layout.
   */
  this.mpph_reset = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      CacheManager.mpph_getPlays(layoutId);
    }else {
      maxPlaysPerHour[layoutId].plays = 0;
      maxPlaysPerHour[layoutId].time = new Date();
    }
  };
  this.setCampaignOrder = function(groupkey,campaignLayout){
    campaignOrder[groupkey] = campaignLayout;
  }
  this.getCampaignOrder = function(groupkey){
    return campaignOrder[groupkey];
  }
  this.getCampaignOrderKeys = function (){
    return Object.keys(campaignOrder);
  }
  this.deleteCampaign = function(groupkey){
    delete campaignOrder[groupkey];
  }
  this.setCurrentGroupKey= function (groupkey){
    currentGroupKey=groupkey;
  }
  this.getCurrentGroupKey = function (){
    return currentGroupKey;
  }
  // </editor-fold>
  // <editor-fold defaultstate="collapsed" desc="Native CMS Calls">
  
  // </editor-fold>
};