/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */
/*
 * Communication Manager
 * 
 * V0.0.1
 */

/* global CommunicationManager, DeviceManager, SOAPClient, XPathResult, utils, base64js, LibraryManager, LayoutManager, ERRORS, XMLDocument, application, logic, Logger, defined, SOAPClientParameters */
(function () {
  "use strict";

  /**
   * @classdesc Provide communication with CMS through XMDS calls.
   * @type CommunicationManager
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.CommunicationManager = new function _CommunicationManager() {

    /* +------------------------+ Private variables +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private variables">

    var MAXCHUNKSIZE = 512000;
    var inSendMediaInventory = false;

    var v_ReportProgress = 0;

    /**
     * Dispaly Authentication server URL.
     */
    var c_authServerURL = "https://auth.signlicence.co.uk";

    /**
     * Display Authentication details object
     * @param {String} userCode The user code
     * @param {String} deviceCode The device code
     * @param {Number} connectTimer Reference to the timer when active
     */
    var v_DisplayAuthDetails = {
      userCode: null,
      deviceCode: null,
      connectTimer: undefined
    };
    // </editor-fold>

    /* +------------------------+ Private functions +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Private functions">

//		/**
//		 * Download big (>10240 bytes) media file and writes to file.
//		 * @param {Function} cbSuccess Function to call on success.
//		 * @param {Function} cbFailure Function to call on failure.
//		 * @param {Object} options
//		 * @see options must have URL and filename fields set.
//		 * @version 1.8.1
//		 * @since 1.8.1
//		 */
//		var downloadFileHttp = function (cbSuccess, cbFailure, options) {
//			//function downloadFile(URL, targetURL) {
//			var MAXCHUNKSIZE = 10240;
//			var startRange = 0;
//			var endRange = MAXCHUNKSIZE - 1;
//
//			var isGetFileSize = false;
//			var totalFileSize;
//			var downloadedFileSize = 0;
//
//			var xhr_timeout;
//			var fileStartTime = 0;
//
//
//			function do_downloadFile() {
//				function appendFile(data) {
//					var successCb = function (cbObject) {
//						// Calculate download progress
//						var percentage = parseInt((downloadedFileSize / totalFileSize) * 100);
//						var fileRate = (downloadedFileSize / ((Date.now() - fileStartTime) / 1000)) | 0;
//
//						if ((v_ReportProgress++) > 10) {
//
//							console.debug(utils.getTimeStamp() + " - [" + percentage + "%] Downloading : " + downloadedFileSize +
//									"/" + totalFileSize + " Bytes" + ", Rate: " + fileRate);
//							v_ReportProgress = 0;
//						}
//						// Download next data
//						if (downloadedFileSize < totalFileSize) {
//							do_downloadFile();
//						}
//						// Download completed
//						else if (downloadedFileSize === totalFileSize) {
//							if (defined(xhr_timeout)) {
//								clearTimeout(xhr_timeout);
//							}
//							console.debug("[" + percentage + "%] Downloading : " +
//									downloadedFileSize + "/" +
//									totalFileSize + " Bytes" + ", Rate: " + fileRate);
//
//							cbSuccess();
//						}
//						// Error case. Downloaded data size > Original data size
//						else {
//							console.error("Invalid write");
//							do_downloadFile();
//						}
//					};
//
//					var fileoptions = {
//							data: data,
//							path: options.filename,
//							length: MAXCHUNKSIZE
//					};
//
//					DeviceManager.FileSystem.appendFile(successCb, cbFailure, fileoptions);
//				}
//
//				// Download data using AJAX
//				var xhr = new XMLHttpRequest();
//				xhr.open("GET", options.URL, true);
//				xhr.setRequestHeader("Content-Type", "application/octet-stream");
//				xhr.responseType = "arraybuffer";
//
//				// Set the download range for file
//				if (isGetFileSize) {
//					xhr.setRequestHeader("Range", "bytes=" + startRange + "-" + endRange);
//				}
//
//				xhr.onerror = function (e) {
//					if (defined(xhr_timeout)) {
//						clearTimeout(xhr_timeout);
//					}
//					console.log("[ERROR]", e);
//					cbFailure(e);
//				};
//
//				xhr.onreadystatechange = function () {
//
//					// Not found
//					if (xhr.readyState === 4 && xhr.status === 404) {
//						xhr.abort();
//						console.error("Download error 404 ");
//						cbFailure({ errorCode : 404, errorText : "Not found"});
//					}
//
//					// First connection
//					if (xhr.readyState === 4 && xhr.status === 200) {
//						console.debug("Starting to download small file ....");
//
//					}
//					// Download data chunk
//					else if (xhr.readyState === 4 && xhr.status === 206) {
//						downloadedFileSize += Number(xhr.getResponseHeader("Content-Length"));
//						var buf = xhr.response;
//						if (buf) {
//							var responseBuffer = new Int8Array(buf);
//							startRange = endRange + 1;
//							endRange += MAXCHUNKSIZE;
//							appendFile(responseBuffer);
//						}
//					}
//					// Initialize case. Get File Size.
//					else if (!isGetFileSize) {
//						isGetFileSize = true;
//						fileStartTime = Date.now();
//						totalFileSize = Number(xhr.getResponseHeader("Content-Length"));
//						console.debug("File size: " + totalFileSize + ", Download: " + downloadedFileSize);
//						xhr.abort();
//
//						if (totalFileSize < MAXCHUNKSIZE)
//						{
//							//Logger.log("Downloading small file!!", "info");
//							endRange = totalFileSize - 1;
//							do_downloadFile();
//						} else {
//							if (downloadedFileSize < totalFileSize){
//								do_downloadFile();
//							} else if (downloadedFileSize == totalFileSize){
//								console.debug("The file is downloaded completely already.");
//							} else {
//								console.debug("Invalid download");
//							}
//						}
//
//					}
//				};
//
//				/* Timeout */
//				if (defined(xhr_timeout)) {
//					clearTimeout(xhr_timeout);
//				}
//
//				xhr_timeout = setTimeout(function () {
//					console.debug("Download interrupted! Resuming...");
//					if (isGetFileSize) {
//						xhr.abort();
//						do_downloadFile();
//					}
//				}, 5000);
//
//				xhr.send();
//			}
//
//			do_downloadFile();
//		};

    /**
     * @version 1.8.1
     * @since 1.8.1
     * @argument {Object} rt
     */
    var getFaultResponse = function (rt) {
      var faultcode = rt.evaluate('.//faultcode', rt, null,
        XPathResult.FIRST_ORDERED_NODE_TYPE,
        null).singleNodeValue;
      var faultstring = rt.evaluate('.//faultstring', rt, null,
        XPathResult.FIRST_ORDERED_NODE_TYPE,
        null).singleNodeValue;

      var errObj = {
        errorCode: faultcode.textContent,
        errorText: faultstring.textContent
      };
      return errObj;
    };
    // </editor-fold>

    /* +------------------------+ Public functions  +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Public functions">

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    //jshint unused:true
    this.init = function (cbSuccess, cbFailure) {
      //jshint unused:false
      cbSuccess();
      //getCMSVersion(cbSuccess, cbFailure);
    };

    /**
     * Connect to CMS instance in order to register or get approval status from.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} opts Object with options
     * <br>opts.URL The URL of the CMS to connect. If not set, confιguration value will be used instead.
     * <br>opts.serverKey The Key of the CMS. If not set, confιguration value will be used instead.
     * @version 2.0.201
     * @since 1.8.1
     */
    this.ConnectCMS = function (cbSuccess, cbFailure, opts) {

//			var connectFailedCb = function(cbSuccess, cbFailure){

//			};

      var doActions = function (cbObj) {

        DeviceManager.Config.setCMSValid(true);
        DeviceManager.Config.setOptions(cbObj.options);
        DeviceManager.Config.setCommands(cbObj.commands);
        DeviceManager.Config.saveConfiguration();

        var options = cbObj.options;

        application.statusPage.displayName(options.displayName);
        application.statusPage.screenShotInterval(options.screenShotRequestInterval);
        application.statusPage.downloadWindowStart(options.downloadStartWindow);
        application.statusPage.downloadWindowEnd(options.downloadEndWindow);
        application.statusPage.updateWindowStart(options.updateStartWindow);
        application.statusPage.updateWindowEnd(options.updateEndWindow);
        application.setMenuHideTimeout(options.actionBarDisplayDuration);
        application.setMenuInitialState(options.actionBarMode);
        application.statusPage.cmsAddr(DeviceManager.Config.getURL());

        if (DeviceManager.Device.getOrientation() !== options.orientation) {
          DeviceManager.Device.setNativePortraitMode(function () {
            LayoutManager.orientationChanged();
          }, logic.nonFatalError, options.orientation);
        }

        if (cbSuccess) {
          cbSuccess(cbSuccess, cbFailure, cbObj);
        }
      };

      registerDisplay(function (cbObj) {
          var i, j, k, maxj, maxk;
          var parser = new DOMParser();
          var xmlDoc = parser.parseFromString(cbObj, "text/xml");

          var displayElement = xmlDoc.childNodes[0];
          var display = {name: 'display'};
          for (i = 0; i < displayElement.attributes.length; i++) {
            display[displayElement.attributes[i].name] = displayElement.attributes[i].value;
          }
          var options = {};
          var commands = {};
          for (i = 0; i < displayElement.childNodes.length; i++) {
            //Logger.log(displayElement.childNodes[i], Logger.DEBUG);
            if (displayElement.childNodes[i].childNodes.length === 0 || /* Nodes with out value are setting with no value but hava to add! */
              displayElement.childNodes[i].childNodes[0].nodeType === XMLDocument.TEXT_NODE) {
              options[displayElement.childNodes[i].nodeName] = displayElement.childNodes[i].textContent;
            } else {
              if (displayElement.childNodes[i].nodeName === "commands") {
                for (j = 0, maxj = displayElement.childNodes[i].childNodes.length; j < maxj; j++) {
                  var node = displayElement.childNodes[i].childNodes[j];
                  commands[node.localName] = {};
                  for (k = 0, maxk = node.childNodes.length; k < maxk; k++) {
                    commands[node.localName][node.childNodes[k].localName] = node.childNodes[k].textContent;
                  }
                }
              } else {
                console.debug("Found unknown element " + displayElement.childNodes[i].localName + "!", {
                  method: "registerDisplay",
                  data: displayElement.childNodes[i]
                });
              }
            }
          }
          display.options = options;
          display.commands = commands;
          doActions(display);

        }, function (cbObj) {
          /* 
           * Connection to CMS failed. What does this mean to us?
           * Inform the Schedule manager that connection is unavailable!
           */
          console.warn("Connection to CMS failed!");
          if (cbObj.errorCode === ERRORS.WRONGSERVERKEYCODE) {
            cbSuccess(cbSuccess, cbFailure, cbObj);
          } else {
            cbFailure(cbObj);
          }
        },
        opts);
    };

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.getRequiredFiles = function (cbSuccess, cbFailure) {
      getRequiredFiles(cbSuccess, cbFailure);
    };


    /**
     * Download a file with XMDS.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} options Options for downloading file.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.downloadFile = function (cbSuccess, cbFailure, options) {
      if ((v_ReportProgress++) > 10) {
        console.debug("Remain size: " + options.remainSize);
        v_ReportProgress = 0;
      }
      if (!options.hasOwnProperty("remainSize")) {
        options.remainSize = options.size;
        options.offset = 0;
        DeviceManager.FileSystem.deleteFile(function () {
          CommunicationManager.downloadFile(cbSuccess, cbFailure, options);
        }, function () {
          CommunicationManager.downloadFile(cbSuccess, cbFailure, options);
        }, {path: options.path});
        return;
      }
      var size = options.remainSize * 1;
      if (options.remainSize > MAXCHUNKSIZE) {
        size = MAXCHUNKSIZE;
      }

      var fileOptions = {
//					downloadMethod: options.download,
        fileId: options.id,
        fileType: options.type,
        chunkOffset: options.offset,
        chunkSize: size
      };

      getFile(function (cbObject) {
        options.offset += size;
        options.remainSize -= (size);
        //console.log("downloadFile-getFile:", options, options.offset, options.size, options.offset+options.size);
        /* Save the downloaded chunk into the file */
        //Need to convert Uint8Array to ByteArray
        var arr = Array.from(cbObject);
        var fileOptions = {
          data: arr,
          path: options.path,
          size: size,
          writeType: "byteArray"
        };
        DeviceManager.FileSystem.appendFile(function () {
          if (options.remainSize === 0) /* Download file finished */
          {
            v_ReportProgress = 0;
            //console.log("Download finished!");

            /* We have to revalidate the file in order to know if the download was success! */

            options.lastChecked = utils.getTimeStamp();
            //options.isCorrect = true;
            cbSuccess(options);
          } else if (options.remainSize < 0) {
            cbFailure({errorCode: 10003, errorText: "Download file lailed. Filesize less than zero."});
          } else {
            CommunicationManager.downloadFile(cbSuccess, cbFailure, options);
          }
        }, cbFailure, fileOptions);

      }, function (cbObject) {
        cbFailure(cbObject);
      }, fileOptions);
    };

    /**
     * Download a resource from CMS.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} options Options for downloading resource.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.downloadResource = function (cbSuccess, cbFailure, options) {

      var fileOptions = {
        layoutId: options.layoutid,
        regionId: options.regionid,
        mediaId: options.mediaid
      };


      var getResourceSuccessCb = function (data) {
        var fileOptions = {
          data: data,
          path: options.saveAs,
          size: options.size
        };

        DeviceManager.FileSystem.saveFile(function () {
          cbSuccess();
        }, cbFailure, fileOptions);
      };

      getResource(function (data) {
        options.lastChecked = utils.getTimeStamp();
        options.isCorrect = true;
        options.size = data.length;
        getResourceSuccessCb(data);
      }, cbFailure, fileOptions);
    };

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendMediaInventory = function (cbSuccess, cbFailure) {
      if (inSendMediaInventory === false) {
        inSendMediaInventory = true;

        var inventory = '<files>';

        /* fill the invedory with status */
        for (var i = 0, max = LibraryManager.length(); i < max; i++) {
          var fileInfo = LibraryManager.getFileByIndex(i);

          if (fileInfo.type === "media" || fileInfo.type === "layout" || fileInfo.type === "resource") {
            var comp = 0;
            if (fileInfo.hasOwnProperty("isCorrect")) {
              comp = (fileInfo.isCorrect === true) ? 1 : 0;
            }

            var fileEntry = "<file type=\"" + fileInfo.type + "\" id=\"" + fileInfo.id + "\" complete=\"" + comp + "\" md5=\"" + fileInfo.md5 + "\" lastChecked=\"" + fileInfo.lastChecked + "\" />";

            inventory += fileEntry;
          }
        }
        inventory += '</files>';

        mediaInventory(cbSuccess, cbFailure, {MediaInventory: inventory});
      }

    };

    /**
     * Notify status to CMS
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendInfo = function (cbSuccess, cbFailure, additionalInfo) {

      var v_CurLayoutId = LayoutManager.getCurrentLayoutId();
      if (!defined(v_CurLayoutId) || v_CurLayoutId === 'splash') {
        v_CurLayoutId = "0";
      }

      var vStatus = {
        currentLayoutId: v_CurLayoutId,
      };

      /*Whether or not the last received Command was executed successfully*/
      vStatus.lastCommandSuccess = logic.status.getLastCommandStatus();

      /*The name of the physical device*/
      vStatus.deviceName = DeviceManager.Device.getDeviceName();

      /*"An IANA timezone identifier for the Displays timezone"*/
      vStatus.timeZone = DeviceManager.Device.getTimeZone();

      vStatus.width = screen.width;
      vStatus.height = screen.height;

      /*Add additional info*/
      if (additionalInfo) {
        vStatus.additionalInfo = additionalInfo;
      }

      var send = function () {
        notifyStatus(cbSuccess, cbFailure, {status: vStatus});
      };

      /* Other properties */
      DeviceManager.Device.collectRealTimeDeviceInfo(function () {
        var props = DeviceManager.Device.getRealtimeDeviceInformation();
        for (var prop in props) {
          vStatus[prop] = props[prop];
        }
        vStatus.availableSpace = vStatus.storage_free;
        vStatus.totalSpace = vStatus.storage_total;
        send();
      }, send);

    };

    /**
     * Send the status screen to CMS. For debugging purpuses!
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendStatusInfo = function (cbSuccess, cbFailure) {

      var stat = {
        "Version ": (application.statusPage.appVersion()),
        "Code Version ": application.statusPage.appVerCode(),
        "Content Management System ": application.statusPage.cmsAddr() + " (" + application.statusPage.xmrAddr() + ")",
        "Device Model ": application.statusPage.deviceModel(),
        "Firmware Version ": application.statusPage.firmwareVersion(),
        "Tizen Version ": application.statusPage.tizenVersion(),
        "Memory Limit ": application.statusPage.memoryTotal() + " MB",
        "Memory Free ": application.statusPage.memoryFree() + " MB",
        "Memory Used ": application.statusPage.memoryUsed() + " MB",

        spaceTotal: application.statusPage.spaceTotal(),
        spaceFree: application.statusPage.spaceFree(),
        spaceUsed: application.statusPage.spaceUsed(),
        //"Storage ": application.statusPage.spaceTotal(),
        // "Free Space ": application.statusPage.spaceFree(),
        // "Used Space ": application.statusPage.spaceUsed(),
        "Space Available ": application.statusPage.spaceAvail(),

        "Display Name ": application.statusPage.displayName(),

        "License ": application.statusPage.licenseValue(),
        "Registered ": application.statusPage.cmsRegistered(),
        "Current Layout ": application.statusPage.currentLayout(),
        "Screen Size ": application.statusPage.screenSize(),
        "Screen Shot Request Interval ": application.statusPage.screenShotInterval(),

        "Register to CMS ": application.statusPage.cmsRegInfo() + " Checked at " + application.statusPage.cmsRegInfoTime(),

        "Schedule Status ": application.statusPage.scheduleStatus(),
        "Files ": application.statusPage.downloadedFiles() + "/" + application.statusPage.totalFiles(),
        "Remaining Files ": application.statusPage.remainingFiles(), // Queued Number Connections
        "Failed XMDS Connections ": application.statusPage.failedXMDSConnections(),


        allLayouts: application.statusPage.allLayouts.decorated().join(", "),
        "Next Layout ": application.statusPage.nextLayout(),
        scheduledLayouts: application.statusPage.scheduledLayouts_view.decorated().join(", "),
        validLayouts: application.statusPage.validLayouts.decorated().join(", "),
        invalidLayouts: application.statusPage.invalidLayouts.decorated().join(", "),

        // "All Layouts ": application.statusPage.allLayouts.decorated(),
        // "Next Layout ": application.statusPage.nextLayout(),
        // "Scheduled Layouts ": application.statusPage.scheduledLayouts_view.decorated(),
        // "Valid Layouts ": application.statusPage.validLayouts.decorated(),
        // "Invalid Layouts ": application.statusPage.invalidLayouts.decorated(),

        // "Black List ": application.statusPage.blacklist(),
        // "Last 5 Log Messages " : application.statusPage.logMessages(),
        blacklistArray: application.statusPage.blacklist().join(", "),
        logMessagesArray: application.statusPage.logMessages().map(function (itm) {
          return "category : " + itm.category + ", date : " + itm.date + ", message: " + itm.message
        }),

        "Download Window ": application.statusPage.downloadWindowStart(),
        "Download Window End": application.statusPage.downloadWindowEnd(),
        "Update Window ": application.statusPage.updateWindowStart(),
        "Update Window End ": application.statusPage.updateWindowEnd(),
        "CMS Version Information ": application.statusPage.versionInformationCMS()
      };

      var send = function () {
        notifyStatus(cbSuccess, cbFailure, {status: {statusDialog: JSON.stringify(stat)}});
      };

      send();

    };

    /**
     * Notify CMS for an event.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} notification Object with property/ies, value pair(s).
     * @version 1.8.1
     * @since 1.8.1
     */
    this.emergencyNotify = function (cbSuccess, cbFailure, notification) {
      var vNotify = {};
      for (var prop in notification) {
        vNotify[prop] = notification[prop];
      }

      notifyStatus(cbSuccess, cbFailure, {status: vNotify});
    };

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.fetchSchedule = function (cbSuccess, cbFailure) {
      getSchedule(cbSuccess, cbFailure);
    };

    /**
     * Tries to find the CMS Version
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.findCMSVersion = function (cbSuccess, cbFailure) {
      getCMSVersion(cbSuccess, cbFailure);
    };

    /**
     * Submit a screen shot of the display.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} options Object that provides information about the screenshot. See below.
     * @see options.imagedata {Object} The screenshot as a Base64-encoded JPEG string.
     * @returns {undefined}
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendScreenshot = function (cbSuccess, cbFailure, options) {
      if (options.hasOwnProperty('imagedata')) {
        CommunicationManager.submitScreenShot(cbSuccess, cbFailure, options);
      } else {
        var errObj = new ERRORS.WrongImageFormat();
        cbFailure(errObj);
      }
    };

    /**
     * Send the log to the CMS.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} options The log to send in XML format.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendLog = function (cbSuccess, cbFailure, options) {
      submitLog(cbSuccess, cbFailure, options);
    };

    /**
     * Send statistics to the CMS.
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @param {Object} options Statistics to send in XML format.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.sendStats = function (cbSuccess, cbFailure, options) {
      submitStats(cbSuccess, cbFailure, options);
    };

    /**
     * Acquire codes from authentication server
     * @param {Function} cbSuccess Callback on success if response was success
     * @param {Function} cbFailure Callback in case of a failure
     * @returns {undefined} None
     * @version 2.0.205
     * @since 2.0.205
     */
    this.acquireCodes = function (cbSuccess, cbFailure) {
      var url = c_authServerURL + "/generateCode";

      var procData = function (t) {
        var dt;
        try {
          dt = JSON.parse(t);

          v_DisplayAuthDetails.userCode = dt.user_code;
          v_DisplayAuthDetails.deviceCode = dt.device_code;
          cbSuccess(dt);
        } catch (e) {
          cbFailure(e);
        }
      };

      //utils.httpGetAsync(url,procData, cbFailure);

      var xmlHttp = new XMLHttpRequest();
      xmlHttp.onreadystatechange = function () {
        if (xmlHttp.readyState === 4) {
          if (xmlHttp.status === 200) {
            procData(xmlHttp.responseText);
          }
        }
      };
      xmlHttp.onerror = function (errObj, er) {
        if (cbFailure) {
          cbFailure(errObj);
        }
      };

      var sendData = {
        hardwareId: DeviceManager.Config.getHardwareKey(),
        type: DeviceManager.Device.getPlatformInfo().operatingSystem,
        version: DeviceManager.Device.getPlatformInfo().clientCode
      };

      xmlHttp.open("POST", url, true); // true for asynchronous 
      xmlHttp.setRequestHeader('Content-Type', 'application/json');
      xmlHttp.send(JSON.stringify(sendData));

    };

    /**
     * Start the Display details acquisition
     * @param {Function} cbSuccess Callback on success on CMS details acquisition
     * @param {Function} cbFailure Callback in case of a failure
     * @returns {undefined}
     */
    this.startAcquireDisplayAuthDetails = function (cbSuccess, cbFailure) {
      var url = c_authServerURL + "/getDetails?user_code=" + v_DisplayAuthDetails.userCode + "&device_code=" + v_DisplayAuthDetails.deviceCode;


      var procData = function (d) {
        //Cancel timer on success!
        try {
          var dt = JSON.parse(d);
          if (dt.cmsAddress) {
            int_clearInterval();
            cbSuccess(dt);
          }
        } catch (e) {

        }
      };

      var int_clearInterval = function () {
        clearInterval(v_DisplayAuthDetails.connectTimer);
        v_DisplayAuthDetails.connectTimer = null;
      };

      var onFailure = function (e) {
        int_clearInterval();
        cbFailure(e);
      };

      var makeRequests = function () {
        utils.httpGetAsync(url, procData, onFailure);
      };

      if (!v_DisplayAuthDetails.connectTimer) {
        v_DisplayAuthDetails.connectTimer = setInterval(makeRequests, 10000);
      }

    };

    /**
     * Stop the Display details acquisition
     * @returns {Boolean} True if Display details acquisition was active.
     */
    this.stopAcquireDisplayAuthDetails = function () {
      if (v_DisplayAuthDetails.connectTimer) {
        clearInterval(v_DisplayAuthDetails.connectTimer);
        v_DisplayAuthDetails = {
          userCode: null,
          deviceCode: null,
          connectTimer: undefined
        };
        return true;
      }
      return false;
    };

    /**
     * Send details about any issues encountered back to the CMS.
     * @param {function} cbSuccess Function to call on success.
     * @param {function} cbFailure Function to call on failure.
     * @param {Object} fault JSON encoded key/value string of properties with fault details. See below.
     * @returns undefined
     * @version 3.0.300
     * @since 3.0.300
     */
    this.reportFaults = function (cbSuccess, cbFailure, fault) {
      int_reportFaults(cbSuccess, cbFailure, fault);
    };

    // </editor-fold>

    /* +------------------------+ Native CMS Calls  +------------------------+ */
    // <editor-fold defaultstate="collapsed" desc="Native CMS Calls">
    var wsURL = "xmds.php"; /* Remember to add ?v=4 or ?v=5 */

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    var getCMSVersion = function (cbSuccess, cbFailure) {
      var url = DeviceManager.Config.getURL() + "xmds.php?what";
      utils.httpGetAsync(url, function (response) {
        if (DeviceManager.Config.setCMSVersion(response)) {
          cbSuccess();
        } else {
          cbFailure();
        }
      }, cbFailure);
    };

    /**
     * Call the registerDisplay function in XMDS service.
     * @param {Function} success Function to call on success.
     * @param {Function} failure Function to call on failure.
     * @param {Object} options Object with options
     * <br>options.URL The URL of the CMS to connect. If not set, confιguration value will be used instead.
     * <br>options.serverKey The Key of the CMS. If not set, confιguration value will be used instead.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    var registerDisplay = function (success, failure, options) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//ActivationMessage', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;

        if (node === null) { /* Error if this is true*/

          if (failure !== null) {
            var errObj = getFaultResponse(rt);
            if (errObj.errorText === "The Server key you entered does not match with the server key at this address") {
              errObj = ERRORS.WrongServerKey(errObj.errorText);
            }
            failure(errObj);
          }
        } else {

          var nodetext = node.textContent;
          success(nodetext);
//					var parser = new DOMParser();
//					var xmlDoc = parser.parseFromString(nodetext, "text/xml");

//					var displayElement = xmlDoc.childNodes[0];
//					var display = {name: 'display'};
//					for (var i = 0; i < displayElement.attributes.length; i++) {
//					display[displayElement.attributes[i].name] = displayElement.attributes[i].value;
//					}
//					var options = {};
//					for (var i = 0; i < displayElement.childNodes.length; i++) {
//					options[displayElement.childNodes[i].nodeName] = displayElement.childNodes[i].textContent;
//					}
//					display['options'] = options;
//					success(display);
        }
      };

      var serverKey;
      if (options && options.serverKey && options.serverKey !== "") {
        serverKey = options.serverKey;
      } else {
        serverKey = DeviceManager.Config.getPassword();
      }

      var pl = new SOAPClientParameters();
      pl.add("serverKey", serverKey);
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("displayName", DeviceManager.Config.getDisplayName());

      //DONT CALL FUNCTION 4 TIMES
      var v_platformInfo = DeviceManager.Device.getPlatformInfo();

      pl.add("clientType", v_platformInfo.clientType);
      pl.add("clientVersion", v_platformInfo.clientVersion);
      pl.add("clientCode", v_platformInfo.clientCode);
      pl.add("operatingSystem", v_platformInfo.operatingSystem);
      //DONT CALL FUNCTION 4 TIMES

      pl.add("macAddress", DeviceManager.Config.getMacAddress());

      /* Add the new functions of version 5 of CMS */
      if (DeviceManager.Config.getCMSVersion() >= 5) {
        pl.add("xmrChannel", DeviceManager.Config.getXmrChannel());
        pl.add("xmrPubKey", DeviceManager.Config.getXmrPubKey());
      }

      //var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      var lurl;
      if (options && options.URL && options.URL !== "") {
        lurl = options.URL;
      } else {
        lurl = DeviceManager.Config.getURL();
      }

      if (!lurl.endsWith('/')) {
        lurl += '/';
      }

      var furl = lurl + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "registerDisplay", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    /**
     * @param {Function} success Function to call on success.
     * @param {Function} failure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    var getRequiredFiles = function (success, failure) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//RequiredFilesXml', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;
        var nodetext = node.textContent;

        var parser = new DOMParser();
        var xmlDoc = parser.parseFromString(nodetext, "text/xml");

        var reqFilesxml = xmlDoc.childNodes[0];
        var reqFiles = [];

        for (var i = 0; i < reqFilesxml.childElementCount; i++) {
          var x = reqFilesxml.children[i];
          var reqFile = {};
          for (var j = 0; j < x.attributes.length; j++) {
            reqFile[x.attributes[j].name] = x.attributes[j].value;
          }
          if (reqFile.saveAs !== "") {
            reqFiles.push(reqFile);
          } else {
            //Logger.cmsLog({category:"error", message: "Invalid file with id "+reqFile.id, type:"ERROR", method:"getRequiredFiles", thread:"CommunicationManager"});
            Logger.cmsFault(undefined, undefined,
              {
                code: ERRORS.FaultCodes.RemoteResourceFailed,
                reason: "Invalid file with id " + reqFile.id + "",
                method: "getRequiredFiles",
                thread: "CommunicationManager",
                date: new Date(),
                expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
                mediaId: reqFile.id
              });
          }
        }

        success(reqFiles);
      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "requiredFiles", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    /**
     * @param {Function} success Function to call on success.
     * @param {Function} failure Function to call on failure.
     * @param {Object} options Options about the file chunk to download.
     * @version 1.8.1
     * @since 1.8.1
     */
    var getFile = function (success, failure, options) {
      var reqCB = function (r, rt) {
        //console.timeEnd('D');
        /* Call the DeviceManager.Filesystem.appendFile to save the data chunk */
        var node = rt.evaluate('.//file', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;

        if (node === null) { /* Error if this is true*/

          if (failure !== null) {
            var errObj = getFaultResponse(rt);
            failure(errObj);
          }
        } else {
          var nodetext = node.textContent;

          var ar = base64js.toByteArray(nodetext);
          success(ar);
        }
      };

      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("fileId", options.fileId);
      pl.add("fileType", options.fileType);
      pl.add("chunkOffset", options.chunkOffset);
      pl.add("chuckSize", options.chunkSize);

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        //console.time('D');
        SOAPClient.invoke(furl, "getFile", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    /**
     * Send the media inventory to CMS.
     * @param {function} success Function to call when sending the media inventory completes.
     * @param {function} failure Function to call in case of an error.
     * @param {Object} options Object that provides the XML of the media inventory.
     * @sa {String} options.MediaInventory String with the XML of the media inventory.
     * @returns {undefined}
     * @version 1.8.1
     * @since 1.8.1
     */
    var mediaInventory = function (success, failure, options) {
      inSendMediaInventory = false;
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;
        var nodetext;
        try {
          nodetext = node.textContent;
        } catch (e) {
          failure({errorCode: r.message, errorText: e.message});
        }


        inSendMediaInventory = false;
        if (success !== null) {
          success(nodetext);
        }
      };

      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("mediaInventory", options.MediaInventory);


      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();

      try {
        SOAPClient.invoke(furl, "mediaInventory", pl, true, reqCB, failure);
      } catch (e) {
        failure({errorCode: 0, errorText: e.message});
      }
    };

    /**
     * Get a resource from the CMS.
     * @param {Function} success Function to call on success.
     * @param {Function} failure Function to call on failure.
     * @param {Object} options Object that provides relative information about the resource. See below.
     * @see options.layoutId {Number}
     * @see options.regionId {String}
     * @see options.mediaId {String}
     * @returns {undefined}
     * @version 1.8.1
     * @since 1.8.1
     */
    var getResource = function (success, failure, options) {
      var reqCB = function (r, rt) {

        /* Call the DeviceManager.Filesystem.appendFile to save the data chunk */
        var node = rt.evaluate('.//resource', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;

        if (node === null) {
          var errObj = getFaultResponse(rt);
          failure(errObj);
        } else {
          var nodetext = node.textContent;
          success(nodetext);
        }
      };

      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("layoutId", options.layoutId);
      pl.add("regionId", options.regionId);
      pl.add("mediaId", options.mediaId);

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "getResource", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    /**
     * Send status to the CMS.
     * @param {function} success
     * @param {function} failure
     * @param {Object} options Object that provides relative information about monitor status. See below.<br>
     * <table>
     * <tr><th>Field</th><th>Type</th><th>Description</th></tr>
     * <tr><td>options.status</td><td>Object</td><td>
     * Object with properties "currentLayoutId", "availableSpace", "totalSpace" and values as Strings.<br>
     * Other properties can be included.
     * </td></tr>
     * </table>
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    var notifyStatus = function (success, failure, options) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;


        if (node === null) { /* Error if this is true*/

          if (failure !== null) {
            var errObj = getFaultResponse(rt);
            if (errObj.errorText === "This display client is not licensed") {
              errObj = ERRORS.ClientIsNotLicensed(errObj.errorText);
            }
            failure(errObj);
          }
        } else {
          var nodetext = node.textContent;
          if (success !== null) {
            success(nodetext);
          }
        }

      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("status", JSON.stringify(options.status));

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "notifyStatus", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    /**
     * Submit log to CMS
     * @param {Function} successCb Function to call on success.
     * @param {Function} failureCb Function to call on failure.
     * @param {String} log The log in XML format.
     * @version 1.8.1
     * @since 1.8.1
     */
    var submitLog = function (successCb, failureCb, log) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;


        if (node === null) { /* Error if this is true*/

          if (failureCb !== null) {
            var errObj = getFaultResponse(rt);
//						if (errObj.errorText === "This display client is not licensed"){
//						var errObj = ERRORS.ClientIsNotLicensed(errObj.errorText);
//						}
            failureCb(errObj);
          }
        } else {
          var nodetext = node.textContent;
          if (successCb !== null) {
            successCb(nodetext);
          }
        }

      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("logXml", log);

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "submitLog", pl, true, reqCB, failureCb);
      } catch (e) {
        failureCb(e);
      }
    };

    /**
     * Submit statistics to CMS
     * @param {Function} successCb Function to call on success.
     * @param {Function} failureCb Function to call on failure.
     * @param {String} stats The statistics in XML format.
     * @version 1.8.1
     * @since 1.8.1
     */
    var submitStats = function (successCb, failureCb, stats) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;


        if (node === null) { /* Error if this is true*/

          if (failureCb !== null) {
            var errObj = getFaultResponse(rt);
//						if (errObj.errorText === "This display client is not licensed"){
//						var errObj = ERRORS.ClientIsNotLicensed(errObj.errorText);
//						}
            failureCb(errObj);
          }
        } else {
          var nodetext = node.textContent;
          if (successCb !== null) {
            successCb(nodetext);
          }
        }

      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("statXml", stats);

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "submitStats", pl, true, reqCB, failureCb);
      } catch (e) {
        failureCb(e);
      }
    };

    /**
     * @param {Function} cbSuccess Function to call on success.
     * @param {Function} cbFailure Function to call on failure.
     * @version 1.8.1
     * @since 1.8.1
     */
    var getSchedule = function (cbSuccess, cbFailure) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//ScheduleXml', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;
        var nodetext = node.textContent;

        if (cbSuccess !== null) {
          cbSuccess(nodetext);
        }
      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();

      try {
        SOAPClient.invoke(furl, "schedule", pl, true, reqCB, cbFailure);
      } catch (e) {
        cbFailure(e);
      }
    };

    /**
     * Submit a screen shot of the display.
     * @param {function} success Function to call on success.
     * @param {function} failure Function to call on failure.
     * @param {Object} options Object that provides relative information about the status. See below.
     * @see options.imagedata {Object} The screenshot as a Base64-encoded JPEG string.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.submitScreenShot = function (success, failure, options) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;


        if (node === null) { /* Error if this is true*/

          if (failure !== null) {
            var errObj = getFaultResponse(rt);
            if (errObj.errorText === "This display client is not licensed") {
              errObj = ERRORS.ClientIsNotLicensed(errObj.errorText);
            }
            failure(errObj);
          }
        } else {
          var nodetext = node.textContent;
          if (success !== null) {
            success(nodetext);
          }
        }

      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("screenShot", options.imagedata);

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=" + DeviceManager.Config.getCMSVersion();
      try {
        SOAPClient.invoke(furl, "submitScreenShot", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };


    /**
     * Send details about any issues encountered back to the CMS.
     * @param {function} success Function to call on success.
     * @param {function} failure Function to call on failure.
     * @param {Array} faults Array of faults.
     * @returns undefined
     * @version 3.0.300
     * @since 3.0.300
     */
    var int_reportFaults = function (success, failure, faults) {
      var reqCB = function (r, rt) {
        var node = rt.evaluate('.//success', rt, null,
          XPathResult.FIRST_ORDERED_NODE_TYPE,
          null).singleNodeValue;


        if (node === null) { /* Error if this is true*/

          if (failure !== null) {
            var errObj = getFaultResponse(rt);
            failure(errObj);
          }
        } else {
          var nodetext = node.textContent;
          if (success !== null) {
            success(nodetext);
          }
        }
      };
      var pl = new SOAPClientParameters();
      pl.add("serverKey", DeviceManager.Config.getPassword());
      pl.add("hardwareKey", DeviceManager.Config.getHardwareKey());
      pl.add("fault", JSON.stringify(faults));

      var furl = DeviceManager.Config.getURL() + wsURL + "?v=6";

      try {
        SOAPClient.invoke(furl, "reportFaults", pl, true, reqCB, failure);
      } catch (e) {
        failure(e);
      }
    };

    // </editor-fold>
  }();
}());