/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */

/* global utils, DeviceManager, X2J, Logger, application, LibraryManager, ScheduleManager, CacheManager, ERRORS, EventTarget */
/*
* Layout Manager
* 
* V3.0.300
*/

(function () {
  "use strict";
  // <editor-fold defaultstate="collapsed" desc="Class Region">
  {
    var m_nativetohtml = ["embedded", "ticker", "text", "webpage", "datasetview"];
    /**
     * Create a Region
     * @param {type} id
     * @param {type} targetElement
     * @param {type} x
     * @param {type} y
     * @param {type} width
     * @param {type} height
     * @param {type} zIndex
     * @param {type} isOverlay
     * @param {type} layoutId
     * @param {type} scheduleId
     * @param {type} loop
     * @param {Array} actions
     * @returns {LayoutManager.Region}
     */
    var Region = function (id, targetElement, x, y, width, height, zIndex, isOverlay, layoutId, scheduleId, loop, actions) {
      EventEmitter.call(this);
      // Region should instruct media to stopMediaTimeout - Done
      // Region should notify when media start playing
      // Region should notify when media stop playing
      // Region should notify when all media finish playing
      // Region should start - Done
      // Region should stop - Done
      // Region should pause
      // Region should resume
      // Region should have a loop property

      try {

        var v_instance = this;

        var v_domRegion = document.createElement("div");
        var v_id = id;
        var v_expired = false;
        var v_mediaSequence = [];
        var v_currentMediaIndex = 0;
        var v_currentWidget = null;
        var v_loop = loop;
        var v_isOverlay = isOverlay;
        var v_overlayId = layoutId;
        var v_layoutId = layoutId;
        var v_scheduleId = scheduleId; //XXX: This is not right. Find something clever to do.
        var v_actions = actions;
        var v_isStopping = false;
        if (v_isOverlay === true) {
          v_domRegion.id = "overlay_" + v_overlayId + "_" + id;
        } else {
          v_domRegion.id = id;
        }
        v_domRegion.style.position = "absolute";
        v_domRegion.style.width = width;
        v_domRegion.style.height = height;
        v_domRegion.style.top = y;
        v_domRegion.style.left = x;
        v_domRegion.style.display = 'flex';
        v_domRegion.style.justifyContent = 'center';
        v_domRegion.style.alignItems = 'center';
        if (zIndex) {
          v_domRegion.style.zIndex = zIndex;
        }

        utils.const(this, 'id', id);


        //this.mediaSequence = v_mediaSequence;
        this.currentMediaIndex = v_currentMediaIndex;

        this.addMediaInSequence = function (media) {
          if (media.cyclePlayback == "1") {
            var candidate = CacheManager.getNextWidgetFromCycle(media);
            media = candidate;
          }
          v_mediaSequence.push(media);
        };

        this.hideHWTextures = function () {
          if (v_currentWidget.hideMedia) {
            v_currentWidget.hideMedia();
          }
        };

        this.showHWTextures = function () {
          if (v_currentWidget.showMedia) {
            v_currentWidget.showMedia();
          }
        };

        var mediacallback = function () {
          if (v_currentWidget) {
            v_currentWidget.removeEventListener('widgetEnded', mediacallback);
          }
          v_currentMediaIndex += 1;
          displayMedia();
        };

        var getWidget = function (parsedMedia) {
          var scp;
          if (v_overlayId) {
            scp = "overlay";
          } else {
            scp = "main";
          }
          if (parsedMedia.render === "native") {
            if (m_nativetohtml.indexOf(parsedMedia.type) > -1) {
              return LayoutManager.factory("html", parsedMedia, v_domRegion, mediacallback, v_layoutId, v_scheduleId, scp);
            } else {
              return LayoutManager.factory(parsedMedia.type, parsedMedia, v_domRegion, mediacallback, v_layoutId, v_scheduleId, scp);
            }

          } else if (parsedMedia.render === "html") {
            return LayoutManager.factory("html", parsedMedia, v_domRegion, mediacallback, v_layoutId, v_scheduleId, scp);
          } else {
            console.warn('Media with render ' + parsedMedia.render + ' and type ' +
              parsedMedia.type + " could not play");
          }
        };

        /* Start playing media */
        var displayMedia = function () {

          //TODO: This should be moved outside region
          //      if (v_currentMediaIndex === 0) {
          //        layoutStartedTime = utils.getISODate();
          //      }
          //XXX: Sanity check?
          if (v_domRegion === null) {
            return;
          }

          var shouldnotloop = false;
          var shouldnotloopByContainer = false;
          if (v_domRegion) {
            shouldnotloopByContainer = (v_domRegion.childElementCount !== 0);
          }
          if ((v_loop === 0) && (v_mediaSequence.length === 1) && shouldnotloopByContainer) {
            shouldnotloop = true;
          }
          if (v_currentMediaIndex >= v_mediaSequence.length) {
            v_currentMediaIndex = 0;
            v_expired = true;
            var regionExpired = new CustomEvent('regionExpired', {
              detail: {
                isOverlay: v_isOverlay,
                layoutId: v_layoutId
              }
            });
            v_instance.dispatchEvent(regionExpired);
          } else if (v_currentMediaIndex < 0) {
            v_currentMediaIndex = v_mediaSequence.length - 1;
            v_expired = true;
            var regionExpired = new CustomEvent('regionExpired', {
              detail: {
                isOverlay: v_isOverlay,
                layoutId: v_layoutId
              }
            });
            v_instance.dispatchEvent(regionExpired);
          }

          if ((!v_isStopping) && (!shouldnotloop)) {
            v_currentWidget = getWidget(v_mediaSequence[v_currentMediaIndex]);
            if (v_currentWidget === null) {
              if (v_mediaSequence.length > 1) {
                mediacallback();
              } else {
                console.error('Region ' + v_id + ' is invalid!!!');
              }
              v_expired = true;
            } else {
              v_currentWidget.addEventListener('widgetEnded', mediacallback);
            }
          }
        };

        this.isExpired = function () {
          return v_expired;
        };
        /**
         * Start playing media sequence
         */
        this.start = function () {
          displayMedia();
        };

        this.stop = function () {
          v_isStopping = true;
          if (v_currentWidget) {
            v_currentWidget.stopMediaTimeout();
            v_currentWidget.removeEventListener('widgetEnded', mediacallback);
          }
          v_currentMediaIndex = 0;

          //TODO: Do we need to clear the region or it will be cleared by the next widget.
        };

        this.pause = function () {
          v_currentWidget.pause();
        };

        this.resume = function () {
          v_currentWidget.resume();
        };

        this.playImmediateWidget = function (widget) {
          //        v_stackedWidgets.push(widget);

          v_currentWidget.pause();
          v_currentWidget = getWidget(widget);
          v_currentWidget.addEventListener('widgetEnded', function () {
            console.info('Immediate widget expired');
            //          v_currentWidget = prevWidget;
            //          v_currentWidget.resume();
            v_currentMediaIndex = 0;
            var pLoop = v_loop;
            v_loop = 1;
            displayMedia();
            v_loop = pLoop;
          });
        };

        this.playNextWidget = function () {
          mediacallback();
        };

        this.playPreviousWidget = function () {
          v_currentWidget.removeEventListener('widgetEnded', mediacallback);
          v_currentMediaIndex -= 1;
          displayMedia();
        };

        this.expireWidget = function () {
          v_currentWidget.expireMedia();
        };
        this.extendWidgetDuration = function (duration) {
          v_currentWidget.extendDuration(duration);
        };
        this.setWidgetDuration = function (duration) {
          v_currentWidget.setDuration(duration);
        };

        targetElement.appendChild(v_domRegion);

        if (v_actions) {
          for (var i = 0, max = v_actions.length; i < max; i++) {
            var v_domActionOverlay = undefined;
            if (v_actions[i].jAttr.triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
              v_domActionOverlay = document.createElement("div");
              v_domActionOverlay.style.position = "absolute";
              v_domActionOverlay.style.width = width;
              v_domActionOverlay.style.height = height;
              v_domActionOverlay.style.top = y;
              v_domActionOverlay.style.left = x;
              targetElement.appendChild(v_domActionOverlay);
            }
            InteractiveManager.addEventAction(v_domActionOverlay,
              v_actions[i].jAttr.id,
              v_actions[i].jAttr.triggerType, v_actions[i].jAttr.triggerCode,
              v_actions[i].jAttr.actionType, v_actions[i].jAttr.layoutCode,
              v_actions[i].jAttr.source, v_actions[i].jAttr.sourceId,
              v_actions[i].jAttr.target, v_actions[i].jAttr.targetId,
              v_actions[i].jAttr.widgetId, v_layoutId);
          }
        }
      } catch (e) {
        throw e;
      }

    };
  }
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Class LayoutManager">
  /**
   * @classdesc This module is responsible to handle layouts.
   * @extends EventTarget
   * @type LayoutManager
   * @event layoutStart
   * @event layoutFinish
   * @version 3.0.300
   * @since 1.8.0
   */
  window.LayoutManager = new function () {

    // Attach to event emitter so you could dispach events.
    EventEmitter.call(this);
    //    utils.const(LayoutManager, "SYSTEMLAYOUTS", ["loading", "splash"]);
    this.SYSTEMLAYOUTS = ["loading", "splash"];


    /**
     * The name of the html element that hosts the layouts
     * @param {type} type
     * @param {type} idx
     * @param {type} e
     * @param {type} mediacallback
     * @param {type} layoutid
     * @param {type} scheduleid
     * @param {type} scp
     * @returns {Object}
     */
    this.factory = function (type, idx, e, mediacallback, layoutid, scheduleid, scp) {
      if (this.hasOwnProperty(type)) {
        return new this[type](idx, e, mediacallback, layoutid, scheduleid, scp);
      } else {
        console.warn("Unsupported type : " + type);
        return null;
      }
    };

    /** The HTML element to display content */
    var targetElementName = null;

    /** The last Layout that has been displayed */
    var lastShowPage = null;

    /** The array with layouts in queue */
    var v_RunningLayouts = [];

    /** */
    var v_displayOverLayLayout = new Array();

    /** The currently displayed layout index */
    var v_CurrentLayoutIndex = 0;


    /** The next displayed layout index */
    var v_nextLayoutIdx;

    /** The next Layout ID */
    var v_NextLayoutId;
    /**
     * The id of the current layout.<br>
     * @type String
     */
    var v_CurrentLayoutId = "loading";

    /**
     * The id of the schedule.
     */
    var v_ScheduleId = "0";

    /**
     * @type Array
     */
    var v_parsedLayouts = [];

    /**
     * The regions of the current layout.
     * @type object
     */
    var v_CurrentLayoutRegions = {};

    /** The num of active overlay layouts. */
    var v_activeOverlay = 0;

    /** Flag if schedule is in pause state in order to so non scheduled screens like settings */
    var v_ScheduleIsPaused = false;

    /** Should the layout start playing */
    var shouldStartPlaying = false;

    /* */
    var layoutStartedTime = null;

    /**
     * Check if manager is started!
     */
    var v_isStarted = false;

    /**
     * Should the external input be displayed
     */
    this.shouldDisplayExternalInput = true;

    /**
     * Set if layout is shown as a result of interaction.
     * @type boolean
     */
    var v_isInteractiveLayout = false;

    /**
     * @type Array
     */
    var v_PreviousRunningLayouts = [];
    /**
     * User has opened the settings screen.
     * @type {boolean}
     */
    var v_userShownedSettingsScreen = false;
    // </editor-fold>
    /**
     * Array with Layout ids that are invalid.
     * Possible reasons are:
     * Region is empty.
     */
    this.invalidLayoutIds = [];
    // </editor-fold>
    /* +------------------------+ Private functions +------------------------+ */

    //    /* Start playing media */
    //    var displayMedia = function (mediaSequence, nextMediaIndex, regionIndex, layout, overlayIndex) {
    //      //nextMediaIndex++;
    //      //if (defined(LayoutElements[layout.id][mediaSequence.regionID])) { LayoutElements[layout.id][mediaSequence.regionID].stopMediaTimeout(); }
    //      var layoutRegionsIndex;
    //      if (layout.isoverlay) {
    //        //layoutRegionsIndex = "overlay" + layout.id;
    //        layoutRegionsIndex = "overlay" + overlayIndex;
    //      } else {
    //        layoutRegionsIndex = layout.id;
    //      }
    //      if (nextMediaIndex === 0) {
    //        layoutStartedTime = utils.getISODate();
    //      }
    //      var e = document.getElementById(mediaSequence.regionID);
    //      if (e === null) {
    //        return;
    //      }
    //
    //      var shouldnotloop = false;
    //      var shouldnotloopByContainer = false;
    //      if (e) {
    //        shouldnotloopByContainer = (e.childElementCount !== 0);
    //      }
    //      if ((layout.regions[regionIndex].loop === 0) && (mediaSequence.media.length === 1) && shouldnotloopByContainer) {
    //        shouldnotloop = true;
    //      }
    //      if (nextMediaIndex >= mediaSequence.media.length) {
    //        nextMediaIndex = 0;
    //        mediaSequence.sequenceExpired = true;
    //        v_CurrentLayoutRegions[layoutRegionsIndex][regionIndex].regionExpired = true;
    //        /* call Region Expired */
    //        if (allRegionsExpired(layoutRegionsIndex))
    //        {
    //          if (layout.id !== 'splash') {
    //            var ltodt = utils.getISODate();
    //            var dur = (((new Date()) - new Date(layoutStartedTime)) / 1000) | 0;
    //            CacheManager.setLayoutDuration(layout.id, dur);
    //            if (layout.enableStat !== "0") {
    //              Logger.logStat({fromdt: layoutStartedTime, todt: ltodt, scheduleid: layout.scheduleid, layoutid: layout.id});
    //            }
    //          }
    //          console.debug("All Regions Expired");
    //          if (layout.isoverlay) {
    //            displayLayouts(layout.isoverlay, overlayIndex);
    //          } else {
    //            displayLayouts();
    //          }
    //          return;
    //        }
    //      }
    //      //console.debug("Element "+ mediaSequence.media[nextMediaIndex].type +" should not loop", [(layout.regions[regionIndex].loop === 0),(mediaSequence.media.length === 1), (e.childElementCount !== 0)], (layout.regions[regionIndex].loop === 0) && (mediaSequence.media.length === 1) && (e.childElementCount !== 0));
    //
    //      var mediacallback = function () {
    //        displayMedia(mediaSequence, nextMediaIndex + 1, regionIndex, layout, overlayIndex);
    //      };
    //      var nativetohtml = ["embedded", "ticker", "text", "webpage", "datasetview"];
    //      var scp;
    //      if (defined(overlayIndex)) {
    //        scp = "overlay";
    //      } else {
    //        scp = "main";
    //      }
    //      //        var skipthis = false;
    //      //        if (mediaSequence.media[nextMediaIndex].hasOwnProperty('shouldnotloop')) {
    //      //            if (mediaSequence.media[nextMediaIndex].hasOwnProperty('shouldnotloop') === true) {
    //      //                skipthis = true;
    //      //            }
    //      //        }
    //      if (!(shouldnotloop)) {
    //        if (mediaSequence.media[nextMediaIndex].render === "native") {
    //          if (nativetohtml.indexOf(mediaSequence.media[nextMediaIndex].type) > -1) {
    //            var mediaElement = LayoutManager.factory("html", mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
    //          } else {
    //            var mediaElement = LayoutManager.factory(mediaSequence.media[nextMediaIndex].type, mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
    //          }
    //          if (mediaElement === null) {
    //            mediacallback();
    //          }
    //        } else if (mediaSequence.media[nextMediaIndex].render === "html") {
    //          var mediaElement = LayoutManager.factory("html", mediaSequence.media[nextMediaIndex], e, mediacallback, layout.id, layout.scheduleid, scp);
    //          if (mediaElement === null) {
    //            mediacallback();
    //          }
    //        } else {
    //          mediacallback();
    //        }
    //        if (layout.isoverlay) {
    //          LayoutElements["overlay" + overlayIndex][mediaSequence.regionID] = mediaElement;
    //        } else {
    //          LayoutElements[layout.id][mediaSequence.regionID] = mediaElement;
    //        }
    //      }
    //      //        if (shouldnotloop) {
    //      //            mediaSequence.media[nextMediaIndex].shouldnotloop = true;
    //      //        }
    //    };


    // <editor-fold defaultstate="collapsed" desc="Private functions">
    /* Media ended */
    /* Region expired */
    var allRegionsExpired = function (evt) {

      console.debug("Checking expired regions", evt.detail.layoutId);
      var layoutsExpired = true; /* Reverse logic */
      var layout = v_CurrentLayoutId;
      var layoutIDX = -1;
      var lookupArray = evt.detail.isOverlay ? v_displayOverLayLayout : v_RunningLayouts;

      for (var i = 0; i < lookupArray.length; i++) {
        if (lookupArray[i].id == evt.detail.layoutId) {
          layoutIDX = i;
          layout = (evt.detail.isOverlay ? "overlay_" : "") + evt.detail.layoutId;
        }
      }

      // if (evt.detail.isOverlay) {
      // 	for (var i = 0; i < v_displayOverLayLayout.length; i++) {
      // 		if (v_displayOverLayLayout[i].id == evt.detail.layoutId) {
      // 			layoutIDX = i;
      // 			layout = "overlay_" + evt.detail.layoutId
      // 		}
      // 	}
      // }else{
      // 	for (var i = 0; i < v_RunningLayouts.length; i++) {
      // 		if (v_RunningLayouts[i].id == evt.detail.layoutId) {
      // 			layoutIDX = i;
      // 			layout = evt.detail.layoutId
      // 		}
      // 	}
      // }
      for (var i = 0; i < v_CurrentLayoutRegions[layout].length; i++) {
        console.debug("Layout " + evt.detail.layoutId + ", Region[" + i + "] " + v_CurrentLayoutRegions[layout][i].id + " expired " + v_CurrentLayoutRegions[layout][i].isExpired());
        if (v_CurrentLayoutRegions[layout][i].isExpired() === false) {
          layoutsExpired = false;
          break;
        }
      }
      if (layoutsExpired) {
        console.debug('Layout expired', evt.detail.layoutId);
        if (layout !== 'splash') {
          var ltodt = utils.getISODate();
          var dur = (((new Date()) - new Date(layoutStartedTime)) / 1000) | 0;
          CacheManager.setLayoutDuration(layout, dur);
          CacheManager.mpph_addPlay(layout);
          if (!evt.detail.isOverlay) {
            if (lookupArray[layoutIDX].enableStat !== "0") {
              Logger.logStat({
                fromdt: layoutStartedTime,
                todt: ltodt,
                scheduleid: lookupArray[layoutIDX].scheduleid,
                layoutid: lookupArray[layoutIDX].id
              });
            }
          }
        }
        displayLayouts(evt.detail.isOverlay, layoutIDX);
      }
      return layoutsExpired;
    };

    var count = 0;
    var setCampaignOrder = function (layout) {
      var j;
      var layoutflag = 0;// 1 if layout exits !
      var campaignOrder = CacheManager.getCampaignOrder(layout.groupKey);
      if (!campaignOrder) {
        campaignOrder = {
          groupKey: layout.groupKey,
          count: layout.playCount,
          counter: layout.playCount,
          layoutIdx: -1,
          layouts: []
        };
      } else {
        campaignOrder.count = layout.playCount;
      }
      for (j = 0; j < campaignOrder.layouts.length; j++) {
        if (campaignOrder.layouts[j].layoutId === layout.file) {
          layoutflag = 1;
          break;
        }
      }

      if (!layoutflag) {
        campaignOrder.layouts.push({
          layoutId: layout.file,
          layout: layout
        });
        campaignOrder.counter = layout.playCount;
      }
      CacheManager.setCampaignOrder(layout.groupKey, campaignOrder);

      // console.log("my campaign list is ready")
    };
    var checkCampaignOrder = function (layouts) {
      var cpkeys; // campaigns ids
      var i, layout, campFound;
      var index, layoutfound;
      var l_index = [];
      var campaignOrder;
      var currentID;
      for (i = 0; i < layouts.length; i++) {
        if (layouts[i].cyclePlayback == "1") {
          layout = layouts[i];
          setCampaignOrder(layout); // add in the list only the layouts that matter
        }
      }
      cpkeys = CacheManager.getCampaignOrderKeys();
      for (i = 0; i < cpkeys.length; i++) { //for every campaign
        campFound = 0;
        index = i;
        for (var j = 0; j < layouts.length; j++) { //for every layout
          campaignOrder = CacheManager.getCampaignOrder(cpkeys[i]);
          if (campaignOrder.layoutIdx > -1) {
            currentID = campaignOrder.layouts[campaignOrder.layoutIdx].layoutId;
          }
          ;
          if (layouts[j].cyclePlayback == "1") { //if they belong in a campaign
            layoutfound = 0;
            if (cpkeys[i] == layouts[j].groupKey) {
              campFound = 1;
              for (var k = 0; k < campaignOrder.layouts.length; k++) { // By the way we should check if the layouts in the campaign matches with the layout we got from cms
                layoutfound = layouts.map(function (object) {
                  return object.file;
                }).indexOf(campaignOrder.layouts[k].layoutId);
                if (layoutfound == -1) {
                  l_index.push(k);
                }
              }
              break;
            }
          }
        }
        var flag = 0;
        if (!campFound) {//remove the campaigns that didnt match with the layouts we should play
          delete CacheManager.deleteCampaign(cpkeys[i]);
        } else { // remove the layouts from the campaign, that we dodnt need any more
          if (l_index.length > 0) {
            for (var j = (l_index.length - 1); j >= 0; j--) {
              if (campaignOrder.layoutIdx == l_index[j]) {
                campaignOrder.counter = 0;
                campaignOrder.layoutIdx = 0;
                flag = 1;
              }
              campaignOrder.layouts.splice(l_index[j], 1);
              // campaignOrder[cpkeys[i]].layoutIdx = -1
            }
          }
          if (!flag) {
            for (var k = 0; k < campaignOrder.length; k++) {
              if (currentID == campaignOrder.layouts[k].layoutid) {
                campaignOrder.layoutIdx = k;
                break;
              }
            }
          }
        }
        CacheManager.setCampaignOrder(cpkeys[i], campaignOrder);
      }
      //console.log("campaign is checked");

    };
    var checkLayout = function (layout) {
      var cpkeys;
      var flag, i, id;
      var campaignOrder;
      var currentGroupKey;
      id = layout.id;
      flag = -1;
      cpkeys = CacheManager.getCampaignOrderKeys();
      currentGroupKey = CacheManager.getCurrentGroupKey();
      if (cpkeys.length > 1) {// if there is more than one campaign search else keep playing from this camp
        for (i = 0; i < cpkeys.length; i++) {
          campaignOrder = CacheManager.getCampaignOrder(cpkeys[i]);
          var layouts = campaignOrder.layouts;
          for (var j = 0; j < layouts.length; j++) {
            if (layout.id == layouts[j].layoutId) {
              if (currentGroupKey == cpkeys[i]) {
                flag = 0;
                break;
              } else { // else keep the current campaign id
                flag = 1;
                currentGroupKey = cpkeys[i];
                CacheManager.setCurrentGroupKey(currentGroupKey);
              }
            }
          }
          if (flag === 0) {// if you found a match with tha campaign that played before
            break;
          }
        }
      } else if (cpkeys.length == 1) {
        campaignOrder = CacheManager.getCampaignOrder(cpkeys[0]);
        if (campaignOrder) {
          currentGroupKey = cpkeys[0];
          CacheManager.setCurrentGroupKey(currentGroupKey);
          var layouts = campaignOrder.layouts;
          if (layouts.length > 1) {
            if (layout.id == layouts[0].layoutId) {
              flag = 1;
            } else {
              for (var j = 1; j < layouts.length; j++) {
                if (layout.id == layouts[j].layoutId) {
                  flag = 0;
                  break;
                } else { // else keep the current campaign id
                  flag = -1;
                }
              }
              // currentGroupKey = 0;
            }
          } else {
            flag = -1;
          }
        }
      }
      return flag;
    };
    var campaignCounter = function () {
      var currentGroupKey = CacheManager.getCurrentGroupKey();
      var campaignParent = CacheManager.getCampaignOrder(currentGroupKey);
      if (campaignParent.count == campaignParent.counter) {
        campaignParent.counter = 0;
      } else {
        campaignParent.counter += 1;
      }
      return campaignParent.counter;
    };
    var getNextLayoutFromCamp = function () {
      var c, layoutID;
      var currentGroupKey = CacheManager.getCurrentGroupKey();
      var campaignOrder = CacheManager.getCampaignOrder(currentGroupKey);
      var current_layoutIdx = campaignOrder.layoutIdx;
      var allLayouts = campaignOrder.layouts;

      c = campaignCounter();

      if (c === 0) {
        campaignCounter();
        current_layoutIdx += 1;
        if (current_layoutIdx >= allLayouts.length) {
          current_layoutIdx = 0;
        }
      }
      campaignOrder.layoutIdx = current_layoutIdx;
      layoutID = allLayouts[current_layoutIdx].layoutId;
      return layoutID;
    };
    var check = function (layouts, nIdx) {
      if (layouts.length > 0) {// check the layouts so you will find what you will play next with the self called function
        if (layouts[nIdx].id == v_NextLayoutId) {
          if ((nIdx) < v_nextLayoutIdx) {
            v_nextLayoutIdx = nIdx;
          }
          nIdx++;
          if (nIdx == layouts.length) {
            nIdx = 0;
          }
        }

        var l_idx = checkLayout(layouts[nIdx]);
        var idx = 0;//while we assign this value we deal with the racing condition between updating the campaign list and the layout list
        if (l_idx === 1) { //that means we found the next campaign
          var id = getNextLayoutFromCamp(layouts[nIdx]); //get the next layout from that campaign
          for (var i = 0; i < layouts.length; i++) {
            if (layouts[i].id == id) {
              idx = i; // we match it with the list of layouts. We have to get its position in the array
              break;
            }
          }
          nIdx = idx;
        } else if (l_idx === 0) { // keep looking for the right campaign
          nIdx++;
          if (nIdx + 1 > layouts.length) {
            nIdx = 0;
          }
          count++;
          if (count > layouts.length) { // if the search is hopeless
            var id = getNextLayoutFromCamp(layouts[nIdx]); //use the current index to find the next layout until the next update from the cms come
            for (var i = 0; i < layouts.length; i++) {
              if (layouts[i].id == id) {
                idx = i;
                break;
              }
            }
            count = 0;
            nIdx = idx;
          } else {
            nIdx = check(layouts, nIdx);//check again
          }
        } else {// layout is alone!
          if (layouts.length == 1) {
            nIdx = 0;
          }
        }
        count = 0;
        v_NextLayoutId = layouts[nIdx].id;
      } else {
        var defLayout = {
          layoutId: ScheduleManager.getScheduleData().default.file,
          scheduleId: "0"
        };
        LayoutManager.setLayouts([defLayout]);
        v_NextLayoutId = defLayout.layoutId;
        nIdx = 0;
      }
      ;
      return nIdx;
    };

    function getLayoutIdxById(id) {
      var idx = -1;
      for (var i = 0; i < v_RunningLayouts.length; i++) {
        if ((v_RunningLayouts[i].id == id)) {
          idx = i;
        }
      }//else get the layout you should get next
      return idx;
    }

    var displayLayouts = function (isoverlay, overlayIndex) {

      if (isoverlay === true) {
        displayLayout(null, true, overlayIndex);
      } else {
        /* if we show something like settings screen, don't change layouts */
        if (v_ScheduleIsPaused) {
          return;
        }
        if (v_CurrentLayoutIndex < 0 || v_CurrentLayoutIndex > (v_RunningLayouts.length - 1)) {
          v_CurrentLayoutIndex = check(v_RunningLayouts, 0);//if you played slpash check(if it is in the camp) and play the next layout or the layouts is not updated yet do the index is out of the array
          console.debug('Displaylayouts: Current layout index after check:', v_CurrentLayoutIndex, "Layouts length", v_RunningLayouts);
        } else {
          v_CurrentLayoutIndex = getLayoutIdxById(v_NextLayoutId);

          console.debug('Displaylayouts: Current layout index after get next layout:', v_CurrentLayoutIndex, "Layouts length", v_RunningLayouts);
          if (v_CurrentLayoutIndex >= v_RunningLayouts.length) {
            v_CurrentLayoutIndex = 0;
            console.debug('Displaylayouts: But we have to reset to 0 because it is more than the length of the layouts');
          } else if (v_CurrentLayoutIndex < 0) {
            v_CurrentLayoutIndex = 0;
            if (v_nextLayoutIdx < v_RunningLayouts.length) {
              if (v_nextLayoutIdx != v_CurrentLayoutIndex) {
                v_CurrentLayoutIndex = v_nextLayoutIdx;
              }
            }

            console.debug('Displaylayouts: But we couldn\'t find the id so we reset to 0');
          }
        }

        lastShowPage = v_CurrentLayoutIndex;

        // here I can choose the next layout
        displayLayout(v_CurrentLayoutIndex);
        v_nextLayoutIdx = check(v_RunningLayouts, v_CurrentLayoutIndex);// set the next layout that it will play
        if (v_CurrentLayoutIndex < 0) {
          application.statusPage.currentLayout("");
        } else {
          application.statusPage.currentLayout(v_RunningLayouts[v_CurrentLayoutIndex].id);
        }
        application.statusPage.nextLayout(v_RunningLayouts[v_nextLayoutIdx].id);

      }

    };

    var displayLayout = function (layoutIndex, isoverlay, overlayIndex) {
      var v_Layout;
      var e;
      var overlaycontainer;
      var overlayinnercontainer;
      var overlayoutercontainer;
      var scaleroptions;
      var layoutRegionsIndex;

      if (isoverlay === true) {
        v_Layout = v_displayOverLayLayout[overlayIndex];
        var c_id = "#overlay-" + v_Layout.id + "-outer";
        overlaycontainer = document.getElementById('overlayContainer');
        overlayoutercontainer = overlayContainer.querySelector(c_id);
        if (!overlayoutercontainer) {
          overlayoutercontainer = document.createElement("div");
          overlayoutercontainer.setAttribute('id', "overlay-" + v_Layout.id + "-outer");
          overlayoutercontainer.setAttribute('style', "position: absolute;");
          overlaycontainer.appendChild(overlayoutercontainer);
        } else {
          overlayoutercontainer.innerHTML = "";
        }

        overlayinnercontainer = document.createElement("div");
        e = document.createElement("div");
        overlayinnercontainer.appendChild(e);
        overlayoutercontainer.appendChild(overlayinnercontainer);
        //utils.remove("#overlay-" + v_Layout.id + "-outer");
        v_Layout.isoverlay = isoverlay;
        e.setAttribute('id', "overlay-" + v_Layout.id);
        overlayinnercontainer.setAttribute('id', "overlay-" + v_Layout.id + "-innercontainer-" + overlayIndex);
        overlayinnercontainer.setAttribute('style', "left: 0; top: 0; position: absolute; width: " + document.body.getBoundingClientRect().width + "px; height: " + document.body.getBoundingClientRect().height + "px;");
        layoutRegionsIndex = "overlay_" + v_Layout.id;
      } else {
        e = document.getElementById(targetElementName);
        //Remove any actions the layout has
        if (v_RunningLayouts[layoutIndex] && v_RunningLayouts[layoutIndex].actions) {
          for (var i = 0, max = v_RunningLayouts[layoutIndex].actions.length; i < max; i++) {
            InteractiveManager.removeEventAction(v_RunningLayouts[layoutIndex].actions[i].jAttr.id);
          }
        } else {
          console.warn('Remove actions from previous layout.');
          // This might be invalid place to do this as an overlay layout might also has actions that will

          if (LayoutManager.SYSTEMLAYOUTS.indexOf(v_CurrentLayoutId) === -1) {
            InteractiveManager.removeLayoutEventActions(v_CurrentLayoutId);
          }
        }
        if (v_CurrentLayoutRegions[v_CurrentLayoutId]) {
          v_CurrentLayoutRegions[v_CurrentLayoutId].forEach(function (reg) {
            reg.stop();
          });
        }
        e.innerHTML = null;
        var prevIdx = layoutIndex - 1;
        if (prevIdx < 0) {
          prevIdx = v_RunningLayouts.length - 1;
        }
        //XXX: This might be a good place to call the regions stop.
        var layoutendevt = new CustomEvent('layoutEnd', {
          detail: {
            layoutId: v_CurrentLayoutId,
            isOverlay: false,
            isInteractive: v_isInteractiveLayout
          }
        });

        if (ScheduleManager.isMaxPlaysPerHourThresholdHit(v_CurrentLayoutId)) {
          var tmpObj = v_RunningLayouts.filter(function (l) {
            return l.id === v_CurrentLayoutId;
          })[0];
          if (tmpObj) {
            var layoutIndexToRemove = v_RunningLayouts.indexOf(tmpObj);
            v_RunningLayouts.splice(layoutIndexToRemove, 1);
            var s_layouts = ScheduleManager.getScheduleData().layouts;
            var c_layouts;
            c_layouts = s_layouts.filter(function (sl) {
              for (var i = 0; i < v_RunningLayouts.length; i++) {
                if (sl.file == v_RunningLayouts[i].id) {
                  return sl;
                }
              }
              return false;
            });

            checkCampaignOrder(c_layouts);
            var idx = getLayoutIdxById(v_NextLayoutId);
            //v_CurrentLayoutIndex = v_CurrentLayoutIndex - 1;
            if (idx === -1 || v_RunningLayouts.length === 0) {
              displayLayouts();
              return;
            } else {
              v_CurrentLayoutIndex = idx;
              layoutIndex = v_CurrentLayoutIndex;
            }
          }
        }
        LayoutManager.dispatchEvent(layoutendevt);
        //v_CurrentLayoutId = (v_RunningLayouts[prevIdx].length>0)?v_RunningLayouts[prevIdx].id:"";
        v_isInteractiveLayout = false; // Unless there is a better way to detect if the layout that finish playing is interactive, use this.

        //            while (e.firstChild) {
        //                e.removeChild(e.firstChild);
        //            }
        if (v_RunningLayouts.length === 0) {
          v_RunningLayouts.push(LayoutParser.getDefault());
        }
        v_Layout = v_RunningLayouts[layoutIndex];
        v_CurrentLayoutId = v_RunningLayouts[layoutIndex].id;
        clearLayoutTimeouts("main", v_Layout.id);
        layoutRegionsIndex = v_Layout.id;
      }
      //        e.style.width = v_Layout.width+'px';
      //        e.style.height = v_Layout.height+'px';
      if (!defined(isoverlay)) {
        e.style.backgroundColor = v_Layout.bgcolor;
      }
      if (v_Layout.background !== "") {
        setTimeout(function () {
          utils.setBackground(e, v_Layout.background);
        }, 0);
      } else {
        //e.style.backgroundImage = "none";
        utils.removeBackground(e);
      }

      scaleroptions = {
        "originalWidth": v_Layout.width,
        "originalHeight": v_Layout.height,
        "previewWidth": document.body.getBoundingClientRect().width,
        "previewHeight": document.body.getBoundingClientRect().height,
        "scaleOverride": 0,
        "widgetDesignWidth": v_Layout.width,
        "widgetDesignHeight": v_Layout.height,
        "widgetDesignPadding": 0,
        "itemsPerPage": 0
      };

      if (isoverlay === true) {
        //jQueryWebOs("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay).xiboLayoutScaler(scaleroptions);
        utils.scaler("#overlay-" + v_Layout.id + "-innercontainer-" + overlayIndex, scaleroptions);
        var el = utils.elementSelector("#overlay-" + v_Layout.id + "-innercontainer-" + overlayIndex + " > div");
        el.style.margin = "auto";
        el.style.top = "auto";
        el.style.left = "auto";
        el.style.bottom = "auto";
        el.style.right = "auto";
      } else {
        utils.scaler("#viewContainer", scaleroptions);
      }

      v_CurrentLayoutRegions[layoutRegionsIndex] = [];

      for (var i = 0; i < v_Layout.regions.length; i++) {
        var region = new Region(v_Layout.regions[i].id, e,
          v_Layout.regions[i].left, v_Layout.regions[i].top, v_Layout.regions[i].width,
          v_Layout.regions[i].height, v_Layout.regions[i].zindex, isoverlay,
          v_Layout.id, v_Layout.scheduleid, v_Layout.regions[i].loop, v_Layout.regions[i].action);
        v_CurrentLayoutRegions[layoutRegionsIndex].push(region);

        for (j = 0; j < v_Layout.regions[i].media.length; j++) {

          if (defined(v_Layout.regions[i].media[j].cyclePlayback)) {
            if (v_Layout.regions[i].media[j - 1]) {
              if (v_Layout.regions[i].media[j].parentWidgetId === v_Layout.regions[i].media[j - 1].parentWidgetId) {
                continue;
              }
            }
          }
          region.addMediaInSequence(v_Layout.regions[i].media[j]);
        }
        // Register event handlers
        region.addEventListener('regionExpired', allRegionsExpired);

        region.start();
        //        displayMedia(region.mediaSequence, 0, i, v_Layout, overlayIndex);
      }
      //Add drawer into layout
      if (v_Layout.drawer) {
        for (var i = 0, max = v_Layout.drawer.length; i < max; i++) {
          var region = new Region(
            v_Layout.drawer[i].id,
            e,
            v_Layout.drawer[i].left,
            v_Layout.drawer[i].top,
            v_Layout.drawer[i].width,
            v_Layout.drawer[i].height,
            v_Layout.drawer[i].zindex,
            isoverlay,
            v_Layout.id,
            v_Layout.scheduleid,
            v_Layout.drawer[i].loop,
            v_Layout.drawer[i].action);
          v_CurrentLayoutRegions[layoutRegionsIndex].push(region);

          for (var j = 0; j < v_Layout.drawer[i].media.length; j++) {
            region.addMediaInSequence(v_Layout.drawer[i].media[j]);
            if (v_Layout.drawer[i].media[j].action) {
              for (var k = 0; k < v_Layout.drawer[i].media[j].action.length; k++) {
                var action = v_Layout.drawer[i].media[j].action[k];
                InteractiveManager.addEventAction(
                  e,
                  action.id,
                  action.triggerType, action.triggerCode,
                  action.actionType, action.layoutCode,
                  action.source, action.sourceId,
                  action.target, action.targetId,
                  action.widgetId, v_Layout.id
                );

              }
            }

          }
        }
      }

      //      for (var i = 0; i < v_Layout.regions.length; i++) {
      //        var nodeRegion = document.createElement("div");
      //        if (isoverlay === true) {
      //          nodeRegion.id = "overlay_" + v_activeOverlay + "_" + v_Layout.regions[i].id;
      //        } else {
      //          nodeRegion.id = v_Layout.regions[i].id;
      //        }
      //
      //        v_Layout.regions[i].regionExpired = false;
      //        v_CurrentLayoutRegions[layoutRegionsIndex].push(v_Layout.regions[i]);
      //        nodeRegion.style.position = "absolute";
      //        //nodeRegion.style.background = "red";
      //        nodeRegion.style.width = v_Layout.regions[i].width;
      //        nodeRegion.style.height = v_Layout.regions[i].height;
      //        nodeRegion.style.top = v_Layout.regions[i].top;
      //        nodeRegion.style.left = v_Layout.regions[i].left;
      //        nodeRegion.style.display = 'flex';
      //        nodeRegion.style.justifyContent = 'center';
      //        nodeRegion.style.alignItems = 'center';
      //        if (defined(v_Layout.regions[i].zindex)) {
      //          nodeRegion.style.zIndex = v_Layout.regions[i].zindex;
      //        }
      //        //console.debug("Number of children: " + nodeRegion.childElementCount);
      //        /* Prepare media sequence */
      //        var mediaSequense = {
      //          regionID: nodeRegion.id,
      //          currentMedia: 0,
      //          sequenceExpired: false,
      //          media: []
      //        };
      //        for (var j = 0; j < v_Layout.regions[i].media.length; j++) {
      //          mediaSequense.media.push(v_Layout.regions[i].media[j]);
      //        }
      //        e.appendChild(nodeRegion);
      //        displayMedia(mediaSequense, 0, i, v_Layout, overlayIndex);
      //      }
      console.debug("Layout regions", v_CurrentLayoutRegions);

      //If layout has events register it to Interactive Manager
      if (v_Layout.actions) {
        for (var i = 0, max = v_Layout.actions.length; i < max; i++) {
          InteractiveManager.addEventAction(
            e,
            v_Layout.actions[i].jAttr.id,
            v_Layout.actions[i].jAttr.triggerType, v_Layout.actions[i].jAttr.triggerCode,
            v_Layout.actions[i].jAttr.actionType, v_Layout.actions[i].jAttr.layoutCode,
            v_Layout.actions[i].jAttr.source, v_Layout.actions[i].jAttr.sourceId,
            v_Layout.actions[i].jAttr.target, v_Layout.actions[i].jAttr.targetId,
            v_Layout.actions[i].jAttr.widgetId, v_Layout.id
          );
        }
      }
      var layoutstartevt = new CustomEvent('layoutStart', {
        detail: {
          layoutId: v_Layout.id,
          isOverlay: isoverlay === true,
          isInteractive: v_isInteractiveLayout
        }
      });
      LayoutManager.dispatchEvent(layoutstartevt);
      layoutStartedTime = utils.getISODate();
    };

    var findRegionById = function (regionId) {
      if (v_CurrentLayoutRegions && v_CurrentLayoutRegions[v_CurrentLayoutId]) {
        for (var i = 0, max = v_CurrentLayoutRegions[v_CurrentLayoutId].length; i < max; i++) {
          if (v_CurrentLayoutRegions[v_CurrentLayoutId][i].id === regionId) {
            return v_CurrentLayoutRegions[v_CurrentLayoutId][i];
            break;
          }
        }
      }
      return null;
    };
    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Public functions">

    /* +------------------------+ Public functions +------------------------+ */
    this.init = function (target) {
      targetElementName = target;
      v_RunningLayouts.push(LayoutParser.getDefault());
    };


    /* Create a signle audioContext to play all audio files! */
    this.audioContext = new AudioContext();


    this.startPlaying = function (isOverlay, overlayIndex) {
      if (!v_isStarted) {
        v_isStarted = true;
        v_ScheduleIsPaused = false;
        v_CurrentLayoutIndex = -1;
        displayLayouts(isOverlay, overlayIndex);
      } else if (shouldStartPlaying) {
        removeSplash();
        shouldStartPlaying = false;
        displayLayouts(isOverlay, overlayIndex);
      }
      if (isOverlay) {
        displayLayouts(isOverlay, overlayIndex);
      }
    };

    this.stopPlaying = function () {
      clearLayoutTimeouts("main", null);
      clearLayoutTimeouts("overlay", null);
      utils.elementSelector("#viewContainer").style.backgroundColor = 'transparent';
      utils.elementSelector("#viewContainer").style.backgroundImage = 'none';
      utils.empty('#viewContainer');
      utils.empty('#overlayContainer');
      lastShowPage = null;
    };

    this.hideExternalInputWidget = function () {
      LayoutManager.shouldDisplayExternalInput = false;
      for (var layoutName in v_CurrentLayoutRegions) {
        var len = v_CurrentLayoutRegions[layoutName].length;
        for (var i = 0; i < len; i++) {
          v_CurrentLayoutRegions[layoutName][i].hideHWTextures();
        }
      }
    };

    this.showExternalInputWidget = function () {
      var isStatusPageVisible = document.getElementById("statusContainer").style.visibility === "visible";
      var isAppMenuBarVisible = document.getElementById("appMenu").className === "showMenu";
      var isLogContainerVisible = document.getElementById("logContainer").style.display === "block";
      if (isStatusPageVisible || isAppMenuBarVisible || isLogContainerVisible) {
        return;
      }
      LayoutManager.shouldDisplayExternalInput = true;
      for (var layoutName in v_CurrentLayoutRegions) {
        var len = v_CurrentLayoutRegions[layoutName].length;
        for (var i = 0; i < len; i++) {
          v_CurrentLayoutRegions[layoutName][i].showHWTextures();
        }
      }
    };

    /**
     * Show the settings screen!
     * @param {type} cbFunction Function to call when the user press the Connect to CMS button.
     * @returns {undefined}
     */
    this.showSettingsScreen = function (cbFunction) {
      LayoutManager.stopPlaying();
      v_isStarted = false;
      if (lastShowPage !== "cmsSettings.html") {
        utils.loadPagetoTarget("cmsSettings.html", "connectCMSContainer", "documentLoad", cbFunction);
        lastShowPage = "cmsSettings.html";
        document.getElementById('connectCMSContainer').style.display = "block";
        v_userShownedSettingsScreen = true;
      }
    };

    this.hideSettingsScreen = function () {
      document.getElementById('connectCMSContainer').style.display = "none";
      document.getElementById('connectCMSContainer').innerHTML = "";
      v_userShownedSettingsScreen = false;
    };
    this.isSettingsScreenOpen = function () {
      return v_userShownedSettingsScreen;
    };

    /**
     * Show a message on the screen.
     * @param {type} cbSuccess
     * @param {type} cbFailure
     * @param {type} message
     * @param {type} showTime How many milliseconds will be show the message. -1 is for ever or until the next message.
     * @returns {undefined}
     */
    this.toast = function (cbSuccess, cbFailure, message, showTime) {
      if (message === undefined) {
        throw {
          name: "Parameter Error",
          level: "Show Stopper",
          message: "Message could not be undefined.",
          htmlMessage: "Message could not be undefined.",
          toString: function () {
            return this.name + ": " + this.message;
          }
        };
      }

      var usehtml = function () {
        if (!defined(showTime)) {
          showTime = 5000;
        }

        var e = document.getElementById("toastMessage");
        e.innerHTML = message.message || message;
        if (showTime !== -1) {
          setTimeout(function () {
            e.innerHTML = "";
          }, showTime);
        }
        if (cbSuccess) {
          cbSuccess();
        }
      };

      DeviceManager.Device.showMessage(cbSuccess, usehtml, message);
    };

    this.getCurrentLayoutId = function () {
      if (v_RunningLayouts.length === 0) {
        return null;
      }
      if (defined(v_RunningLayouts[v_CurrentLayoutIndex])) {
        if (v_RunningLayouts[v_CurrentLayoutIndex].hasOwnProperty("id")) {
          return v_RunningLayouts[v_CurrentLayoutIndex].id;
        } else {
          return null;
        }
      } else {
        return null;
      }
    };

    var removeSplash = function () {
      if (v_RunningLayouts.length > 0) {
        if (v_RunningLayouts[0].id === "splash") {
          // We should also remove any timers in the layout
          v_RunningLayouts = [];
        }
      }
    };

    /**
     * Set the next layouts to play from Schedule Manager.
     * @param {Array} layouts Array of layouts
     * @see If the current layout is in the new layouts array then we should let it expire and then play the new layouts else start immediately.
     **/

    this.setLayouts = function (layouts) {
      var c_layouts = [];

      if (parseOverlayLayout === null) {
        v_ScheduleIsPaused = false;
        //console.debug("Layouts:", layouts);
        var layoutsids = [];
        //application.statusPage.scheduledLayouts_schedule.removeAll();
        layouts.forEach(function (item) {
          layoutsids.push(item.layoutId);
          //application.statusPage.scheduledLayouts_schedule.push(item.layoutId);
        });
        //application.statusPage.scheduledLayouts(layoutsids.join(", "));

        if (!v_RunningLayouts.hasOwnProperty(v_CurrentLayoutIndex)) {
          console.log("Invalid layout index (" + v_CurrentLayoutIndex + ").");
          v_CurrentLayoutIndex = 0;
          shouldStartPlaying = true;
        } else if (layoutsids.indexOf(v_RunningLayouts[v_CurrentLayoutIndex].id) === -1) { //FIXME: That gives cannot read property 'id' of undefined.
          console.debug("Layout " + v_RunningLayouts[v_CurrentLayoutIndex].id + " not in array");
          v_CurrentLayoutIndex = -1;
          shouldStartPlaying = true;
          //          v_RunningLayouts.splice(v_CurrentLayoutIndex, 1);
        } else {
          console.debug("Layout " + v_RunningLayouts[v_CurrentLayoutIndex].id + " in array");
          shouldStartPlaying = false;
        }
        //console.debug('SHOULD START PLAYING: ' + shouldStartPlaying);
        //v_RunningLayouts = [];
        console.debug('Layouts', layouts);
        var options = {'layouts': layouts};

        LayoutParser.parse(
          function () {
            v_parsedLayouts = this;
            if (v_parsedLayouts.length === 0) {
              var schedData = ScheduleManager.getScheduleData();
              if (defined(schedData)) {
                if (ScheduleManager.getScheduleData().hasOwnProperty('default')) {
                  if (ScheduleManager.getScheduleData().default.isValid) {
                    LayoutParser.parse(
                      function () {
                        v_parsedLayouts = this;
                        v_RunningLayouts = [];
                        v_parsedLayouts.forEach(
                          function (l) {
                            v_RunningLayouts.push(l);
                          }
                        );
                        console.debug("Running layouts:", v_RunningLayouts);
                        if (shouldStartPlaying) {
                          LayoutManager.startPlaying();
                        }
                      },
                      null,
                      {'layouts': [{layoutId: ScheduleManager.getScheduleData().default.file}]}
                    );
                  } else {
                    console.warn('I should not be here! v_parsedLayouts.length = 0 but I try to add more from the same list?');
                    v_RunningLayouts = [];
                    v_RunningLayouts.push(LayoutParser.getDefault());
                    v_parsedLayouts.forEach(
                      function (pushlayout) {
                        v_RunningLayouts.push(pushlayout);
                      }
                    );
                  }
                }
              }
            } else {
              if (v_parsedLayouts.length > 0) {
                v_RunningLayouts = [];
                v_parsedLayouts.forEach(
                  function (pushlayout) {
                    v_RunningLayouts.push(pushlayout);
                  }
                );
                var s_layouts = ScheduleManager.getScheduleData().layouts;
                c_layouts = s_layouts.filter(function (sl) {
                  for (var i = 0; i < v_parsedLayouts.length; i++) {
                    if (sl.file == v_parsedLayouts[i].id) {
                      return sl;
                    }
                  }
                  return false;
                });

                checkCampaignOrder(c_layouts);

              }
            }

            console.debug("Running layouts:", v_RunningLayouts);
            if (shouldStartPlaying) {
              LayoutManager.startPlaying();
            }
          },
          null,
          options
        );
      }
    };

    /**
     * Set the next overlay layouts to play from Schedule Manager.
     * @param {Object} overlay Object with layoutID and scheduleID of overlay layout to show.
     * @returns None
     * @see
     * Multiple overlay layouts can be shown.<br>
     * Overlay layout is a standard layout than can be scheduled to play on top of the currently playing layout.<br>
     * The overlay layout is scaled to match the screen dimension as the normal layout does maintaining the aspect ratio.<br>
     * If the background of the overlay layout is just a color then the background is transparent else if it has an image, the image is displayed.
     **/
    var parseOverlayLayout = null;
    this.showOverlay = function (overlay) {
      console.debug("Show Overlay layout " + overlay.layoutId);

      var parseOverLayLayout = [overlay];
      //v_displayOverLayLayout = overlay;
      var options = {'layouts': parseOverLayLayout};
      LayoutParser.parse(
        function () {
          v_parsedLayouts = this;
          v_parsedLayouts.forEach(
            function (pushlayout) {
              v_displayOverLayLayout[v_activeOverlay] = pushlayout;
              v_displayOverLayLayout[v_activeOverlay].overlayIndex = v_activeOverlay;
            }
          );

          LayoutManager.startPlaying(true, v_activeOverlay);
          v_activeOverlay++;
        },
        null,
        options
      );
    };

    /**
     * Remove a previously shown layout.
     * @param {String} overlay ID on overlay to remove.
     * @returns None
     */
    this.removeOverlay = function (overlay) {
      try {
        console.debug("Remove Overlay layout " + overlay.layoutId);
        var id = "overlay_" + overlay.layoutId;
        var e = utils.elementSelector("#overlay-" + overlay.layoutId + "-outer");

        /* If strange things happen then this is something we have to implement! */

        //        e.children[0].children[0].children.each(function() {
        //            console.log(this);
        //            for (var i = 0, max = this.children.length; i < max; i++) {
        //                utils.off(this.childer[i]);
        //            }
        //            //jQueryWebOs(this).children("video").off();
        //        });

        //        jQueryWebOs("#overlay-"+overlay.layoutId+"-outer").children().first().children().first().children().each(function() {
        //            jQueryWebOs(this).children("video").off();
        //        });
        if (e) {
          v_CurrentLayoutRegions[id].forEach(function (reg) {
            reg.stop();
          });
          utils.empty(e.children[0]);
          utils.remove(e);
        }
      } catch (e) {
        console.error("Error occured while remove overlay layout", e);
      }
    };


    /** Orientation Changed
     * @version 2.0.3
     * @since 2.0.3
     */
    this.orientationChanged = function () {
      //Main content
      if (v_RunningLayouts.length === 0) {
        return;
      }
      if (lastShowPage === "cmsSettings.html") {
        return;
      }

      var v_Layout = v_RunningLayouts[v_CurrentLayoutIndex];
      var scaleroptions = {
        "originalWidth": v_Layout.width,
        "originalHeight": v_Layout.height,
        "previewWidth": document.body.getBoundingClientRect().width,
        "previewHeight": document.body.getBoundingClientRect().height,
        "scaleOverride": 0,
        "widgetDesignWidth": v_Layout.width,
        "widgetDesignHeight": v_Layout.height,
        "widgetDesignPadding": 0,
        "itemsPerPage": 0
      };

      utils.scaler("#viewContainer", scaleroptions);

      // Overlay content
      //utils.scaler("#overlay-"+v_Layout.id+"-innercontainer-"+v_activeOverlay,scaleroptions);
    };

    /* Handling layout and region changes. Mostly used in interactive scenarios */

    /**
     * Set the next layout to be displayed next and switch to it immediately.
     * <br><i>navLayout</i> equivalent.
     * @param {string} layoutID The id of the layout
     * @param {"touch"|"webhook"} source One of "touch" or "webhook"
     * @returns {undefined}
     */
    this.setLayoutByAction = function (layoutID, source) {
      console.log('Switch to interactive layout with id ', layoutID);
      ScheduleManager.Commands.changeLayout(layoutID, CacheManager.getLayoutDuration(layoutID, 0), false, "replace", source);
      v_isInteractiveLayout = true;

    };

    /**
     * Is the current layout interactive?
     * @returns {boolean} True if layout is interactive
     */
    this.isInteractiveLayout = function () {
      return v_isInteractiveLayout;
    };


    /**
     * Display a specific media on the specific region.
     * <br><i>navWidget</i> equivalent.
     */
    this.displaySpecificMedia = function () {
      console.warn('displaySpecificMedia not implimentend yet.');
    };

    /**
     * Display next layout.
     * <br><i>next</i> layout equivalent.
     */
    this.nextLayout = function () {
      displayLayouts();
    };

    /**
     * Display previous layout.
     * <br><i>previous</i> layout equivalent.
     */
    this.previousLayout = function () {
      v_CurrentLayoutIndex -= 2;
      if (v_CurrentLayoutIndex < 0) {
        v_CurrentLayoutIndex = v_RunningLayouts.length + v_CurrentLayoutIndex;
      }
      displayLayouts();
    };

    /**
     * Display next widget.
     * <br><i>next</i> widget equivalent.
     * @param {String} regionId The Region Id.
     */
    this.nextWidget = function (regionId) {
      var region = findRegionById(regionId);
      if (region) {
        region.playNextWidget();
      }
    };

    /**
     * Display previous widget.
     * <br><i>previous</i> widget equivalent.
     * @param {String} regionId The Region Id.
     */
    this.previousWidget = function (regionId) {
      var region = findRegionById(regionId);
      if (region) {
        region.playPreviousWidget();
      }
    };

    /**
     * Get layout with specific code.
     * @param {Strign} code The code to look for.
     * @returns {Object|null} Return the layout with the specific code or null.
     */
    this.getLayoutByCode = function (code) {
      for (var i = 0; i < v_parsedLayouts.length; i++) {
        if (v_parsedLayouts[i].code === code) {
          return v_parsedLayouts[i];
        }
      }
      return null;
    };

    /**
     * Play widget on specific region
     * @param {type} widgetId
     * @param {type} regionId
     * @returns {undefined}
     */
    this.playWidgetToRegion = function (widgetId, regionId) {
      console.debug(v_CurrentLayoutRegions, v_RunningLayouts);
      console.log('Play widget ' + widgetId + ' in region ' + regionId);
      if (!(v_RunningLayouts[v_CurrentLayoutIndex].drawer.length)) {
        return;
      }

      //Find the index of the widget from the drawer.
      var drawerWidgetIndex = -1;
      for (var i = 0, max = v_RunningLayouts[v_CurrentLayoutIndex].drawer.length; i < max; i++) {
        if (v_RunningLayouts[v_CurrentLayoutIndex].drawer[i].id === widgetId) {
          //Found
          drawerWidgetIndex = i;
          break;
        }
      }

      if (regionId === "screen") {
        console.info('Run a shell command action');
        //TODO: Test if that works correctly, especially with statistics.
        LayoutManager.factory(
          'shellcommand',
          v_RunningLayouts[v_CurrentLayoutIndex].drawer[drawerWidgetIndex],
          "screen",
          function (something) {
            console.info('Back from shellcommand', something);
          },
          v_CurrentLayoutId,
          v_ScheduleId,
          ""
        );
      } else {
        var regIdx = -1;
        for (var i = 0, max = v_CurrentLayoutRegions[v_CurrentLayoutId].length; i < max; i++) {
          if (v_CurrentLayoutRegions[v_CurrentLayoutId][i].id === regionId) {
            regIdx = i;
            break;
          }
        }
        if (regIdx > -1 && drawerWidgetIndex > -1) {
          //Found
          console.log("Region " + regionId + " found. Play widget " + widgetId);
          v_CurrentLayoutRegions[v_CurrentLayoutId][regIdx].playImmediateWidget(v_RunningLayouts[v_CurrentLayoutIndex].drawer[drawerWidgetIndex]);
        }
      }

    };

    var findRegionByWidgetId = function (widgetId) {
      if (v_CurrentLayoutRegions[v_CurrentLayoutId]) {
        for (var i = 0, max = v_CurrentLayoutRegions[v_CurrentLayoutId].length; i < max; i++) {
          if (v_CurrentLayoutRegions[v_CurrentLayoutId][i].id === widgetId) {
            return v_CurrentLayoutRegions[v_CurrentLayoutId][i];
          }
        }
      }
      return null;
    };

    /**
     * Expires a widget immediately
     * @param {String} widgetId Widget Id
     */
    this.expireWidget = function (widgetId) {
      var region = findRegionByWidgetId(widgetId);
      if (region) {
        region.expireWidget();
      }
    };

    /**
     * Extend the duration of a widget by the specified value
     * @param {String} widgetId Widget Id
     * @param {Number} duration Duration in seconds to extend.
     */
    this.extendWidgetDuration = function (widgetId, duration) {
      var region = findRegionByWidgetId(widgetId);
      if (region) {
        region.extendWidgetDuration(duration);
      }
    };

    /**
     * Set the duration of a widget to the specified value
     * @param {String} widgetId Widget Id
     * @param {Number} duration Duration in seconds to set.
     */
    this.setWidgetDuration = function (widgetId, duration) {
      var region = findRegionByWidgetId(widgetId);
      if (region) {
        region.setWidgetDuration(duration);
      }
    };

    // Testing functions
    this.pauseAllMedia = function () {
      v_CurrentLayoutRegions[v_CurrentLayoutId].forEach(function (reg) {
        reg.pause();
      });
    };

    this.resumeAllMedia = function () {
      v_CurrentLayoutRegions[v_CurrentLayoutId].forEach(function (reg) {
        reg.resume();
      });
    };

    this.stopAllMedia = function () {
      var len = v_CurrentLayoutRegions[v_CurrentLayoutId].length;
      for (var i = 0; i < len; i++) {
        v_CurrentLayoutRegions[v_CurrentLayoutId][i].stop();
      }
    };

    this.startAllMedia = function () {
      var len = v_CurrentLayoutRegions[v_CurrentLayoutId].length;
      for (var i = 0; i < len; i++) {
        v_CurrentLayoutRegions[v_CurrentLayoutId][i].start();
      }
    };
    // </editor-fold>

  };

  // </editor-fold desc="Class LayoutManager">

  // <editor-fold defaultstate="collapsed" desc="Class LayoutParser">
  window.LayoutParser = new function () {
    /* Splash Layout - This will be displayed if no array layouts is empty */
    var SplashLayout = {
      id: "splash",
      width: "1920",
      height: "1080",
      resolutionid: "1",
      bgcolor: "#000000",
      schemaVersion: "1",
      background: "",

      regions: [
        {
          id: "default_1",
          userId: "1",
          width: "1920",
          height: "1080",
          position: "absolute",
          top: "0",
          left: "0",
          loop: 0,

          media: [
            {
              id: "0",
              type: "image",
              render: "native",
              duration: "10",
              lkid: "2226",
              userId: "1",
              schemaVersion: "1",
              isbase64: 1,
              media: {
                options: [{
                  uri: [{jValue: null}],
                  transIn: [{jValue: "fadeIn"}],
                  transInDirection: [{jValue: "North"}],
                  transInDuration: [{jValue: "1500"}],
                  transOut: [{jValue: "fadeOut"}],
                  transOutDirection: [{jValue: "North"}],
                  transOutDuration: [{jValue: "1500"}]
                }]
              },
              raw: [],
              mediaExpired: false
            }
          ]
        }
      ]
    };


    var parseMedia = function (media) {
      var l_media = {
        id: media.jAttr.id,
        type: media.jAttr.type,
        render: media.jAttr.render,
        duration: media.jAttr.duration,
        media: media,
        enableStat: (media.jAttr.hasOwnProperty('enableStat') ? media.jAttr.enableStat : "1"),
        parentWidgetId: media.jAttr.parentWidgetId
      };
      if (media.action) {
        l_media.action = media.action;
      }
      if (media.jAttr.cyclePlayback == "1") {
        l_media.cyclePlayback = "1";
        l_media.countplay = media.jAttr.playCount;
        l_media.IsRandom = media.jAttr.isRandom;

        CacheManager.setCycleListSequence(l_media);
      }
      return l_media;
    };
    /* Public functions */
    /**
     * Get the default layout until Scheduler provide other layouts
     * @returns {LayoutParser.DefaultLayout} The default layout
     */

    /**
     *  @param LayoutID - The id of the layout to be parsed
     *  @param callback - Callback function
     *  */
    var loadLayout = function (LayoutID, callback) {
      var xhttp = new XMLHttpRequest();
      xhttp.onreadystatechange = function () {
        if (this.readyState === 4) {
          switch (this.status) {
            case 0:
            case 200:
              if (this.responseText.length === 0) {
                console.error("Empty response on load layout!");
                Logger.cmsFault(undefined, undefined,
                  {
                    code: ERRORS.FaultCodes.RemoteResourceFailed,
                    reason: "Could not load layout " + LayoutID.layoutId + ".xlf.",
                    method: "loadLayout",
                    thread: "LayoutParser",
                    date: new Date(),
                    expires: (DeviceManager.Config.getOptions() && DeviceManager.Config.getOptions().collectInterval) ? new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000) : new Date((new Date()).valueOf() + 3600000),
                    layoutId: LayoutID.layoutId,
                    sceduleId: LayoutID.scheduleId,
                    onlyOnce: true
                  });
                return;
              }
              var x2jObj = X2J.parseXml(this.responseText);
              //var parser = new DOMParser();

              //x2jObj.rawjqueryparse = jQueryWebOs.parseXML(this.responseText);
              //x2jObj.rawjqueryparse = parser.parseFromString(this.responseText, "text/xml");
              callback.apply(x2jObj);
              break;
            default:
              Logger.cmsFault(undefined, undefined,
                {
                  code: ERRORS.FaultCodes.RemoteResourceFailed,
                  reason: "Failed to parse layout with Id " + LayoutID.layoutId,
                  method: "loadLayout",
                  thread: "LayoutParser",
                  date: new Date(),
                  expires: (DeviceManager.Config.getOptions() && DeviceManager.Config.getOptions().collectInterval) ? new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000) : new Date((new Date()).valueOf() + 3600000),
                  layoutId: LayoutID.layoutId,
                  sceduleId: LayoutID.scheduleId
                });
              //console.warn("Error "+ this.status + " - " + LayoutID.layoutId+".xlf : " + this.statusText);
              break;
          }
        }
      };

      xhttp.open("GET", "file:///opt/usr/home/owner/content/Downloads/" + LayoutID.layoutId + ".xlf", true);
      try {
        xhttp.send();
      } catch (e) {
        throw e;
      }

    };

    /** Parse layout
     * @param LayoutID - The id of the layout to be parsed.
     * @param {function} callback - Callback function after layout is parsed.
     * @version 2.0.202
     * @since 1.8.1
     * */

    var parseLayout = function (LayoutID, callback) {
      loadLayout(LayoutID, function () {
        console.debug('Layout Log:', this[0].layout[0]);
        //console.debug('Raw layout parse:',this.rawjqueryparse);
        /* Get the layout attributes */
        //XLG-137: Failed to parse layout - shows only splash screen
        try {
          var attributes = this[0].layout[0].jAttr;
        } catch (e) {
          LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
          var objIdx1 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
          if (objIdx1 !== -1) {
            LayoutID.layoutId[objIdx1].isValid = false;
            application.statusPage.allLayouts.notifySubscribers();
          }
          isBlacklisted = true;
          Logger.cmsLog({
            category: "audit",
            message: "Layout with Id " + LayoutID.layoutId + " has no attributes set. " + e.toString(),
            method: "parseLayout",
            thread: "LayoutParser",
            onlyOnce: true
          });
        }

        //var attributes = this[0].layout[0].jAttr;
        /* initiate the regions array to be appended on layout object */
        var regions = [];
        /* Set the layout as blacklisted */
        var isBlacklisted = false;
        //var rawjqueryparse = this.rawjqueryparse;
        this[0].layout[0].region.forEach(function (region, idx) {
          /* initiate the media array to be appended on region */
          var regionmedia = [];
          //var rawregion = rawjqueryparse.children[0].children[idx];
          try {

            if (defined(region.media)) {
              region.media.forEach(function (media, idx) {
                //XLG-111: Widget from and to dates should be parsed to exclude expired widgets
                var shouldAdd = true;
                if (media.jAttr.hasOwnProperty('fromDt') && (media.jAttr.hasOwnProperty('toDt'))) {
                  var now = utils.getTimeStamp();
                  var df = utils.parseDate(media.jAttr.fromDt);
                  var dt = utils.parseDate(media.jAttr.toDt);

                  if (df > now || now > dt) {
                    shouldAdd = false;
                    Logger.cmsLog({
                      category: "audit",
                      message: "Media with id " + media.jAttr.id + " on Region with id " + region.jAttr.id + " and on Layout with Id " + LayoutID.layoutId + " is expired.",
                      method: "parseLayout",
                      thread: "LayoutParser",
                      onlyOnce: true
                    });
                  }
                }
                //Logger.log(media, Logger.DEBUG);
                if (LibraryManager.blacklisted_widgets.indexOf(media.jAttr.fileId) > -1) {
                  console.warn("Layout with ID " + LayoutID.layoutId + " has blacklisted media with ID " + media.jAttr.fileId);
                  isBlacklisted = true;
                  shouldAdd = false;
                }
                if (shouldAdd) {
                  var thismedia = parseMedia(media);
                  regionmedia.push(thismedia);
                }
              });
            }
          } catch (e) {
            LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
            var objIdx2 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
            if (objIdx2 !== -1) {
              LayoutID.layoutId[objIdx2].isValid = false;
              application.statusPage.allLayouts.notifySubscribers();
            }
            isBlacklisted = true;
            Logger.cmsLog({
              category: "audit",
              message: "Region with id " + region.jAttr.id + " and on Layout with Id " + LayoutID.layoutId + " is erroneous. " + e.toString(),
              method: "parseLayout",
              thread: "LayoutParser",
              onlyOnce: true
            });
          }

          var regionloop;
          try {
            if (defined(region.options)) {
              if (region.options[0].hasOwnProperty('loop')) {
                if (region.options[0].loop[0].jValue === "1") {
                  regionloop = 1;
                } else {
                  regionloop = 0;
                }
              } else {
                regionloop = 0;
              }
            } else {
              regionloop = 0;
            }
          } catch (e) {
            LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
            var objIdx3 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
            if (objIdx3 !== -1) {
              LayoutID.layoutId[objIdx3].isValid = false;
              application.statusPage.allLayouts.notifySubscribers();
            }
            isBlacklisted = true;
            Logger.cmsLog({
              category: "audit",
              message: "Region with id " + region.jAttr.id + " and on Layout with Id " + LayoutID.layoutId + ". Unable to handle region options. " + e.toString(),
              method: "parseLayout",
              thread: "LayoutParser",
              onlyOnce: true
            });
          }

          try {
            var regionoutput = {
              id: region.jAttr.id,
              userId: "1",
              width: region.jAttr.width,
              height: region.jAttr.height,
              top: region.jAttr.top,
              left: region.jAttr.left,
              loop: regionloop,
              media: regionmedia
            };

            if (defined(region.jAttr.zindex)) {
              regionoutput.zindex = region.jAttr.zindex;
            }

            if (region.action) {
              regionoutput.action = region.action;
            }
          } catch (e) {
            LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
            var objIdx4 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
            if (objIdx4 !== -1) {
              LayoutID.layoutId[objIdx4].isValid = false;
              application.statusPage.allLayouts.notifySubscribers();
            }
            isBlacklisted = true;
            Logger.cmsLog({
              category: "audit",
              message: "Unable to set region attributes on Region with id " + region.jAttr.id + " and on Layout with Id " + LayoutID.layoutId + ". " + e.toString(),
              method: "parseLayout",
              thread: "LayoutParser",
              onlyOnce: true
            });
          }


          regions.push(regionoutput);
        });

        /* Preload the background of the layout to avoid black screen between layout switching */
        try {
          var layout = {
            id: LayoutID.layoutId,
            scheduleid: LayoutID.scheduleId,
            width: attributes.width,
            height: attributes.height,
            resolutionid: "1",
            bgcolor: attributes.bgcolor,
            schemaVersion: attributes.schemaVersion,
            regions: regions,
            enableStat: (attributes.hasOwnProperty('enableStat') ? attributes.enableStat : "1")
          };
          if (attributes.code) {
            layout.code = attributes.code;
          }
          //XXX: I don't like passing the whole object without parsing it first.
          if (this[0].layout[0].drawer && this[0].layout[0].drawer[0]) {
            var dwr = [];
            if (this[0].layout[0].drawer[0].media) {
              for (var i = 0, max = this[0].layout[0].drawer[0].media.length; i < max; i++) {
                dwr.push(parseMedia(this[0].layout[0].drawer[0].media[i]));
              }
            }
            layout.drawer = dwr;
          }

          if (this[0].layout[0].action) {
            layout.actions = this[0].layout[0].action;
          }

          if (defined(attributes.background)) {
            layout.background = "file:///opt/usr/home/owner/content/Downloads/" + attributes.background;
          } else {
            layout.background = "";
          }
        } catch (e) {
          LayoutManager.invalidLayoutIds.push(LayoutID.layoutId);
          var objIdx5 = application.statusPage.allLayouts().indexOf(LayoutID.layoutId);
          if (objIdx5 !== -1) {
            LayoutID.layoutId[objIdx5].isValid = false;
            application.statusPage.allLayouts.notifySubscribers();
          }
          isBlacklisted = true;
          Logger.cmsLog({
            category: "audit",
            message: "Unable to set layout attributes on Layout with Id " + LayoutID.layoutId + ". " + e.toString(),
            method: "parseLayout",
            thread: "LayoutParser",
            onlyOnce: true
          });
        }

        if (!isBlacklisted) {
          callback.apply(layout);
        } else {
          callback.apply(null);
        }
      });
    };

    this.getDefault = function () {
      var splashimage = LayoutParser.SplashImage();
      SplashLayout.regions[0].media[0].media.options[0].uri[0].jValue = splashimage;

      //Orientation is set in DeviceManager.Device.setNativePortraitMode function.
      var o = DeviceManager.Device.getOrientation();
      if (o === DeviceManager.Device.SCREENORIENTATION_PORTRAIT ||
        o === DeviceManager.Device.SCREENORIENTATION_REVERSE_PORTRAIT) {
        SplashLayout.width = 1080;
        SplashLayout.height = 1920;
        SplashLayout.regions[0].width = 1080;
        SplashLayout.regions[0].height = 1920;
      }
      return SplashLayout;
    };

    /**
     * Parse all layouts in options
     * @param {function} cbSuccess
     * @param {function} cbFailure
     * @param {object} options An object with a .layout property
     * <br>options.layout {Array} Array of layout objects. Layouts objects have layout id and schedule id.
     * @returns {undefined}
     */
    this.parse = function (cbSuccess, cbFailure, options) {
      var layouts = [];
      var layoutsindex = -1;

      function nextlayout() {
        layoutsindex++;
        if (typeof options.layouts[layoutsindex] !== 'undefined') {
          parseLayout(options.layouts[layoutsindex], function () {
            if (this !== null) {
              var shouldAdd = true;
              for (var i = 0; i < this.regions.length; i++) {
                if (this.regions[i].media.length === 0) {
                  shouldAdd = false;
                  Logger.cmsFault(undefined, undefined,
                    {
                      code: ERRORS.FaultCodes.XlfNoContent,
                      reason: "Region " + this.regions[i].id + " is empty on Layout with Id " + options.layouts[layoutsindex].layoutId + ".",
                      method: "parseLayout",
                      thread: "LayoutParser",
                      date: new Date(),
                      expires: (DeviceManager.Config.getOptions() && DeviceManager.Config.getOptions().collectInterval) ? new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000) : new Date((new Date()).valueOf() + 3600000),
                      layoutId: options.layouts[layoutsindex].layoutId,
                      sceduleId: options.layouts[layoutsindex].scheduleId,
                      onlyOnce: true
                    });
                  break;
                }
              }
              var invalidLayoutIdIndex = LayoutManager.invalidLayoutIds.indexOf(options.layouts[layoutsindex].layoutId);
              if (shouldAdd) {
                layouts.push(this);
                if (invalidLayoutIdIndex > -1) {
                  LayoutManager.invalidLayoutIds.splice(invalidLayoutIdIndex, 1);
                }
              } else {
                if (invalidLayoutIdIndex === -1) {
                  LayoutManager.invalidLayoutIds.push(options.layouts[layoutsindex].layoutId);
                }
              }
            }
            nextlayout();
          });
        } else {
          cbSuccess.apply(layouts);
        }
      }

      nextlayout();
    };
  };
  // </editor-fold>
}());