/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */
/* global DeviceManager, CommunicationManager, LibraryManager, debug, utils, LayoutManager, ERRORS, XMRManager, logic, Power, LicenseManager, application, Logger, AppVersionInformation, lblMainLoop, CacheManager */
(function(){
	"use strict";
	/**
	 * @classdesc Schedule handling.
	 * @type ScheduleManager
	 * @version 1.8.1
	 * @since 1.8.1
	 */
	//jshint -W057
	window.ScheduleManager = new function _ScheduleManager() {
		"use strict";
		/* +------------------------+ Private variables +------------------------+ */
		// <editor-fold defaultstate="collapsed" desc="Private variables">
		/**
		 * Flag if collect Interval is running.
		 */
		var v_collectIntervalRunningState = false;

		/**
		 * Layouts that are shown by changeLayout command.
		 */
		var v_changeLayoutLayouts = [];

		/**
		 * Flag if the Layout should receive layouts or stay on a specific state like settings screen!
		 */
		var v_runningSchedule = false;

		/**
		 * Object with schedule data.
		 */
		var v_scheduleData = null;

		/**
		 * Reference to collect Interval timer.
		 */
		var v_collectIntervalTimer = null;

		/**
		 * Array that holds which overlay layout is currently showned.
		 */
		var v_CurrentOverlayLayouts = [];

		/**
		 * Flag if the schedule has been run at least once.
		 */
		var v_runScheduleFirstTime = false;
		// </editor-fold>


		/**
		 * How many times concurrent connection to XMDS failed.
		 */
		var v_ConcurrentFailedXMDSConnetion = 0;

		/**
		 * Indicate if the content is ready to play.
		 */
		var contentValid = false;

	    /**
	     * Last "required files" CRC (from CMS).
	     */
		var v_localCheckRfCRC = "";

		/**
   		* Last "required files" CRC helper (from CMS).
   		*/
  		var v_localCheckRfHelperCRC = "";

	    /**
	     * Last schedule CRC (from CMS).
	     */
	    var v_localCheckScheduleCRC = "";

		/**
   		* Last schedule CRC helper (from CMS).
   		*/
  		var v_localCheckScheduleHelperCRC = "";

	    // </editor-fold>

		/* +------------------------+ Private functions +------------------------+ */
		// <editor-fold defaultstate="collapsed" desc="Private functions">

		var errorCB = function(cbError){
			logic.nonFatalError(cbError);
		};

		var failedToConnectToCMS = function(){
			v_ConcurrentFailedXMDSConnetion++;
			application.statusPage.failedXMDSConnections(v_ConcurrentFailedXMDSConnetion);
		};

		var resetFailedToConnectToCMS = function(){
			v_ConcurrentFailedXMDSConnetion = 0;
			application.statusPage.failedXMDSConnections(v_ConcurrentFailedXMDSConnetion);
		};



		var requestScheduleFromCMS = function(successCb, failureCb, cmsResponse){
			var createDataForLayoutManager = function(){
				v_collectIntervalRunningState = false;
				if (!v_runScheduleFirstTime){
					v_runScheduleFirstTime = true;
					informLayoutWithSchedule();
				}

				/* As per Dan suggestion we should not send MediaInvetory on collect for bandwidth usage */
				if(successCb){
					successCb();
				}
			};

			var downloadScheduleCompleted = function(){
				contentValid = validateContent();

				if (contentValid){
					/* Content is validated */
					console.debug("Content is valid");
					application.statusPage.scheduleStatus("Up to date");
					ScheduleManager.saveCachedSchedule();
					v_localCheckRfCRC = v_localCheckRfHelperCRC;
					v_localCheckScheduleCRC = v_localCheckScheduleHelperCRC;
					alwaysSendToCMS(createDataForLayoutManager,function(cbError){
						v_collectIntervalRunningState = false;
						errorCB(cbError);
					});
				}else{
					/* Content is NOT validated */
					console.debug("Content is not valid");
					application.statusPage.scheduleStatus("Content is not valid.");
					v_collectIntervalRunningState = false;
					alwaysSendToCMS(failureCb ,function(cbError){
						v_collectIntervalRunningState = false;
						errorCB(cbError);
					});
				};

			};

				var downloadSchedule = function(successCb, failureCb){

					        /* Only in trial or licensed displays can download schedule */

					        console.debug("Download schedule!");
					        // XLG-103 : Add more logs on application start!
					        //application.statusPage.scheduleStatus("Pending...");
					        application.statusPage.scheduleStatus("Download schedule ...");

					        var shouldDownload = true;
					        if (cmsResponse.hasOwnProperty('checkSchedule') && cmsResponse.checkSchedule!==""){
					          shouldDownload = cmsResponse.checkSchedule !== v_localCheckScheduleCRC;
					        }
					        if (shouldDownload){
					          CommunicationManager.fetchSchedule(function(data){
								v_localCheckScheduleHelperCRC = cmsResponse.checkSchedule;
					            parseSchedule(data);
					            successCb();
					          },failureCb);
					        }else{
							  application.statusPage.scheduleStatus("Skipped");
					          successCb();
					        }
					      };




			var parseRequiredFilesCompleted = function(){
				application.statusPage.scheduleStatus("Download schedule ...");
				downloadSchedule(downloadScheduleCompleted,function(cbError){
					v_collectIntervalRunningState = false;
					alwaysSendToCMS(function(){errorCB(cbError);},errorCB);
				});
			};

			var getRequiredFilesCompleted = function(cbObj){
				v_localCheckRfHelperCRC = cmsResponse.checkRf;
				// XLG-103 : Add more logs on application start!
				application.statusPage.scheduleStatus("Check required files ...");
				parseRequiredFiles(
					parseRequiredFilesCompleted,
						function(cbError){
						v_collectIntervalRunningState = false;
							alwaysSendToCMS(function(){errorCB(cbError);},errorCB);
						},
						cbObj
					);
			};


			var cbSISuccess = function(){
				if (LicenseManager.inTrial() || LicenseManager.isLicenseValid()){
					console.debug("I have a valid license!");
					DeviceManager.Device.Power.setInternalClock();
					/* First option is get Required Files */
	                // Honor download time window
	                var inDownloadWindow=true;
	                if (DeviceManager.Config.getOptions().hasOwnProperty("downloadStartWindow") &&
	                		DeviceManager.Config.getOptions().hasOwnProperty("downloadEndWindow")){
					  var dlwindow = new Date();
					  dlwindow.setSeconds(0);
	                  var s = DeviceManager.Config.getOptions().downloadStartWindow;
	                  var e = DeviceManager.Config.getOptions().downloadEndWindow;
	                  try {
	                    inDownloadWindow = dlwindow.inDateTimeRange(s,e);
	                  } catch (e) {
	                    //Logger.cmsLog({category:"error", message:e.errorText, type:"error", method:"connectToCMS", thread:"Schedule Manager"});
						// XXX: make a suggestion for a proper error code for this case
						Logger.cmsFault(undefined, undefined,
							{
								code:ERRORS.FaultCodes.RemoteResourceFailed,
								reason: e.errorText,
								method:"connectToCMS",
								thread:"Schedule Manager",
								date:new Date(),
								expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000)
							}
						);
	                  }
	                }
	                if (inDownloadWindow){
						application.statusPage.isInsideDownloadWindow(true);
                   		application.statusPage.isInsideDownloadWindow.notifySubscribers();
	                	//XLG-154: Bandwidth usage improvements: checkSchedule and checkRf on RegisterDisplay
	                	var shouldDownload = true;
	                	if (cmsResponse.hasOwnProperty('checkRf') && cmsResponse.checkRf!==""){
	                		shouldDownload = cmsResponse.checkRf !== v_localCheckRfCRC;
	                	}
	                	if (shouldDownload){
							application.statusPage.scheduleStatus("Get required files ...");
	                		application.statusPage.requiredFilesSkipped(false);
	                		CommunicationManager.getRequiredFiles(getRequiredFilesCompleted,
	                		function(cbError){
	                	      v_collectIntervalRunningState = false;
	                	        console.error("Error", cbError);
	                	        alwaysSendToCMS(function(){errorCB(cbError);},errorCB);
	                			}
	                		);
	                	}else{
	                		application.statusPage.requiredFilesSkipped(true);
	                		parseRequiredFilesCompleted();
	                  }
	                }else{
					  if(!validateContent()){
						application.statusPage.scheduleStatus('Postponed download')
					  }
					  application.statusPage.isInsideDownloadWindow(false);
                  	  application.statusPage.isInsideDownloadWindow.notifySubscribers();
	                  console.debug("Outside download window");
	                  v_collectIntervalRunningState = false;
	                  alwaysSendToCMS(function(){
	                    playLocalContent();
	                    if (successCb){
	                      successCb();
	                    }
	                  },errorCB);
	                }
				}else{
					console.debug("Not license, no schedule!");
					v_collectIntervalRunningState = false;
					playLocalContent();
					if (successCb){
						successCb();
					}
				}
			};

			var alwaysSendToCMS = function(successASTCCb, failureASTCCb){
				Logger.submitStats(function(){
					console.debug("Send stats completed");
					Logger.submitLog(function(){
					  console.debug("Send log completed");
					  Logger.submitFaults(function(){
						console.debug("Send faults completed");
					}, failureASTCCb);
						// XLG-103 : Add more logs on application start!
						application.statusPage.scheduleStatus("Up to date.");
						if (successASTCCb){
							successASTCCb();
						}
					}, failureASTCCb);
				}, failureASTCCb);
			};

			/* First option */
			CommunicationManager.sendInfo(cbSISuccess,function(cbError){
				v_collectIntervalRunningState = false;
				errorCB(cbError);
			});
		};

		/**
		 * Try to connect to the CMS.
		 * @param {Boolean} saveSettings If coming from User parameter view, set it true to save the parameters.
		 * @returns {undefined} None
		 * @version 1.8.1
		 * @since 1.8.1
		 */
		var connectToCMS = function (saveSettings) {
			if (DeviceManager.Config.settingsValid()) {
				if (saveSettings === true)
				{
					DeviceManager.Config.saveConfiguration();
				}

				console.debug("Connect to cms!");
				CommunicationManager.findCMSVersion(
						function(){
							CommunicationManager.ConnectCMS(
									function(successCb, faulureCb, response){
										resetFailedToConnectToCMS();
										connectCMSResponse(null, faulureCb, response);
									},
									function(errObj){
										failedToConnectToCMS();
										errorCB(errObj);

									},
									{URL:DeviceManager.Config.getURL()}
							);
						},
						function(){
							failedToConnectToCMS();
							console.error("Could not connect to the CMS");
							LayoutManager.toast(null,null,"Could not connect to the CMS");
						}
				);

			} else
			{
				failedToConnectToCMS();
				errorCB(new ERRORS.ErrorWithSettings());
			}
		};


		var connectCMSResponse = function (successCb, failureCb, connectResponse) {
			console.debug("Handle the responce: ", connectResponse);
			v_ConcurrentFailedXMDSConnetion = 0;

			if (connectResponse.hasOwnProperty("status")) /* Normal response from CMS */
			{
				if (connectResponse.status === "0") /* Monitor is licensed! */
				{
					application.statusPage.cmsRegistered('True');
					application.statusPage.cmsRegInfo("Registered.");
					application.statusPage.cmsRegInfoTime(utils.getLocalDate());
					var nextScheduleUpdateTime = Date.now() + Number(DeviceManager.Config.getOptions().collectInterval) * 1000;
					application.statusPage.nextScheduleUpdate(utils.getISODate(nextScheduleUpdateTime));

					//CMS Move can happen here!
					if (connectResponse.options){
						if (connectResponse.options.newCmsAddress && connectResponse.options.newCmsKey
							&& connectResponse.options.newCmsAddress!=="" && connectResponse.options.newCmsKey!==""){
							CommunicationManager.ConnectCMS(
							function(s,f,newcmsresp){
								if(newcmsresp.status === "0"){/* Monitor is authorized on new server! */
								DeviceManager.Config.setURL(connectResponse.options.newCmsAddress);
								DeviceManager.Config.setPassword(connectResponse.options.newCmsKey);
								DeviceManager.Config.saveConfiguration();
								Logger.clearLogs(function(){
									Logger.clearStats(function(){
									//Restart? Clean up?
									logic.softRestart();
									}, failureCb);
								}, failureCb);
								}
							},
							function(r){
								console.error("CMS Move :", r);
							},
							{URL:connectResponse.options.newCmsAddress, serverKey:connectResponse.options.newCmsKey}
							);
						}
					}

					/* Inform that we are a licensed display */
					//v_runningSchedule = v_changeLayoutLayouts.length === 0;
					v_runningSchedule = !LayoutManager.isInteractiveLayout(); //Might lead to deadlock!
					/* This is also started from ScheduleManager.start() */
					if (!v_collectIntervalTimer){
						ScheduleManager.startInterval();
					}
					try {
						LayoutManager.startPlaying();
					} catch (e) {
						console.error(e);
					}

					/* TODO: Here we have to connect to XMR. */

					if(DeviceManager.Config.getOptions().xmrNetworkAddress !== XMRManager.getAddress()) {
						XMRManager.close();
					};
					XMRManager.connect();

					if (connectResponse.hasOwnProperty('version_instructions'))
					{
						//console.debug(connectResponse.version_instructions);
						try {
							if (connectResponse.version_instructions !== ""){
								var upg = JSON.parse(connectResponse.version_instructions);

								if (upg.hasOwnProperty('id')){
									if (upg.hasOwnProperty('code')){
                    application.statusPage.versionInformationCMS(upg.code);//TODO: Should show the version instead?
									}else{
										application.statusPage.versionInformationCMS(upg.id);
									}

									if (localStorage.hasOwnProperty('version_instructions')){
										var myUpg = JSON.parse(localStorage.version_instructions);

										if (upg.id !== myUpg.id){
											//if (AppVersionInformation.isDevVersion){
												LayoutManager.toast(function(){},function(){},"Application needs upgrade (ID mismatch L: "+myUpg.id+" - R:"+upg.id+")!");
											//}
											application.applicationUpgrade = true;
										}
									}else{
										if (AppVersionInformation.clientCode != upg.code){
											//if (AppVersionInformation.isDevVersion){
												LayoutManager.toast(function(){},function(){},"Application needs upgrade (No previous ID R: "+upg.id+")!");
											//}
											application.applicationUpgrade = true;
										}
									}
                  localStorage.version_instructions = connectResponse.version_instructions;

									var inUpdateWindow=true;
									if (DeviceManager.Config.getOptions().hasOwnProperty("updateStartWindow") &&
											DeviceManager.Config.getOptions().hasOwnProperty("updateEndWindow")){
									  var dlwindow = new Date();
									  dlwindow.setSeconds(0);
									  var s = DeviceManager.Config.getOptions().updateStartWindow;
									  var e = DeviceManager.Config.getOptions().updateEndWindow;
									  try {
										inUpdateWindow = dlwindow.inDateTimeRange(s,e);
									  } catch (e) {
										Logger.cmsLog({category:"error", message:e.errorText, type:"error", method:"connectToCMS", thread:"Schedule Manager"});
									  }
									}

									if (application.applicationUpgrade && inUpdateWindow && !AppVersionInformation.preventUpdate){
										console.info("Upgrade application!", upg);
										application.statusPage.scheduleStatus("Upgrade application.");
										DeviceManager.Device.upgradeApplication(function(){
											console.debug("Application upgrading!");

											var lsoptions = JSON.stringify(localStorage);
											var saveOptions = {
												path : "upgrade_settings.dat",
												size : lsoptions.length,
												data : lsoptions
											};
											DeviceManager.FileSystem.saveFile(function(){
												/* Restart the application */
												setTimeout(function(){
													DeviceManager.Device.powerCommand("reboot");
												}, AppVersionInformation.isDevVersion ? 15000 : 5000);
											},function(){
												/* Restart the application */
												setTimeout(function(){
													DeviceManager.Device.powerCommand("reboot");
												}, AppVersionInformation.isDevVersion ? 15000 : 5000);
											}, saveOptions);

										}, function(cbObject){
											Logger.cmsLog({category:"audit", message:cbObject.errorText, type:"info", method:"connectCMSResponse", thread:"Schedule Manager"});
										}, upg.url);
									}

								}else{
									Logger.cmsLog({category:"audit", message:'Not compatible version upgrade. ('+connectResponse.version_instructions+')', type:"Audit", method:"connectToCMS", thread:"Schedule Manager"});
								}
							}
						} catch (e) {
							//Logger.cmsLog({category:"error", message:e, type:"error", method:"connectToCMS", thread:"Schedule Manager"});
							// XXX: make a suggestion for a proper error code for this case
							Logger.cmsFault(undefined, undefined,
								{
									code:ERRORS.FaultCodes.RemoteResourceFailed,
									reason: e,
									method:"connectToCMS",
									thread:"Schedule Manager",
									date:new Date(),
									expires: new Date((new Date()).valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000),
								}
							);
						}
					}

					// Fix for XLG-63 : Request screenshot without working XMR issue
					if (DeviceManager.Config.getOptions().hasOwnProperty("screenShotRequested"))
					{
						if (DeviceManager.Config.getOptions().screenShotRequested === "1")
						{
							ScheduleManager.Commands.takeScreenshot();
						}
					}

					requestScheduleFromCMS(successCb,nextStep, connectResponse);
				}else if (connectResponse.status === "1") /* Monitor added */
				{
					v_runningSchedule = false;
					console.debug("Monitor added!");
					LayoutManager.showSettingsScreen(connectToCMS);
					LayoutManager.toast(null,null,connectResponse.message);
					// Why is this here? Also commented in webOS. 2018-10-23
//					if (failureCb){
//						failureCb();
//					}

				} else if (connectResponse.status === "2") /* Monitor waiting approval */
				{
					v_runningSchedule = false;
					console.debug("Monitor waiting approval!");
					LayoutManager.showSettingsScreen(connectToCMS);
					LayoutManager.toast(null,null,connectResponse.message);
					// Why is this here? Also commented in webOS. 2018-10-23
//					if (failureCb){
//						failureCb();
//					}
				}
			} else if (connectResponse.hasOwnProperty("errorCode")){ /* Error occured connecting to CMS */
				if (connectResponse.errorCode === ERRORS.WRONGSERVERKEYCODE) /* Wrong key */
				{
					v_runningSchedule = false;
					console.debug("Wrong CMS key!");
					LayoutManager.showSettingsScreen(connectToCMS);
					LayoutManager.toast(null,null,connectResponse.errorText);
					if (failureCb){
						failureCb(connectResponse);
					}
				}else{
					console.debug("Connect response error", connectResponse);
					if (failureCb){
						failureCb(connectResponse);
					}
				}
			}else{
				console.warn("CMS responded in unusual fasion.",connectResponse);
				if (failureCb){
					failureCb(connectResponse);
				}
			}
		};

		var collectIntervalLogic = function(successCb, failureCb){
			console.log("Scheduler Collect Interval logic fired!");

	        if (LayoutManager.isSettingsScreenOpen()){
	            console.debug("collectInterval:User has opened settings screen! Quit!");
	            return;
	        }

			var connectFailedCb = function(){
				failedToConnectToCMS();
				if (!playLocalContent()){
					if (failureCb){
						failureCb();
					}
				}
				v_collectIntervalRunningState = false;
			};

			/* Schedule Manager should handle the connection */

			CommunicationManager.ConnectCMS(
					function(s,f,response){
						if (v_collectIntervalRunningState){
							console.debug("Scheduler Collect Interval is already running! Quit!");
							return;
						}
						v_collectIntervalRunningState = true;
						connectCMSResponse(successCb, failureCb, response);
					},
					function(obj){
						if (v_collectIntervalRunningState){
							console.debug("Connect to CMS failed but Collect Interval is already running! Quit!");
							return;
						}
						v_collectIntervalRunningState = true;
						connectFailedCb(obj);
					},
					{URL:DeviceManager.Config.getURL()}
			);
		};


		var validateDependents = function(){
			var data;
			//XLG-74 : Changing between CMS instances is a "hit-and-miss" activity
			try {
	            data = ScheduleManager.getScheduleData().dependents;
	        } catch (e) {
	            return 0;
	        }

			var dependentsValid = 1;
			for (var i = 0, max = data.length; i < max; i++) {
				//jshint -W016
				dependentsValid &= LibraryManager.isFileOk(data[i]);
				//jshint +W016
			}
			console.debug("Dependents are valid:" + dependentsValid);
			return dependentsValid;
		};

		var validateDefault = function(){
			var data;
			var defaultValid = 1;

			//XLG-74 : Changing between CMS instances is a "hit-and-miss" activity
			try {
	            data = ScheduleManager.getScheduleData().default;
	        } catch (e) {
	            return 0;
	        }
			if (data.hasOwnProperty("file")){
				//jshint -W016
				defaultValid &= LibraryManager.isFileOk(data.file + ".xlf");
				//jshint +W016
			}else{
				defaultValid = 0;
			}
			if (data.hasOwnProperty("files")){
				for (var i = 0, max = data.files.length; i < max; i++) {
					//jshint -W016
					defaultValid &= LibraryManager.isFileOk(data.files[i]);
					//jshint +W016
				}
			}
			console.debug("Default files are valid:" + defaultValid);

			  var ldef = undefined;
			  for (var i = 0; i < application.statusPage.allLayouts().length; i++) {
				if (application.statusPage.allLayouts()[i].id == data.file){
					ldef = application.statusPage.allLayouts()[i];
					break;
				}
			  }

			  if (ldef){
				ldef.isValid = defaultValid===1;
			  }else{
				application.statusPage.allLayouts.remove(function(e){
				  return e.type==="default";
				});
				application.statusPage.allLayouts.push({id:data.file, type:"default", isValid:defaultValid===1});
			  }



			  //application.statusPage.allLayouts(data.file+"* (Default)");
	  //        if (defaultValid === 1){
	  //          if (LayoutManager.invalidLayoutIds.indexOf(data.file)===-1){
	  ////            application.statusPage.validLayouts_default(data.file+"* (Default)");
	  ////            application.statusPage.invalidLayouts_default("");
	  //          }else{
	  ////            application.statusPage.validLayouts_default("");
	  ////            application.statusPage.invalidLayouts_default(data.file+"* (Default)");
	  //          }
	  //        }else{
	  ////            application.statusPage.validLayouts_default("");
	  ////            application.statusPage.invalidLayouts_default(data.file+"* (Default)");
	  //        }

            // application.statusPage.allLayoutsSet.push(
        	// 	application.statusPage.createLayoutInfo(data.file,true,defaultValid,false,false)
    		// );
	        data.isValid=defaultValid;
			return defaultValid;
		};

		var validateLayout = function(layoutId)
		{
			var layouts;
			var layout;
			var layoutValid = 1;
			var i, maxi;

			try {
	            layouts = ScheduleManager.getScheduleData().layouts;
	        } catch (e) {
	            return 0;
	        }

			for (i = 0, maxi = layouts.length; i < maxi; i++) {
				if (layouts[i].hasOwnProperty("file")){
					if(layouts[i].file === layoutId.toString()){
						layout = layouts[i];
						break;
					}
				}
			}
			if(!defined(layout)){
				return false;
			}

			//jshint -W016
			layoutValid &= LibraryManager.isFileOk(layout.file + ".xlf");
			//jshint +W016

			for (i = 0, maxi = layout.files.length; i < maxi; i++) {
				//jshint -W016
				layoutValid &= LibraryManager.isFileOk(layout.files[i]);
				//jshint +W016
			}

			console.debug("Layout "+layout.file+" files are valid:" + layoutValid);
			return layoutValid;
		};

		/**
		 * @version 2.0.200
		 * @since 1.8.1
		 */
		var validateLayouts = function()
		{
			var data;
			//XLG-74 : Changing between CMS instances is a "hit-and-miss" activity
			  try {
	            data = ScheduleManager.getScheduleData().layouts;
	        } catch (e) {
				application.statusPage.allLayouts.remove(function(itm){
					return itm.type === undefined || itm.type === "normal";
				});
	            return {valid:[], invalid:[]};
	        }

	        var layoutValid = 1;
			var validLayouts = [];
			var invalidLayouts = [];
	        //application.statusPage.validLayouts_schedule.removeAll();
	        //application.statusPage.invalidLayouts_schedule.removeAll();

			for (var j = 0, jmax = data.length; j < jmax; j++) {
				layoutValid = validateLayout(data[j].file);
				if (layoutValid === 1){
					validLayouts.push(data[j]);
					data[j].isValid = true;
	                // if (LayoutManager.invalidLayoutIds.indexOf(data[j].file)===-1){
	                // 	data[j].isValid = true;
	                // 	application.statusPage.validLayouts_schedule.push(data[j].file);
	                // }else{
	                // 	application.statusPage.invalidLayouts_schedule.push(data[j].file);
	                // }
				}else{
					invalidLayouts.push(data[j]);
					data[j].isValid = false;
	                //application.statusPage.invalidLayouts_schedule.push(data[j].file);
				}
				// application.statusPage.allLayoutsSet.push(
	        	// 	application.statusPage.createLayoutInfo(data[j].file,false,layoutValid,false,false)
				// );
				var lobj = undefined;
				for (var i = 0; i < application.statusPage.allLayouts().length; i++) {
				  if (application.statusPage.allLayouts()[i].id == data[j].file){
					lobj = application.statusPage.allLayouts()[i];
					break;
				  }
				}

				if (lobj){
					lobj.isValid = data[j].isValid;
				}else{
					var layoutType = (data[j].shareOfVoice && data[j].shareOfVoice> 0 )?"interrupt":"normal";
					application.statusPage.allLayouts.push({
						id:data[j].file,
						type:layoutType,
						isValid:data[j].isValid
					});
				}
			}

			(application.statusPage.allLayouts()).map(
				function(i){
				  if (i.type === "interrupt" || i.type === "normal"){

					var valid = undefined;
					for (var k1 = 0; k1 < validLayouts.length; k1++) {
					  if (i.id == validLayouts[k1].file){
						valid = validLayouts[k1];
						break;
					  }
					}

					if (!valid){
					  var invalid = undefined;
					  for (var k2 = 0; k1 < invalidLayouts.length; k2++) {
						if (i.id == invalidLayouts[k2].file){
						  invalid = invalidLayouts[k2];
						  break;
						}
					  }
					  if (!invalid){
						application.statusPage.allLayouts.remove(i);
					  }
					}
				  }
				});

			return {valid:validLayouts, invalid:invalidLayouts};
		};

		var validateOverlay = function(overlayId)
		{
			var layouts;
			var layout;

			//XLG-74 : Changing between CMS instances is a "hit-and-miss" activity
			try {
				layouts = ScheduleManager.getScheduleData().overlays;
			} catch (e) {
				return 0;
			}

			for (var i = 0, max = layouts.length; i < max; i++) {
				if (layouts[i].hasOwnProperty("file")){
					if(layouts[i].file === overlayId.toString()){
						layout = layouts[i];
						break;
					}
				}
			}
			if(!defined(layout)){
				return false;
			}

			var layoutValid = LibraryManager.isFileOk(layout.file + ".xlf");

			return layoutValid;
		};

		var validateOverlays = function()
		{
			var data;

			//XLG-74 : Changing between CMS instances is a "hit-and-miss" activity
			try {
				data = ScheduleManager.getScheduleData().overlays;
			} catch (e) {
				application.statusPage.allLayouts.remove(function(itm){
					return itm.type === undefined || itm.type === "overlay";
				});
				return 0;
			}

			var overlayValid = 1;
			var curValid = 0;
			for (var j = 0, jmax = data.length; j < jmax; j++) {
				curValid = validateOverlay(data[j].file);
				overlayValid &= curValid;

				var lobj = undefined;
				for (var i = 0; i < application.statusPage.allLayouts().length; i++) {
					if (application.statusPage.allLayouts()[i].id == data[j].file){
						lobj = application.statusPage.allLayouts()[i];
						break;
					}
				}

				if (lobj){
					lobj.isValid = curValid;
				}else{
					application.statusPage.allLayouts.push({
						id:data[j].file,
						type:"overlay",
						isValid:curValid
					});
				}

				console.debug("Overlay layout "+data[j].file+" file is valid:" + curValid);
			}
			console.debug("Overlay layouts are valid:" + overlayValid);
			return overlayValid;
		};

		/**
		 * Validates the content
		 */
		var validateContent = function(){
//			scheduleParts['default'] = {};
//			scheduleParts['layouts'] = [];
//			scheduleParts['dependents'] = [];
//			scheduleParts['commands'] = [];
//			scheduleParts['overlays'] = [];

			/* First validate dependents */
			var dependentsValid = validateDependents();

			/* Validate default */
			var defaultValid = validateDefault();


			/* Validate layouts */
			var layoutsValid = (validateLayouts().valid.length > 0)*1;

			/* Validate overlays (overlays) */
			var overlayValid = validateOverlays();

			var ret = dependentsValid & (defaultValid | layoutsValid);// &
																		// overlayValid;

			/* Validate commands (commands) */

			return ret;
		};

		var parseRequiredFiles = function(successCb, failureCb, files){
			//WebOSManager.FileSystem.listFiles();
			var options = {};
			options.reqFiles = files;
			options.callback = function(file){

//				/* Inform the media inventory */
				//console.debug("*-* Downloaded:", file);
				LibraryManager.updateFile(file);
			};

			LibraryManager.setRequiredFiles(files);
			application.statusPage.totalFiles(files.length);

			var addFileIdx = 0;

			var downloadMissing = function(successCb, failureCb)  {
				var libFiles = LibraryManager.getLibraryFiles();
				var dlFiles = 0;
				var miInterval = 0;

				if (DeviceManager.Config.getOptions().hasOwnProperty("mediaInventoryTimer"))
				{
					miInterval = DeviceManager.Config.getOptions().mediaInventoryTimer * 60000;
				}


				if (miInterval !== 0){
					var v_sendMediaInventoryTimer = setInterval(function(){
						CommunicationManager.sendMediaInventory(function(){
							console.debug("Send Media inventory (timed)!");
						},function(cbSMIObj){
							console.debug("Fail to send Media inventory (timed)!",cbSMIObj);
						});
					}, miInterval);
				}
				var download  = function(successCb, failureCb){
					if (dlFiles<libFiles.length){
						if (!libFiles[dlFiles].isCorrect){
							var showDownloadingFileId = libFiles[dlFiles].filename.startsWith(libFiles[dlFiles].id) ? 'ID: ' + libFiles[dlFiles].id : '';
                        	application.statusPage.downloadFileId(showDownloadingFileId);
                        	application.statusPage.downloadFileId.notifySubscribers();
							libFiles[dlFiles].download(function(){
								// File downloaded
								application.statusPage.downloadFileId('');
								application.statusPage.downloadedFiles(dlFiles);
								application.statusPage.remainingFiles(libFiles.length - dlFiles);
								console.debug("File "+libFiles[dlFiles].filename+" Downloaded");
								dlFiles++;
								download(successCb, failureCb);
							},function(cbObject){
								//console.debug("File download failed!");
								application.statusPage.downloadFileId('');
								application.statusPage.downloadFileId.notifySubscribers();
	                            Logger.cmsLog({category:"error", message: "Download failed for file "+libFiles[dlFiles].filename + ". Error:"+JSON.stringify(cbObject), type:"ERROR", method:"download", thread:"parseRequiredFiles"});
								dlFiles++;
								download(successCb, failureCb);
							});
						}else{
							dlFiles++;
							download(successCb, failureCb);
						}
					}else{ // Finish downloading
						// Exit recursive
						if (defined(v_sendMediaInventoryTimer)){ clearInterval(v_sendMediaInventoryTimer); }
						application.statusPage.downloadedFiles(files.length);
						application.statusPage.remainingFiles(0);
						CommunicationManager.sendMediaInventory(function(){
							console.debug("Send Media inventory (finish)!");
							successCb();
						},function(cbSMIObj){
							console.debug("Fail to send Media inventory (finish)!",cbSMIObj);
							successCb();
						});
					}

				};
				CommunicationManager.sendMediaInventory(function(){
					console.debug("Send Media inventory (start)!");
					download(successCb, failureCb);
				},function(cbSMIObj){
					console.debug("Fail to send Media inventory (start)!", cbSMIObj);
					download(successCb, failureCb);
				});
			};

			var addFilesToLib = function(successCb, failureCb, files){

				if (addFileIdx < files.length){
					if (files[addFileIdx].type === "media" ||
							files[addFileIdx].type === "layout" ||
							files[addFileIdx].type === "resource"){
						LibraryManager.addFile(function(){
							addFileIdx++;
							addFilesToLib(successCb, failureCb, files);
						},function(cbObject){
							failureCb(cbObject);
						},files[addFileIdx]);
					}else{
						addFileIdx++;
						addFilesToLib(successCb, failureCb, files);
					}
				}else{
					/* Clean files that are not needed */
					LibraryManager.cleanUpLibrary(function(removed){
						if (removed){
	                        // Remove overlay if exists
	                        var ovl = [];
	                        for (var i = 0, max = v_CurrentOverlayLayouts.length; i < max; i++) {
	                          if (!validateOverlay(v_CurrentOverlayLayouts[i].layoutId)){
	                            ovl.push({index:i,layout:v_CurrentOverlayLayouts[i]});
	                          }
	                        }
	                        for (var j = 0, maxj = ovl.length; j < maxj; j++) {
	                            LayoutManager.removeOverlay(ovl[j].layout);
	                            v_CurrentOverlayLayouts.splice(ovl[j].index,1);
                            	//application.statusPage.scheduledLayouts_overlay.remove(ovl[j].layout.layoutId+" (O)");
								application.statusPage.allLayouts.remove(function(olitem){
									return olitem.id==ovl[j].layout.layoutId;
								});
							  	//scheduledLayouts_overlay.remove(ovl[j].layout.layoutId+" (O)");
	                        }
							var curId = LayoutManager.getCurrentLayoutId();
							if (curId !== "splash"){
								// Check if current layout is valid.
								if (!validateLayout(curId)){
									// Check if default layout is valid.
									if (validateDefault()){
										console.debug("Scheduler: Display default layout!");
										var defLayout = {layoutId:ScheduleManager.getScheduleData()['default'].file,
												scheduleId:"0"};
										LayoutManager.setLayouts([defLayout]);
									}else{
										// play splash
										console.debug("Scheduler: Display splash layout!");
										LayoutManager.setLayouts([]);
	                                    /* Overlay layout not Updating */
	                                    // Remove all overlay layouts
	                                    ScheduleManager.setOverlayLayouts();
									}
								}
							}
						}
						successCb(); /* All files checked */
					},
					failureCb);
				}
			};

			addFilesToLib(function(){
				downloadMissing(function(cbObj){
					LibraryManager.saveLibrary();
					successCb(cbObj);
				},failureCb, options
				);
			}, failureCb,files);

		};


		/**
		 * Parse XML document from web service to get the schedule object.
		 * @param {String} data Raw XML Document fom web service.
		 * @return {undefined} undefined
		 */
		var parseSchedule = function(data){
			var i, j, maxi, maxj;

			var parser = new DOMParser();
			var xmlDoc = parser.parseFromString(data, "text/xml");

			var reqParentxml = xmlDoc.childNodes[0];

			/* Version 4 */
			/*
        for (var i = 0; i < reqParentxml.childElementCount; i++) {
            var x = reqParentxml.children[i];
            var reqFile = {type: x.localName};
            for (var j = 0; j < x.attributes.length; j++) {
                reqFile[x.attributes[j].name] = x.attributes[j].value;
            }
            reqChilds[i] = reqFile;
        } 
			 */

			/* Version 5 */
			var scheduleParts = parseXMLToObject(reqParentxml);

			scheduleParts.default = {};
			scheduleParts.layouts = [];
			scheduleParts.dependents = [];
			scheduleParts.commands = [];
			scheduleParts.overlays = [];


			var currDatetime = utils.getTimeStamp();
			for (i = 0, maxi = reqParentxml.childElementCount; i < maxi ; i++) {
				var x = reqParentxml.children[i];
				switch (x.nodeName) {
				case "default":
					scheduleParts.default = parseXMLToObject(x);

                    if ((!scheduleParts.default.hasOwnProperty('file')) ||
                            scheduleParts.default.file === "0"){
						scheduleParts.default = {};
						Logger.cmsLog({
						    category:"audit",
						message: "No default layout set in CMS.",
						method: "parseSchedule",
						thread: "ScheduleManager"
						});
                    }else if (x.children.length>0){
	                	if (x.children[0].children.length>0)
						{
							scheduleParts.default.files = [];
							for (j = 0, maxj = x.children[0].children.length; j < maxj; j++) {
								scheduleParts.default.files.push(x.children[0].children[j].textContent);
							}
						}
					}
					break;
					case "layout":
						var layout = parseXMLToObject(x);
						layout.files = [];
						if (x.children.length>0)
						{
							if (x.children[0].children.length>0)
							{
								for (j = 0, maxj = x.children[0].children.length; j < maxj; j++) {
									layout.files.push(x.children[0].children[j].textContent);
								}
							}
						}
						scheduleParts.layouts.push(layout);
						break;
					case "dependants":
						/* We have a list of files */
						for (j = 0, maxj = x.children.length; j < maxj; j++) {
							//scheduleParts['dependents'].push(parseXMLToObject(x.children[j]));
							scheduleParts.dependents.push(x.children[j].textContent);
						}
						break;
					case "command":
						//<command date=â€‹"2017-04-25 12:â€‹00:â€‹00" scheduleid=â€‹"6" code=â€‹"displayoff">â€‹</command>â€‹
						var command = parseXMLToObject(x);
						command.timeStamp = utils.parseDate(command.date);
						if (currDatetime<command.timeStamp){
							/* Check if command exists. */
							var exists = false;
							for (j = 0, maxj = scheduleParts.commands.length; j < maxj; j++) {
								if (command.date === scheduleParts.commands[j].date){
									if (command.code === scheduleParts.commands[j].code){
										exists = true;
									}
								}
							}
							if (!exists){
								scheduleParts.commands.push(command);
							}
						}
						break;
					case "overlays":
						for (j = 0, maxj = x.children.length; j < maxj; j++) {
							var overlay = parseXMLToObject(x.children[j]);
							overlay.fromTimeStamp = utils.parseDate(overlay.fromdt);
							overlay.toTimeStamp = utils.parseDate(overlay.todt);

							if (currDatetime<overlay.toTimeStamp){
								scheduleParts.overlays.push(overlay);
							}
						}
						break;
					default:
						console.debug(x);
					break;
				}
			}

			v_scheduleData = scheduleParts;
			// CacheManager.checkCampaignOrder(v_scheduleData.layouts)
			return scheduleParts;
		};

		var parseXMLToObject = function(data){
			var obj = {};
			obj.name = data.nodeName;
//			if (data.hasOwnProperty("textContent"))
//			{
//			obj.value = data.nodeName;
//			}

			if (data.attributes.length>0){
				parseXMLAttributesToObjects(obj,data.attributes);
			}else if (data.hasOwnProperty("textContent")){
				if (data.textContent[0] !== '\n'){
					obj.value = data.textContent;
				}
			}

//			if (data.children.length>0){
//			parseXMLChildsToObjects(obj,data.children);
//			}

			return obj;
		};

		var parseXMLAttributesToObjects = function(object, data){
			for (var i = 0, max = data.length; i < max; i++) {
				object[data[i].nodeName] = data[i].value;
			}
		};

		var parseXMLChildsToObjects = function(object, data){
			for (var i = 0, max = data.length; i < max; i++) {
				var obj = {};
				obj.name = data[i].nodeName;
				if (data[i].attributes.length>0){
					parseXMLAttributesToObjects(obj,data[i].attributes);
				}

				if (data[i].children.length>0){
					parseXMLChildsToObjects(obj,data[i].children);
				}

				object[obj.name] = obj;
			}
		};

		/**
		 * Play cached content.
		 * @returns {Boolean} True if played else false.
		 */
		 var playLocalContent = function(){
			 if (ScheduleManager.loadCachedSchedule()){
				 contentValid = validateContent();
				 if (contentValid){
					 console.info("Local schedule is OK. Continue");
					 /*
					  * That means:
					  * Library is validated.
					  * Schedule is known.
					  */

					 v_runningSchedule = true; // Fix for XLG-39!
					 informLayoutWithSchedule();
					 DeviceManager.Device.collectRealTimeDeviceInfo(null,logic.nonFatalError);
					 return true;
				 }else{
					 console.info("Scheduler: Local schedule is invalid!");
					 if (!v_runningSchedule){
						 console.info("Scheduler: Display slash layout");
						 LayoutManager.setLayouts([]);
						 v_runningSchedule = true; // Fix for XLG-39!
						 informLayoutWithSchedule();
						 return true;
				 	 }
				 }
			 }else{
				 console.info("Scheduler: No local schedule!");
				 if (!v_runningSchedule){
					 console.info("Scheduler: Display slash layout");
					 LayoutManager.setLayouts([]);
					 v_runningSchedule = true; // Fix for XLG-39!
					 informLayoutWithSchedule();
					 return true;
			 	 }
			 }
			 return false;
		 };

		 var informLayoutWithSchedule = function()
		 {
			 console.debug("Check for schedule changes!");

			 var i, maxi;
			 if (v_scheduleData !== null){
				 if (v_runningSchedule){
					 var layouts = ScheduleManager.getCurrentLayouts();
					 //If we still download content don't set layouts.
	                 if (!v_collectIntervalRunningState){

						 if (contentValid){
							 /* Set Schedule */
							 LayoutManager.setLayouts(layouts);
							 setTimeout(function(){
								 /* Set overlay layouts */
								 ScheduleManager.setOverlayLayouts();
							 },0);
						 }else{
							 console.debug("Content not valid to set layouts");
						 }
	                 }

					 /* Run Commands */
					 var currDateTime = utils.getTimeStamp();
					 var commands = ScheduleManager.getScheduleData().commands;
					 for (i = 0, maxi = commands.length; i < maxi; i++) {
						 if ((commands[i].timeStamp-60) <= currDateTime &&
								 currDateTime <= commands[i].timeStamp)
						 {
							 console.debug("Run the command "+commands[i].code+" !");
							 commands[i].delete = true;
							 ScheduleManager.Commands.commandAction(commands[i].code);
						 }else if (currDateTime > commands[i].timeStamp){ /* Command is old. Remove it*/
							 commands[i].delete = true;
						 }
					 }

					 for (i = commands.length-1; i >= 0; --i) {
						 if (commands[i].hasOwnProperty('delete'))
						 {
							 commands.splice(i, 1);
						 }
					 }
				 }
			 }

			 setTimeout(informLayoutWithSchedule,10000);
		 };


		/**
		 * Get the layouts that are to be played
		 * @param {Array} layouts Array of layouts to spread
		 * @return {Array} description
		 */
		var resolveNormalAndInterrupts = function(layouts){


			var normal = [];
			var interrupt = [];

			layouts.forEach(function(l){
			l.durationCommitted = 0;
			if (l.shareOfVoice > 0){
				interrupt.push(l);
			}else{
				normal.push(l);
			}
			});

			var resolved = [];
			var resolvedNormal = [];
			var resolvedInterrupt = [];

			var index = 0;
			var interruptSecondsInHour = 0;

			var duration;

			var isDurationSatisfied = function(itm){
			return itm.durationCommitted >= itm.shareOfVoice;
			};

			lblMainLoop:
			while (true) {
			if (index >= interrupt.length)
			{
				index = 0;
	//          for (var i in interrupt) {
	//            if (interrupt[i].durationCommitted >= interrupt[i].shareOfVoice){
	//              break lblMainLoop;
	//            }
	//          }
				if (interrupt.every(isDurationSatisfied)) {
				break lblMainLoop;
				}
			}

			var item = interrupt[index];
			if (!(interrupt[index].durationCommitted >= interrupt[index].shareOfVoice))
			{
				duration = Number.parseInt(item.duration);

				item.durationCommitted += duration;
				interruptSecondsInHour += duration;
				resolvedInterrupt.push(item);
			}

			index++;

			}

			var normalSecondsInHour = 3600 - interruptSecondsInHour;
			index = 0;

			while (normalSecondsInHour > 0)
			{
			if (index >= normal.length)
			{
				index = 0;
			}

			item = normal[index];
			duration = Number.parseInt(item.duration);
			normalSecondsInHour -= duration;
			resolvedNormal.push(item);

			index++;
			}

			var pickCount = Math.max(resolvedNormal.length, resolvedInterrupt.length);
			var normalPick = Math.floor(1.0 * pickCount / resolvedNormal.length)|0;
			var interruptPick = Math.floor(1.0 * pickCount / resolvedInterrupt.length)|0;
			var normalIndex = 0;
			var interruptIndex = 0;

			// Pick as many times as we need to consume the larger list
			for (var i = 0; i < pickCount; i++)
			{
			// We can overpick from the normal list
			if (i % normalPick === 0)
			{
				if (normalIndex >= resolvedNormal.length)
				{
				normalIndex = 0;
				}
				resolved.push(resolvedNormal[normalIndex]);
				normalIndex++;
			}

			// We can't overpick from the interrupt list
			if (i % interruptPick === 0 && interruptIndex < resolvedInterrupt.length)
			{
				resolved.push(resolvedInterrupt[interruptIndex]);
				interruptIndex++;
			}
			}

			return resolved;
		};

		 var nextStep = function(){
			 console.warn("What to do next;");
			 v_collectIntervalRunningState = false;
		 };

		 // </editor-fold>

		 /* +------------------------+ Public functions +------------------------+ */
		 // <editor-fold defaultstate="collapsed" desc="Public functions">

		 this.start = function()
		 {
			 /**
			  * Callback function if CMS configuration exists.
			  * @param {boolean} cbObj True if exists, false if not.
			  */
			 function cbCmsConfExists(cbObj) {

				 var validSettings = false;
				 if (cbObj === true)
				 {
					 //Load Configuration
					 DeviceManager.Config.loadConfiguration();

					 //Check if it is valid
					 validSettings = DeviceManager.Config.settingsValid();
				 }

				 if (validSettings === false || (!DeviceManager.Config.isCMSValid())) {
					 //Ask user for settings
					 console.debug("logic.showCMSParameterEntry();");
					 LayoutManager.showSettingsScreen(connectToCMS);
				 } else {

					 var options = DeviceManager.Config.getOptions();
					 application.statusPage.cmsAddr(DeviceManager.Config.getURL());
					 application.statusPage.displayName(options.displayName);
					 application.statusPage.screenShotInterval(options.screenShotRequestInterval);
					 application.statusPage.downloadWindowStart(options.downloadStartWindow);
					 application.statusPage.downloadWindowEnd(options.downloadEndWindow);
                     application.statusPage.updateWindowStart(options.updateWindowStart);
					 application.statusPage.updateWindowEnd(options.updateEndWindow);
					 application.statusPage.downloadWindowStart.notifySubscribers();
					 application.statusPage.downloadWindowEnd.notifySubscribers();
                     application.setMenuHideTimeout(options.actionBarDisplayDuration);
					 application.setMenuInitialState(options.actionBarMode);
					 if (options.actionBarMode === "0"){ //Menu is hidden
						 application.hideAppMenu();
					 }else{
						 application.showAppMenu();
					 }


					 if (DeviceManager.Device.getOrientation()!== options.orientation){
						 DeviceManager.Device.setNativePortraitMode(function(){
	                       LayoutManager.orientationChanged();
						 },logic.nonFatalError, options.orientation);
					 }

					 CommunicationManager.init(function () {
						 ScheduleManager.startInterval();
					 }, errorCB);
				 }
			 }

			 DeviceManager.Config.configurationExists(cbCmsConfExists, errorCB);
		 };

		 this.startInterval = function() {
			 /*
			  * Collect interval for requesting staff from the CMS.
			  * @type Number
			  * @sa Default value is 1 hour.
			  */
			 var collectInterval = 60 * 60 * 1000;

			 if (DeviceManager.Config.getOptions().hasOwnProperty('collectInterval'))
			 {
				 collectInterval = DeviceManager.Config.getOptions().collectInterval * 1000;
			 }

			 /* FOR DEVELOPMENT ONLY - CHECK TO CMS INTERVAL */
			 //collectInterval = 1000*60*1;
			 /* FOR DEVELOPMENT ONLY - CHECK TO CMS INTERVAL */

			 /* Set the timer to check to the CMS and update staff! */
			 var intenalCollectIntervalHandler = function(){
				 if (DeviceManager.Config.getOptions().hasOwnProperty('collectInterval'))
				 {
					 collectInterval = DeviceManager.Config.getOptions().collectInterval * 1000;
				 }

				 v_collectIntervalTimer = setTimeout(function(){
					 intenalCollectIntervalHandler();
				 }, collectInterval);
				 console.debug("Fired collect interval timer");
				 collectIntervalLogic();
			 };
			 intenalCollectIntervalHandler();

			 /* Set a new timer to run every minute for the schedule */
			 setTimeout(function(){
				 informLayoutWithSchedule();
			 }, 60000);

		 };

		 /**
		  * Returns schedule data as loaded from remote!
		  * @returns {Object} Schedule data.
		  * @version 1.8.1
		  * @since 1.8.1
		  */
		 this.getScheduleData = function(){
			 return v_scheduleData;
		 };

		 /**
		  * Detects which layouts are scheduled to play and return them as an array.
		  * @returns {Array} Returns array of scheduled layouts.
		  * @see The array is based on the current time.
		  * @version 1.8.1
		  * @since 1.8.1
		  */
		 this.getCurrentLayouts = function(){
			 var i, maxi, prio, l;
			 var schData = ScheduleManager.getScheduleData();
			 var layouts = [];

			 var hasInterruptLayouts = false;

			 if (schData === null)
			 {
				application.statusPage.allLayouts([{id:"splash", type:"normal"}]);
				return layouts;
			 }

			 /* Find the maximum priority of the layouts */
			 var maxprio = Number.MIN_SAFE_INTEGER;

			 var timeValidLayouts = [];

			 /* See if layouts are scheduled to play */
			 var cTime = utils.getTimeStamp();

			 //Timecheck
	         for (var i = 0, max = schData.layouts.length; i < max; i++) {
	        	 if (schData.layouts[i].isValid){
	        		 if ((utils.parseDate(schData.layouts[i].fromdt) < cTime) & (utils.parseDate(schData.layouts[i].todt) > cTime)) /* Layout is valid */
	        		 {
	        			 timeValidLayouts.push(schData.layouts[i]);
	        		 }
	        	 }
	         }


	         //Priority check
	         /* Find the maximum priority of the time valid layouts */
	         for (var i = 0, max = timeValidLayouts.length; i < max; i++) {
	             var prio = parseInt(timeValidLayouts[i].priority);
	             if (prio>maxprio){
	                 maxprio = prio;
	             }
	         }
	         for (var j = 0, max = timeValidLayouts.length; j < max; j++) {
	             var prio = parseInt(timeValidLayouts[j].priority);
	             if (prio === maxprio){ /* Play only layouts with the maximum priority */
	                 var l = { layoutId : timeValidLayouts[j].file};
	                 if (timeValidLayouts[j].hasOwnProperty('scheduleid')){
	                     l.scheduleId = timeValidLayouts[j].scheduleid;
					 }
					 l.duration = timeValidLayouts[j].duration ? timeValidLayouts[j].duration : -1;
					 l.shareOfVoice = timeValidLayouts[j].shareOfVoice ? Number.parseInt(timeValidLayouts[j].shareOfVoice) : 0;
					 if (l.shareOfVoice>0){
					   hasInterruptLayouts = true;
					 }
								 var hasCoverMaxPlays = ScheduleManager.isMaxPlaysPerHourThresholdHit(timeValidLayouts[j].file);
								 if (!hasCoverMaxPlays){
									 layouts.push(l);
								 }
	             }
	         }

			 var hasNormal = layouts.some(function(i){
				return i.shareOfVoice==0;
			  });
			  if (layouts.length === 0 || (!hasNormal ))
			  {
				if(schData.default.hasOwnProperty('file') && schData.default.file !== "0" ){
					var l = {
						layoutId : schData.default.file,
						scheduleId : "0",
						duration:CacheManager.getLayoutDuration(schData.default.file, 60)
					};
					layouts.unshift(l);
				}
			 }

			 if (hasInterruptLayouts){
				layouts = resolveNormalAndInterrupts(layouts);
			 }

			 application.statusPage.setScheduledLayouts(layouts);
			 return layouts;
		 };


		 /**
		  * Load schedule from cache if exists.
		  * @returns {Boolean} True if schedule exists else false.
		  * @version 1.8.1
		  * @since 1.8.1
		  */
		 this.loadCachedSchedule = function(){
			 if (localStorage.hasOwnProperty("schedule")){
				 v_scheduleData = JSON.parse(localStorage.schedule);
				 return true;
			 }
			 return false;
		 };

		 /**
		  * Save schedule to cache for offline usage.
		  * @returns None
		  * @version 1.8.1
		  * @since 1.8.1
		  */
		 this.saveCachedSchedule = function(){
			 localStorage.schedule = JSON.stringify(v_scheduleData);
		 };

		 /**
		  * Detects which overlay layouts are scheduled to play inform Layout manager to display them.
		  * @returns None
		  * @see Also remove overlay layouts where their play to time passed.
		  * @version 1.8.1
		  * @since 1.8.1
		  */
		 this.setOverlayLayouts = function(){
			 console.debug("Check Overlay");

	         /* Only if splash screen is not shown */
	         if (LayoutManager.getCurrentLayoutId() === "splash"){

               for (var i = v_CurrentOverlayLayouts.length - 1; i >= 0; i--) {
	             LayoutManager.removeOverlay(v_CurrentOverlayLayouts[i]);
	           }
	           v_CurrentOverlayLayouts = [];
	           return;
	         }

			 if (v_scheduleData === null){
				application.statusPage.setOverlayLayouts([]);
				return;
			 }

			 var i, j, maxi;
			 var schData = v_scheduleData.overlays;
			 var layouts = [];

			 /* See if layouts are scheduled to play */
			 var cTime = utils.getTimeStamp();

			 for (i = 0, maxi = schData.length; i < maxi; i++) {
				 if ((schData[i].fromTimeStamp < cTime) && (schData[i].toTimeStamp > cTime)) /* Layout is valid */
				 {
						 var libfile = LibraryManager.getFileByName(schData[i].file+".xlf");
						 if (defined(libfile)){
							 var l = { layoutId : schData[i].file, lastUpdated:libfile.lastChecked};
							 if (schData[i].hasOwnProperty('scheduleid')){
							 	 l.scheduleId = schData[i].scheduleid;
							 }
						 layouts.push(l);
						 }
				 }
			 }

			 var cLayoutIdx;
			 /* Remove expired overlay layouts previously showned */
	         /* Also remove overlay layouts that has been updated */
			 for (i = v_CurrentOverlayLayouts.length - 1; i >= 0; i--) {
				 var libfile;
				 cLayoutIdx = -1;
				 for (j = 0; j < layouts.length; j++) {
				   libfile = LibraryManager.getFileByName(schData[i].file+".xlf");
				   if (defined(libfile)){
	                 if (v_CurrentOverlayLayouts[i].lastUpdated !== libfile.lastChecked){
	                     break;
	                 }
					 if(layouts[j].layoutId === v_CurrentOverlayLayouts[i].layoutId){
						 cLayoutIdx = j;
						 break;
					 }
				   }
				 }
				 if (defined(libfile)){
					 if (cLayoutIdx === -1){
						 LayoutManager.removeOverlay(v_CurrentOverlayLayouts[i]);
						 v_CurrentOverlayLayouts.splice(cLayoutIdx,1);
					 }
				 }
			 }

			 /* Show overlay layouts not previously showned */

			 for (j = 0; j < layouts.length; j++) {
				 cLayoutIdx = -1;
				 for (i = 0, maxi = v_CurrentOverlayLayouts.length; i < maxi; i++) {
					 if(layouts[j].layoutId === v_CurrentOverlayLayouts[i].layoutId){
						 cLayoutIdx = j;
						 break;
					 }
				 }

				 if (cLayoutIdx === -1){
					 v_CurrentOverlayLayouts.push(layouts[j]);
					 LayoutManager.showOverlay(layouts[j]);
				 }
			 }

			 application.statusPage.setScheduledOverlayLayouts(layouts);
		 };

		/**
		 * Try connect to the CMS.
		 * @param {Boolean} saveSettings If coming from User parameter view, set it true to save the parameters.
		 * @returns None
		 * @version 1.8.1
		 * @since 1.8.1
		 */
		this.ConnectToCMS = function(saveSettings)
		{
			//Clear the library and schedule here.
			LibraryManager.clearAllFiles();
			v_runningSchedule = false;
			v_scheduleData = null;
			v_localCheckRfCRC = "";
			v_localCheckRfHelperCRC = "";
			v_localCheckScheduleCRC = "";
			v_localCheckScheduleHelperCRC = "";
			LayoutManager.stopPlaying();
			//Also request a new license.
			LicenseManager.checkLicenseServer(function(){
				//console.log("Check license...");
			},function(errObj){
				console.log("Error checking license...",errObj);
			});
			connectToCMS(saveSettings);
		};

		/**
		 * Clear which overlay layout is shown. It is only used when displaying settings screen (app.js).
		 * @returns None
		 * @version 1.8.1
		 * @since 1.8.1
		 */
	    this.userOpenSettingsScreen = function(){
			v_CurrentOverlayLayouts = [];
			v_runningSchedule = false;
		};


		/**
		 * Make the Schedule Manager to revalidate the content. Usually after a media failed to play
		 * @returns None
		 * @version 1.8.1
		 * @since 1.8.1
		 */
		this.revalidateContent = function(){
			validateContent();

		};
		/**
		 * Get the Max Plays Per Hour setting of the layout
		 * @param {string} layoutId The id of the layout
		 * @return {string} The max plays per hour setting of the layout.
		 */
		this.getLayoutMaxPlaysPerHour = function(layoutId){
			return timeValidLayouts[layoutId].maxPlaysPerHour;
		}

		/**
		 * Check if max plays per hour has ovecome is threshold for a specific layout.
		 * @param {string} layoutId The layout id to check for max plays per hour.
		 * @return {boolean} True if it has or false if not!
		 */
		this.isMaxPlaysPerHourThresholdHit = function(layoutId){
			if (!ScheduleManager.getScheduleData()){
				return false;
			}
			var layout = ScheduleManager.getScheduleData().layouts.filter(function(a){return a.file===layoutId})[0];
			if (!layout)
			{
				return false;
			}
			var hasCoverMaxPlays = false;
			//Check if we have played the maximum plays per hour, if applicable
			if (layout.maxPlaysPerHour && layout.maxPlaysPerHour>0){
				// Check if the current layout has passed the max plays per hour.
				var mpph_tgt = layout.maxPlaysPerHour; //ScheduleManager.getLayoutMaxPlaysPerHour(layouts[nIdx].id);
				if (mpph_tgt > 0){
					var mpph_cur = CacheManager.mpph_getPlays(layout.file);
					var curDate = new Date();
					//Check that we are still in the same hour
					if ((mpph_cur.time.getDate() === curDate.getDate()) && (mpph_cur.time.getHours() === curDate.getHours())){
						if (mpph_cur.plays >= mpph_tgt){
							hasCoverMaxPlays = true;
						}
					}else{
						// We aren't so reset the layout mpph data.
						CacheManager.mpph_reset(layout.file);
					}
				}
			}

			return hasCoverMaxPlays;
		};

		// <editor-fold defaultstate="collapsed" desc="Commands">
		this.Commands = new function _Commands(){

			// /**
			//  * Layouts that are shown by changeLayout command.
			//  */
			// var v_changeLayoutLayouts = [];

			/**
			 * Holds schedule layouts from changeLayout
			 */
			var stoppedScheduleLayouts = [];

			/**
			 * Timer for the duration of the changed layout.
			 */
			var changeLayoutTimer;

	        /**
	         * Take a screenshot. It will also send runtime information to CMS
	         * @returns {undefined}
	         * @version 1.8.1
	         * @since 1.8.1
	         */
			this.takeScreenshot = function(){
				var attempts = 0;

				var cbFailure = function(cbObject){
					console.error("Request screenshot error", cbObject);
				};
				/*
	            var bd = document.getElementsByTagName("*");
	            for (var i = 0, max = bd.length; i < max; i++) {
	                        bd[i].classList.add("paused");
	            }
	            var ve = document.getElementsByTagName("video");
	            for (var i = 0, max = ve.length; i < max; i++) {
	                ve[i].pause();
	            }
				 */
				var intTakeScreenshot = function(){
					var ssopt;
					var opts = DeviceManager.Config.getOptions();
					if (opts.hasOwnProperty("screenShotSize")){
						ssopt = opts.screenShotSize;
					}
					DeviceManager.Device.screenShot(
						function(cbObject){
							console.debug("Screen capture succeded");
							var obj = { imagedata: cbObject.data };
							logic.status.setLastCommandStatus("true");
							CommunicationManager.sendScreenshot(function(){
								console.debug("Screenshot send!");
								CommunicationManager.sendInfo(null,cbFailure);
							},cbFailure,obj);
						},function(cbOject){
							if (attempts<100){
								console.warn("Screenshot attemp "+attempts+"/100!");
								attempts++;
								setTimeout(intTakeScreenshot,1000);
							}else{
								cbFailure(cbOject);
							}
						},
						ssopt
					 );
				 };
				 intTakeScreenshot();

				 //Also send the status window
				 CommunicationManager.sendStatusInfo(function(){},function(){});
			 };

			/**
			 * Change the layout.
			 * @param {Number} layoutId The layout Id.
			 * @param {Number} duration The duration to play. 0 means default which is 5.
			 * @param {Boolean} downloadRequired Has to download the resource.
			 * @param { "replace" | "queue" } changeMode The way to change the schedule.<br>
			 * @param { "command" | "touch" | "webhook" } source Who request the switch of the layout.
			 * Replace will immidiately change the layout.<br>
			 * Queue will change the layout after the current layout finish playing.
			 * @returns None
			 */
			this.changeLayout = function(layoutId, duration, downloadRequired, changeMode, source){
				if (contentValid){
					 var f_changeLayout = function(){
						 v_runningSchedule = false;
						 console.debug("Command:Change layout to "+layoutId+" with duration " +duration+ ". Change mode is "+changeMode+". Download required : "+downloadRequired);

						var playNextLayout = function(){
							changeLayoutLayouts.shift();
							if (changeLayoutLayouts.length>0){
								console.debug("Command:Change to layout "+changeLayoutLayouts[0].layoutId+"!");
								LayoutManager.setLayouts(changeLayoutLayouts);
								durationExpired(); // Recursive
							}else{
								console.debug("Command:Change back to normal schedule!");
								v_runningSchedule = true;
								LayoutManager.setLayouts(stoppedScheduleLayouts);
							}
						};

						var shoudlPlayNextlayout = function(evt){
							if (evt.detail.layoutId === layoutId){
								LayoutManager.removeEventListener('layoutEnd', shoudlPlayNextlayout);
								playNextLayout();
							}
						};

						var durationExpired = function(){
							if (source==="touch" || source==="webhook"){
								setTimeout(function(){
								LayoutManager.addEventListener('layoutEnd', shoudlPlayNextlayout);
								},0);
							}else {
								changeLayoutTimer = setTimeout(function(){
									playNextLayout();
								}, changeLayoutLayouts[0].duration*1000);
							}
						};

						 var dryRun = false;
						 if (v_changeLayoutLayouts.length === 0){
							 stoppedScheduleLayouts = ScheduleManager.getCurrentLayouts();
							 dryRun = true;
						 }

						 if (duration*1 === 0){
							 duration = 5;
						 }
						 var newlayout = {layoutId:layoutId+"", duration:duration};

						 if (changeMode === "replace"){
							 clearTimeout(changeLayoutTimer);
							 v_changeLayoutLayouts = [];
							 dryRun = true;
						 }
						 v_changeLayoutLayouts.push(newlayout);

						 if (dryRun){
							 LayoutManager.setLayouts(v_changeLayoutLayouts);
							 durationExpired();
						 }
					 };

					 if (downloadRequired===true){
						 collectIntervalLogic(function(){
							 f_changeLayout();
						 },function(cbError){
							 logic.nonFatalError(cbError);
						 }); /* This has to end before change the layout but it is async. What to do??? */
					 }else{
						 f_changeLayout();
					 }
				 }else{
					 console.debug("Content not valid to run command changeLayout!");
				 }
			};

			 /**
			  * Run collect now.
			  */
			 this.collectNow = function(){
				 console.debug("Run collect!");
				 collectIntervalLogic();
			 };

			 this.commandAction = function(code){

				 var execute = function(cmdString){
					 switch (cmdString) {
					 //<editor-fold defaultstate="collapsed" desc="Power Commands">
					 case "reboot":
						 DeviceManager.Device.powerCommand(DeviceManager.Device.Power.REBOOT);
						 break;
					 case "poweroff":
						 DeviceManager.Device.powerCommand(DeviceManager.Device.Power.SHUTDOWN);
						 break;
					 case "displayoff":
						 DeviceManager.Device.setDisplayMode(
								 function(){
									 logic.status.setLastCommandStatus("true");
									 CommunicationManager.sendInfo(null,logic.nonFatalError);
								 },
								 function(cbError){
									 logic.nonFatalError(cbError);
								 },
								 DeviceManager.Device.Power.DISPLAY_OFF
						 );
						 break;
					 case "displayon":
						 DeviceManager.Device.setDisplayMode(
								 function(){
									 logic.status.setLastCommandStatus("true");
									 CommunicationManager.sendInfo(null,logic.nonFatalError);
								 },
								 function(cbError){
									 logic.nonFatalError(cbError);
								 },
								 DeviceManager.Device.Power.DISPLAY_ON
						 );
						 break;
						 //</editor-fold>
					 default:
						 var cmds = DeviceManager.Config.getCommands();
					 if (cmds.hasOwnProperty(code)){
						 logic.status.setLastCommandStatus("false");
						 console.debug("Command "+code+" found!",
								 {method:"commandAction", execute:cmds[code].commandString, validate:cmds[code].validationString} );
					 }else{
						 logic.status.setLastCommandStatus("false");
						 console.warn("requestCommandAction "+code+" not understood!!!");
					 }
					 break;
					 }
				 };

				 var cmds = DeviceManager.Config.getCommands();
				 if (cmds.hasOwnProperty(code)){
					 console.debug("Command "+code+" found!",
							 {method:"commandAction", execute:cmds[code].commandString, validate:cmds[code].validationString});
					 execute(cmds[code].commandString,cmds[code].validationString);

				 }else{
					 logic.status.setLastCommandStatus("false");
					 console.warn("requestCommandAction "+code+" not understood!!!");
				 }


			 };

			 /**
			  * Show overlay layout.
			  * @param {Number} layoutId The layout Id.
			  * @param {Number} duration The duration to play. 0 means default which is 5.
			  * @param {Boolean} downloadRequired Has to download the resource.
			  * @returns None
			  * @see It will not show again a scheduled overlay layout.
			  */
			 this.overlayLayout = function(layoutId, duration, downloadRequired)
			 {
				 var f_overlayLayout = function()
				 {
					 var scheduledOverlays = ScheduleManager.getScheduleData().overlays;
					 for (var i = 0, max = scheduledOverlays.length; i < max; i++) {
						 if (scheduledOverlays[i].file === layoutId){
							 console.debug("Overlay layout " + layoutId + " is already shown. Don't show again!");
							 return;
						 }
					 }
					 console.debug("Show overlay layout " + layoutId + " for "+ duration + " seconds.");
					 if (duration*1 === 0){
						 duration = 5;
					 }
					 LayoutManager.showOverlay([{layoutId:layoutId+"", scheduleId : "0"}]);

					 setTimeout(function(){
						 console.debug("Hide overlay layout " + layoutId + ".");
						 LayoutManager.removeOverlay([{layoutId:layoutId+"", scheduleId : "0"}]);
					 }, duration*1000);
				 };

				 if (downloadRequired===true){
					 collectIntervalLogic(function(){
						 f_overlayLayout();
					 },function(cbError){
						 logic.nonFatalError(cbError);
					 });
				 }else{
					 f_overlayLayout();
				 }
			 };

			 this.rekeyAction = function(){
				 var st = utils.getTimeStamp();
				 console.info("RekeyAction started");
				 XMRManager.createKey();
				 var et = utils.getTimeStamp();
				 var tt = et - st;
				 console.info("RekeyAction ended in " + tt + " seconds.");
				 //XMRManager.

				 connectToCMS(true);
			 };

			 this.revertToSchedule = function(){
				 console.info("Revert To Schedule");
				 v_runningSchedule = true;
				 LayoutManager.setLayouts(stoppedScheduleLayouts);
				 v_changeLayoutLayouts = [];
				 changeLayoutTimer = null;
			 };
		 }();
		 // </editor-fold desc="Commands">


		// </editor-fold desc="Public functions">
	}();
}());