/* global application, logic, DeviceManager, utils, LayoutManager, ScheduleManager, Logger, XMRManager, LicenseManager, RemoteControl, AppVersionInformation, defined, ERRORS, ko, memoryViewModel, statusViewModel */

(function() {

	// Use the correct AudioContext!
	window.AudioContext = window.AudioContext || window.webkitAudioContext;

	//Log to my comnputer
	var con = console;
	var localConsoleElement = document.getElementById('logContainer'); 
	
	var rlogging = function(){
		var post = function(lvl, data){
			var xhr = new XMLHttpRequest();
			
			var bd = {};
			bd.level = lvl;
			bd.message = data;
			bd.timestamp = utils.getLocalDate();
			
			xhr.open("POST", "http://192.168.1.170:8080", true); // true for asynchronous
			xhr.setRequestHeader("Content-Type", "application/json");
			xhr.send(JSON.stringify(bd));
			
			
		};
		
		var int_error = function(){
			post('error', arguments);
		};
		var int_warn = function(){
			post('warn', arguments);
		};
		var int_log = function(){
			post('log', arguments);
		};
		var int_info = function(){
			post('info', arguments);
		};
		var int_debug = function(){
			post('debug', arguments);
		};
		
		return {
			error:int_error,
			warn:int_warn,
			log:int_log,
			info:int_info,
			debug:int_debug
		};
	};
	
	
	var llogging = function(){
		
		var post = function(lvl, data){
			if (!localConsoleElement){
				localConsoleElement = document.getElementById('logContainer');
			}
			var bd = {};
			bd.level = lvl;
			bd.message = data;
			bd.timestamp = utils.getLocalDate();
			
			var nc = document.createElement('div');
			nc.innerText=JSON.stringify(data);
			if (localConsoleElement){
				localConsoleElement.appendChild(nc);
				
				if(localConsoleElement.children.length > 200){
					localConsoleElement.removeChild(localConsoleElement.firstChild);
					localConsoleElement.lastChild.scrollIntoViewIfNeeded();
				}
				
			}
			
		};
		
		var int_error = function(){
			post('error', arguments);
		};
		var int_warn = function(){
			post('warn', arguments);
		};
		var int_log = function(){
			post('log', arguments);
		};
		var int_info = function(){
			post('info', arguments);
		};
		var int_debug = function(){
			post('debug', arguments);
		};
		
		return {
			error:int_error,
			warn:int_warn,
			log:int_log,
			info:int_info,
			debug:int_debug
		};
	};

	/*
	 * Uncomment the line below to receive log messages on log server. 
	 */
	if(AppVersionInformation.isDevVersion){
//		console = new rlogging();
		console = new llogging();
	}
	
	
    /**
     * @classdesc Tizen Application
     * This is the main application logic.
     * @type logic
     * @version 1.8.1
     * @since 1.8.1
     */
	//jshint -W057
	window.logic = new function _logic() {
		"use strict";

		document.addEventListener('visibilitychange', function() {
			  if (document.hidden) {
				  console.log('App hidden. Start after 3 secs.');
				  setTimeout(function(){
					  console.log('Launch myself.');
					  var myid = tizen.application.getAppInfo().id;
					  tizen.application.launch(myid);
				  },3000);
			    //Something you want to do when an application is hidden.
			  } else {
			    //Something you want to do when an application is resumed.
				  console.log('App resumed');
			  }
			});

        /* Private functions */

        /* Public functions */

        /*
         * Callback function if CMS configuration exists function failed to execute.
         * @param {object} cbObj Return the error object. Object have errorCode and errorText fields.
         */
        function cbFailure(cbObj) {
            console.error("Error occured!", cbObj);
        }

        this.nonFatalError = function(cbError) {
          var errMsg;
          if (cbError.hasOwnProperty('message')){
            errMsg = cbError.message;
        		if (cbError.hasOwnProperty('additional')){
        			errMsg += " "+JSON.stringify(cbError.additional);
        		}
          }else{
            errMsg = JSON.stringify(cbError);
          }
            //Logger.cmsLog({ category: "error", message: errMsg });
            // XXX: make a suggestion for a proper error code for this case
            Logger.cmsFault(undefined, undefined, 
                {
                    code: ERRORS.FaultCodes.RemoteResourceFailed,
                    reason: errMsg,
                    method: "nonFatalError",
                    thread: "application",
                    date: new Date(),
                    expires: new Date(new Date().valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000 )
                }
            );
            Logger.submitLog(function() {
                console.debug("Send error log completed");
            }, cbFailure);
        };

        /**
         * This is the first function called after page load!
         * @returns {undefined} undefined
         */
        this.init = function() {

        	/**************************************************************/
        	var testFunc = function() {
        		
        	};
        	
        	
        	testFunc();
    		/**************************************************************/
        	
            var normalStart = function() {
                //this.statusPage;

                if (AppVersionInformation.displayMemoryUsage === true) {
                    application.memoryStatus = new memoryViewModel();
                    ko.applyBindings(application.memoryStatus, document.getElementById('memoryStatus'));
                    application.memoryStatus.memoryFree("0");
                } else {
                    document.getElementById('memoryViewContainer').style.visibility = 'collapse';
                }

                application.statusPage = new statusViewModel();
                utils.loadPagetoTarget("cmsStatus.html", "statusContainer", "documentLoad", application.hideStatusPage);

                /* Menu bar events */
                application.setHandlers();

                //throw ERRORS.ErrorWithSettings("Just a test");
                DeviceManager.Device.setNTP(
                    function(result){
                        console.log(result)
                    }, function(error){
                        console.error(error)
                    }
                );
                
                RemoteControl.init();

                LicenseManager.init();

                var delayInit = function() {

                    Logger.cmsLog({
                        category:  Logger.cmsLog.CATEGORY.AUDIT,
                        message: "Application start at " + (new Date())
                    });

                    setInterval(function(){
                        application.statusPage.playerDateTime(utils.getLocalDate());
                    }, 1000);

                    //WebOSManager.Device.setNativePortraitMode(null,logic.nonFatalError,"0");

                    utils.enableDebugMode(function() {
                        //WebOSManager.Device.disableAllTimers(function(){
                        DeviceManager.FileSystem.listFiles(function(cbObject) {

                            var files = cbObject;
                            //console.groupCollapsed("List internal files");
                            for (var i = 0; i < files.length; ++i) {
                                var fileInfo = files[i];
                                console.debug("File Name: " + fileInfo.name + ", Size: " + fileInfo.fileSize);
                            }
                            //console.groupEnd();
                            DeviceManager.Device.gatherDeviceInfo(function() {
                            	DeviceManager.Device.registerSystemMonitor(function() {
                                    XMRManager.init();
                                    DeviceManager.FileSystem.emptyDirectory(function() {
                                    	DeviceManager.FileSystem.deleteFiles(function() {
                                            LayoutManager.init("viewContainer");
                                            ScheduleManager.start();
                                            LicenseManager.checkLicenseServer();
                                        }, cbFailure);
                                    }, cbFailure);
                                }, cbFailure);
                            }, cbFailure);
                        }, cbFailure);
                        //}, cbFailure);
                    }, cbFailure);
                };

                setTimeout(delayInit, 3000);

            };
            
            /* After upgrade procedure */
            DeviceManager.Device.getServerProperty(function() {
                    DeviceManager.FileSystem.fileExists(
                        function(cbObj) {
                            if (cbObj.exists) {
                                DeviceManager.FileSystem.readFile(
                                    function(rfObject) {
                                        var t = JSON.parse(rfObject);
                                        for (var item in t) {
                                            localStorage[item] = t[item];
                                        }
                                        DeviceManager.FileSystem.deleteFile(
                                            function() {
                                                normalStart();
                                            },
                                            function() {
                                                normalStart();
                                            }, {
                                                path: "upgrade_settings.dat"
                                            }
                                        );
                                    },
                                    function() {
                                        normalStart();
                                    }, {
                                        path: "upgrade_settings.dat"
                                    }
                                );
                            } else {
                                normalStart();
                            }
                        },
                        function() {
                            normalStart();
                        }, {
                            path: "upgrade_settings.dat"
                        });
                },
                function(cbObject) {
                    logic.nonFatalError(cbObject);
                }
            );

        };

        this.showCMSParameterEntry = function() {
            console.debug("Show CMS parameter entry page");
            LayoutManager.showSettingsScreen();
            //utils.loadPagetoTarget("cmsSettings.html","webOS_Body", "documentLoad");
        };

        /* Application global variables */
        this.status = new function _status() {
            var v_lastCommandStatus = false;
            if (localStorage.hasOwnProperty("lastCommandStatus")) {
                v_lastCommandStatus = localStorage.lastCommandStatus;
            }

            /**
             * Get whether or not the last received Command was executed successfully
             */
            this.getLastCommandStatus = function() {
                return v_lastCommandStatus;
            };

            this.setLastCommandStatus = function(status) {
                v_lastCommandStatus = status;
                localStorage.lastCommandStatus = status;
            };
        }();

        /**
         * Soft restart the application.
         * @returns {undefined}
         * @version 2.0.204
         * @since 2.0.204
         */
        this.softRestart = function(){
            location.reload();
        };
    }();

    /**
     * @classdesc Application UI
     * This is the UI application logic.
     * @type application
     * @version 1.8.1
     * @since 1.8.1
     */
    window.application = new function _application() {
    	"use strict";

        /* Private variables */

        /**
         * Reference to menu timer.
         * @version 1.8.1
         * @since 1.8.1
         */
        var v_mnuTimer = null;

        /**
         * How long to wait before closing the menu.
         * @version 1.8.1
         * @since 1.8.1
         */
        var v_mnuTimeout = 60;

        /**
         * Should the menu be visible or hidden.<br>
         * 0 means hidden, 1 means timed.
         * @version 1.8.1
         * @since 1.8.1
         */
        var v_mnuInitialState = 1;

        /* Public variables */

        this.applicationUpgrade = false;

        /* Private functions */

        /* Public functions */

        /**
         * Set the event handles of the application.
         * @returns None
         */
        this.setHandlers = function() {
            if (AppVersionInformation.enableGlobalErrorHandler) {
                /* Global error handler */
                window.onerror = function(msg, url, lineNo, columnNo, error) {
                    var errorObj = {
                        msg: msg,
                        url: url,
                        lineNo: lineNo,
                        columnNo: columnNo,
                        error: JSON.stringify(error)
                    };

                    //TODO: Remove check after finish development
                    if (false){
	                    //Logger.cmsLog({category : "error", message: JSON.stringify(errorObj), onlyOnce:true});
                        // XXX: make a suggestion for a proper error code for this case
                        Logger.cmsFault(undefined, undefined, 
                            {
                                code: ERRORS.FaultCodes.RemoteResourceFailed,
                                reason: JSON.stringify(errorObj),
                                method: "setHandlers",
                                thread: "application",
                                date: new Date(),
                                expires: new Date(new Date().valueOf() + DeviceManager.Config.getOptions().collectInterval * 1000 ),
                                onlyOnce: true,
                            }
                        );
	                    Logger.submitLog(function(){
	                        console.debug("Global error send!");
	                    },function(cbObject){
	                         console.debug("Global error send failed!", cbObject);
	                    });                    
                    }else{                    
                    	throw error;
                    }
                    return true;
                };
            }
 
            RemoteControl.registerKeyCallback('Yellow', function(evt){
                var e = document.getElementById("appMenu");
                if (e.classList.contains("showMenu")){
                    application.hideAppMenu();
                }else{
                    application.showAppMenu();
                }
            });
            
            /* Add handlers for status page*/
            var e = document.getElementById("webOS_Body");
            // e.onclick = function(evt) {
            //     //            console.debug("I clicked!");
            //     evt.stopPropagation();
            //     evt.preventDefault();
            //     application.showAppMenu();
            // };

            e.oncontextmenu = function() {
                console.debug("I right clicked!");
            };

            // // Touch handling
            // e.ontouchstart = function(e){
            //     console.info("TStart", e);
            // };
            // e.ontouchmove = function(e){
            //     console.info("TMove", e);
            // };
            // e.ontouchend = function(e){
            //     console.info("TEnd", e);
            // };
            
            // e.ontouchcancel = function(e){
            //     console.info("TCancel", e);
            // };
            
            var mnuHide = document.getElementById('mnuHide');
            mnuHide.onclick = function(evt) {
                console.debug("Menu hide clicked!", evt);
                evt.stopPropagation();
                application.hideAppMenu();
            };

            //        mnuHide.onkeydown = function(evt){
            //            console.debug("Menu hide selected!", evt);
            //            evt.stopPropagation();
            //            application.hideAppMenu();
            //        };

            var mnuStatus = document.getElementById('mnuStatus');
            mnuStatus.onclick = function(evt) {
                console.debug("Menu status clicked!", evt);
                evt.stopPropagation();
                application.showStatusPage();
                application.statusPage.timelyClearLogs();
            };

            //        mnuStatus.onkeydown = function(evt){
            //            console.debug("Menu status clicked!", evt);
            //            evt.stopPropagation();
            //            application.showStatusPage();
            //            application.statusPage.timelyClearLogs();
            //        };

            var mnuSettings = document.getElementById('mnuSettings');
            mnuSettings.onclick = function(evt) {
                console.debug("Menu settings clicked!", evt);
                evt.stopPropagation();
                application.showSettings(application.hideSettings);
            };

            //        mnuSettings.onkeydown = function(evt){
            //            console.debug("Menu settings clicked!", evt);
            //            evt.stopPropagation();
            //            application.showSettings(application.hideSettings);
            //        };
        };

        this.showStatusPage = function() {
            console.debug("Show status page!");
            var e = document.getElementById("statusContainer");
            try {
                document.getElementsByClassName("rcbutton_active")[0].classList.remove("rcbutton_active");
            } catch (e) {
                console.log("No active element found...");
            }finally{
                e.firstElementChild.firstElementChild.scrollIntoView(); // Put the page at the begining.
                e.style.visibility = "visible";
                application.hideAppMenu();
                LayoutManager.hideExternalInputWidget();
            }
        };

        this.hideStatusPage = function() {
            console.debug("Hide status page!");
            var e = document.getElementById("statusContainer");
            e.style.visibility = "hidden";
            LayoutManager.showExternalInputWidget();
        };

        this.showSettings = function() {
            console.debug("Show settings page!");
            application.hideAppMenu();
            ScheduleManager.userOpenSettingsScreen();
            LayoutManager.showSettingsScreen(application.hideSettings);
            LayoutManager.hideExternalInputWidget();
        };

        /**
         * Hide the settings page. This occurs only when user press 'Connect CMS' button on settings page.
         * @param {type} save Save the configuration into memory.
         * @version 1.8.1
         * @since 1.8.1
         */
        this.hideSettings = function(save) {
            console.debug("Hide settings page!");
            ScheduleManager.ConnectToCMS(save);
            LayoutManager.showExternalInputWidget();
        };

        this.showAppMenu = function() {
            var isCmsSettingsScreenVisible = document.getElementById("connectCMSContainer").style.display === 'block';
            var isStatusPageVisible = document.getElementById("statusContainer").style.visibility === "visible";
            var isLogContainerVisible = document.getElementById("logContainer").style.display === "block";
            if(isStatusPageVisible || isLogContainerVisible || isCmsSettingsScreenVisible){
                return;
            }
            console.debug("Show Menu!");
            var mnuHide = document.getElementById('mnuHide');
            var mnuStatus = document.getElementById('mnuStatus');
            var mnuSettings = document.getElementById('mnuSettings');
            mnuHide.classList.add("rcbutton");
            mnuStatus.classList.add("rcbutton");
            mnuSettings.classList.add("rcbutton");

            var e = document.getElementById("appMenu");
            e.classList.remove("hideMenu");
            e.classList.add("showMenu");
            if (v_mnuTimer === null) {
                v_mnuTimer = setTimeout(application.appMenuTimeout, v_mnuTimeout * 1000);
            }
            LayoutManager.hideExternalInputWidget();
        };

        this.hideAppMenu = function() {
            console.debug("Hide Menu!");
            var mnuHide = document.getElementById('mnuHide');
            var mnuStatus = document.getElementById('mnuStatus');
            var mnuSettings = document.getElementById('mnuSettings');
            mnuHide.classList.remove("rcbutton");
            mnuStatus.classList.remove("rcbutton");
            mnuSettings.classList.remove("rcbutton");
            var e = document.getElementById("appMenu");
            e.classList.remove("showMenu");
            e.classList.add("hideMenu");
            RemoteControl.clearActiveElement();
            if (v_mnuTimer !== null) {
                clearTimeout(v_mnuTimer);
                v_mnuTimer = null;
            }
            LayoutManager.showExternalInputWidget();
        };

        this.appMenuTimeout = function() {
            console.debug("Menu timeout hit!");
            application.hideAppMenu();
            v_mnuTimer = null;
        };

        this.setMenuHideTimeout = function(t) {
            if (!defined(t)) {
                return;
            }

            v_mnuTimeout = t;
            console.debug("Menu timeout is set to " + v_mnuTimeout);
            if (defined(v_mnuTimer)) {
                clearTimeout(v_mnuTimer);
                v_mnuTimer = setTimeout(application.appMenuTimeout, v_mnuTimeout * 1000);
            }
        };

        this.getMenuInitialState = function() {
            return v_mnuInitialState;
        };

        this.setMenuInitialState = function(state) {
            v_mnuInitialState = state;
        };

    }();
	//jshint +W057

}());