//jshint ignore:start
/* gglobal ZmqSocket, Logger */

(function ()
{
    "use strict";

// Check if module is already initialized.
    if (window.ZmqSocket)
        return;
    
    // Constructor. You may pass socket identity here.
    /**
     * ZmqSocket constructor
     */
    window.ZmqSocket = function (identity)
    {
        this.fd = 0; //Actually it is only one socket
        ZmqSocket.__nextFd++;
        ZmqSocket.__sockets[this.fd] = this;

        this.identity = identity;
        this.state = ZmqSocket.CONNECTING;
        
        this.onopen = function () {};
        this.onmessage = function () {};
        this.onerror = function (msg) {};
        this.onclose = function () {};

        var athis = this;
        //ZmqSocket.__addTask(function ()
        //{
            //ZmqSocket.__flash.create(athis.fd, athis.identity);
    	//DeviceManager.NodeServer.init();
        //});
    };

    ZmqSocket.__sockets = []; //Actually it is only one socket
    ZmqSocket.__tasks = [];
    ZmqSocket.__nextFd = 0;
    ZmqSocket.__flash = null;

    ZmqSocket.CONNECTING = 1;
    ZmqSocket.OPEN = 2;
    ZmqSocket.CLOSING = 3;

    // Called by Flash when it's ready.
    /**
     * Called by IPC when connected.
     * @static
     */
    ZmqSocket.__onFlashReady = function (socket)
    {
        //console.debug("Nodejs socket is ready!!!");
    	
//        if (navigator.appName.indexOf("Microsoft") != -1)
//            ZmqSocket.__flash = window["ZmqSocketMain"];
//        else
//            ZmqSocket.__flash = document["ZmqSocketMain"];
        ZmqSocket.__flash = socket;
        
        
        socket.on('disconnect', function(){
			console.log('IPC Disconnected!');
		});
		socket.on('_error', function (data) {
		    console.warn('IPC error',{message:data.message, stack:data.stack});
		});
		socket.on('log', function (data) {
		    console.log('IPC log',data);
		});
		socket.on('data', function (obj) {
		    //console.log('IPC data',obj);
		    //ZmqSocket.__onmessage(obj.fd, obj.data);
			ZmqSocket.__onmessage(0, obj.data);
		});
		socket.on('control', function (data) {
		    console.log('IPC control',data);
		    if (data.hasOwnProperty('connected')){
			    if (data.connected.status === 'connected'){
			    	//ZmqSocket.__onopen(data.connected.fd);
			    	ZmqSocket.__onopen(0);
			    }
		    	
		    }
		});
        
        
        
        // Make it later to avoid recursion.
        ZmqSocket.__later(function ()
        {
            for (var i = 0; i < ZmqSocket.__tasks.length; i++)
                ZmqSocket.__tasks[i]();
        });
        
        
    };

    // Used to make calls in a separate event handler to avoid
    // recursive calls to Flash - recursion is not supported by many browsers.
    ZmqSocket.__later = function (task)
    {
        var to = setTimeout(function () {
            clearTimeout(to);
            task();
        }, 0);
    };
    
    // Called by Flash to find out if JS is ready.
    ZmqSocket.__isJSReady = function () {
        return true;
    };

    // Called when we are not sure that flash is ready.
    ZmqSocket.__addTask = function (task)
    {
        if (ZmqSocket.__flash)
            task();
        else
            ZmqSocket.__tasks.push(task);
    };

    ZmqSocket.__log = function (message){
        //console.info(message);
    };

    /**
     * Connect to XMR
     * @param fd
     * @param host
     * @param port
     * @static
     */
    ZmqSocket.prototype.connect = function (host, port)
    {
        var athis = this;
        ZmqSocket.__addTask(function ()
        {
            //ZmqSocket.__flash.connect(athis.fd, host, port);
        	ZmqSocket.__flash.emit('control', {'connect' : {fd:athis.fd, host:host, port:port}});
        });
    };

    ZmqSocket.prototype.close = function ()
    {
        this.state = ZmqSocket.CLOSING;
        var athis = this;
        ZmqSocket.__addTask(function ()
        {
            //ZmqSocket.__flash.close(athis.fd);
        	ZmqSocket.__flash.emit('control', {'close' : {fd:athis.fd}});
        });
    };
    
    ZmqSocket.prototype.isConnected = function ()
    {
        var athis = this;
        ZmqSocket.__addTask(function ()
        {
            //ZmqSocket.__flash.close(athis.fd);
        	ZmqSocket.__flash.emit('control', {'status' : 'isConnected', fd:athis.fd});
        });
    };

    /**
     * @depricated
     */
    ZmqSocket.prototype.available = function ()
    {
        if (this.state != ZmqSocket.OPEN)
            throw "Invalid state, socket must be connected!";

        return ZmqSocket.__flash.available(this.fd);
    };

    /**
     * @depricated
     */
    ZmqSocket.prototype.recv = function ()
    {
        if (this.state != ZmqSocket.OPEN)
            throw "Invalid state, socket must be connected!";

        return ZmqSocket.__flash.recv(this.fd);
    };

    ZmqSocket.prototype.send = function (msg)
    {
        if (this.state != ZmqSocket.OPEN)
            throw "Invalid state, socket must be connected!";

        return ZmqSocket.__flash.send(this.fd, msg);
    };

    ZmqSocket.prototype.subscribe = function (subject)
    {
        if (this.state != ZmqSocket.OPEN)
            throw "Invalid state, socket must be connected!";

        //return ZmqSocket.__flash.subscribe(this.fd, subject);
        ZmqSocket.__flash.emit('control', {fd:this.fd, 'subscribe' : subject});
    };

    ZmqSocket.prototype.unsubscribe = function (subject)
    {
        if (this.state != ZmqSocket.OPEN)
            throw "Invalid state, socket must be connected!";

        return ZmqSocket.__flash.unsubscribe(this.fd, subject);
    };
    
    ZmqSocket.__onopen = function (fd)
    {
        // Avoiding recursion
        ZmqSocket.__later(function ()
        {
            ZmqSocket.__sockets[fd].state = ZmqSocket.OPEN;
            ZmqSocket.__sockets[fd].onopen();
        });
    };

    ZmqSocket.__onmessage = function (fd,data)
    {
        // Avoiding recursion
        ZmqSocket.__later(function () {
            ZmqSocket.__sockets[fd].onmessage(data);
        });
    };

    ZmqSocket.__onerror = function (fd, msg)
    {
        // Avoiding recursion
        ZmqSocket.__later(function () {
            ZmqSocket.__sockets[fd].onerror(msg);
        });
    };

    ZmqSocket.__onclose = function (fd)
    {
        // Avoiding recursion
        ZmqSocket.__later(function () {
            ZmqSocket.__sockets[fd].onclose();
        });
    };

})();