/* global LayoutManager */

/**
 * Load and play an overlay audio of an element.<br>
 * @param {Object} media The audio information.
 * @param {Object} container Not Used.
 * @returns {LayoutManager.audioOverlay} Return a audioOverlay object.
 */
(function () {
  "use strict";

  window.LayoutManager.audioOverlay = function (media, container) {
    this.media = media;
    this.container = container;
    var source = media.uri;

    /* Create a buffer source for the specific file. */
    var bufSource = LayoutManager.audioContext.createBufferSource(); // creates a sound source

    /* Create a gain node */
    var gain = LayoutManager.audioContext.createGain();

    /* Wire nodes */
    bufSource.connect(gain); // connect the source to the gain node for volume setting.
    gain.connect(LayoutManager.audioContext.destination); // connect the gain node to the context's destination (the speakers)

    this.preload = function (resolveCb, failureCb) {
      //Logger.log("Preload " + source, Logger.DEBUG);


      var request = new XMLHttpRequest();
      request.open('GET', source, true);
      request.responseType = 'arraybuffer';

      // Decode asynchronously
      request.onload = function () {
        //Logger.log("Loaded " + source, Logger.DEBUG);
        LayoutManager.audioContext.decodeAudioData(request.response,
          function (buffer) {
            //Logger.log("Decoded " + source, Logger.DEBUG);
            bufSource.buffer = buffer;
            if (resolveCb) {
              resolveCb();
            }
          },
          function (evt) {
            //Logger.log("error", Logger.WARN, evt);
            if (failureCb) {
              failureCb(evt);
            }
          });
      };

      request.send();
    };

    this.play = function () {

      //Logger.log("Play " + source, Logger.DEBUG);

      bufSource.onended = function () {
        //Logger.log("Ended: "+ source, Logger.DEBUG);
        bufSource = null;
      };

      bufSource.start(0);
    };

    this.stop = function () {
      //Logger.log("Stop " + source,Logger.DEBUG);
      if (bufSource) {//If source buffer is valid.
        bufSource.stop();
      }
    };

    if (media.volume) {
      try {
        gain.gain.value = Number(media.volume) / 100;
      } catch (e) {
        gain.gain.value = 1;
      }
    }

    if (media.loop) {
      if (Number(media.loop) === 1) {
        bufSource.loop = true;
      }
    }
    //this.preload();

  };
}());