/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */
/* global LayoutManager, utils, LibraryManager, Logger */

(function () {
  "use strict";

  /**
   * Display a HTML widget
   * @syntax new LayoutManager.html(media, container, callback, layoutid, scp)
   * @param {type} media
   * @param {type} container
   * @param {type} callback
   * @param {type} layoutid
   * @param {type} scheduleid
   * @param {type} scp
   * @version 1.8.8
   * @since 1.8.0
   */
  window.LayoutManager.html = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);

      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }

      this.statsStartTime = utils.getISODate();
      this.duration = parseInt(media.duration) * 1000;
      var duration = this.duration;
      /**
       * modules/webpage-form-edit.twig:                        { modeid: 1, mode: openNatively },
       * modules/webpage-form-edit.twig:                        { modeid: 2, mode: manualPosition },
       * modules/webpage-form-edit.twig:                        { modeid: 3, mode: bestFit }
       * @type {string}
       */
      if (media.media.options[0].modeid &&
        media.media.options[0].modeid[0] &&
        media.media.options[0].modeid[0].jValue === "1"){
        this.uri = decodeURIComponent(media.media.options[0].uri[0].jValue);
      }else{
        this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.id + ".html";
      }
      var timeStarted = 0;
      var timePlayed = 0;
      var parent = this;
      var isPaused = false;
      var isFullscreen = false;
      this.timeoutElement = null;
      this.removeMedia = function () {
        container.removeChild(this.elem);
      };
      var hastransIn = false;
      var hastransOut = false;
      var transInDuration = 1000;
      var transOutDuration = 1000;
      var transInType = null;
      var transOutType = null;
      var transInDirection = "S";
      var transOutDirection = "S";

      var v_touchOverlayElements = [];
      var latestHTMLError = undefined;
      var pageLoadErrorTrigger = undefined;

      if (media.media.options[0].pageLoadErrorTrigger) {
        pageLoadErrorTrigger = media.media.options[0].pageLoadErrorTrigger[0].jValue;
      }
      // Clear any error reported pages from the underlying OS
      var recurClearHTMLErrors = function (el) {
        // if (el.classList.contains('moon-application-global-wrapper')){
        if (el.contentDocument.getElementById("Box") && el.contentDocument.getElementById("Box").firstChild && el.contentDocument.getElementById("Box").firstChild.innerText == "This webpage is not available.") {
          var errorDetails = el.contentDocument.getElementById("Box").childNodes[2];
          if (errorDetails) {
            latestHTMLError = errorDetails;
          }
          el.contentDocument.body.innerHTML = "";
        } else {
          var childs = el.contentDocument.getElementsByTagName('iframe');
          Array.from(childs).forEach(function (v) {
            recurClearHTMLErrors(v);
          });
        }
      };

      function recurSetTextureOnVideo(el) {
        Array.from(el.contentDocument.getElementsByTagName('video')).forEach(function (v) {
          v.setAttribute('texture', '');
        });
        Array.from(el.contentDocument.getElementsByTagName('iframe')).forEach(function (ifrm) {
          recurSetTextureOnVideo(ifrm);
        });
      };
      
      function disableAlert(el) {
  	    el.contentWindow.alert = function(arg){console.log(arg)};
		Array.from(el.contentDocument.getElementsByTagName('iframe')).forEach(function (ifrm) {
		  disableAlert(ifrm);
		});
      };



      // Fix for XLG-57
      var checkUpdateInterval = function () {
        var invalidateAfter = 300;

        var f = LibraryManager.getFileByName(media.id + ".html");
        if (f !== null) {
          try {
            if (media.media.options[0].hasOwnProperty("updateInterval")) {
              invalidateAfter = parseInt(media.media.options[0].updateInterval[0].jValue) * 60;
            }
            if (utils.getTimeStamp() >= (f.times.mtime / 1000 + invalidateAfter)) {
              f.manuallyInvalidated = true;
              f.download(function () {
                console.log("Refreshing resource " + f.filename);
                f.lastChecked = utils.getTimeStamp();
              }, function (cbErr) {
                console.warn("Something wend wrong for resource " + f.filename, cbErr);
              });
            }
            //                else{
            //                    console.log("Resource " + f.filename + " will update after "+ ((f.times.mtime/1000 + invalidateAfter - utils.getTimeStamp())|0) +" seconds.");
            //                }

          } catch (e) {
            console.warn('Error for file ' + f.filename, e);
          }
        }
      };

      if (media.media.options[0].hasOwnProperty("transIn")) {
        hastransIn = true;
        transInType = media.media.options[0].transIn[0].jValue;
        if (media.media.options[0].transInDirection) {
          transInDirection = media.media.options[0].transInDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transInDuration")) {
          transInDuration = media.media.options[0].transInDuration[0].jValue;
        }
      }
      if (media.media.options[0].hasOwnProperty("transOut")) {
        hastransOut = true;
        transOutType = media.media.options[0].transOut[0].jValue;
        if (media.media.options[0].transOutDirection) {
          transOutDirection = media.media.options[0].transOutDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transOutDuration")) {
          transOutDuration = media.media.options[0].transOutDuration[0].jValue;
        }
      }


      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        parent.dispatchEvent(widgetendevt);
      };

      var widgetExpired = function () {
        if (hastransOut) {
          LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            if (scheduleid && media.enableStat !== "0") {
              Logger.logStat({
                fromdt: parent.statsStartTime,
                todt: utils.getISODate(),
                scheduleid: scheduleid,
                layoutid: layoutid,
                mediaid: media.id
              });
            }
            //            callback();
            widgetEnded();
          });
        } else {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
          //          callback();
          widgetEnded();
        }
      };

      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();
        checkUpdateInterval();
        //var tag = "r: " + container.getAttribute("id") + " m: " + media.id + " html";
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };
      var timeoutMediaInvoke = this.timeoutMedia;

      var prepareHtml = function (cbReady) {
        var ifrm = document.createElement("iframe");
        ifrm.onload = function (r1) {
          latestHTMLError = undefined;
          disableAlert(ifrm);
          recurClearHTMLErrors(ifrm);
          if (!latestHTMLError) {
            recurSetTextureOnVideo(ifrm);
          } else { // Some error happened
            console.warn('HTML error', latestHTMLError);
            if (pageLoadErrorTrigger) {
              InteractiveManager.handleWebHookEvent(pageLoadErrorTrigger);
            }
          }
          window.focus();
          cbReady(ifrm);
        };
        parent.elem = ifrm;
        ifrm.sandbox ="allow-scripts allow-forms allow-pointer-lock allow-same-origin";
        ifrm.src = parent.uri;
        ifrm.style.position = "absolute";
        ifrm.style.left = 0;
        ifrm.style.top = 0;
        ifrm.style.width = "100%";
        ifrm.style.height = "100%";
        ifrm.style.overflow = "hidden";
        ifrm.style.border = "none";
        if (hastransIn) {
          parent.elem.style.visibility = "hidden"
        }

        container.insertBefore(parent.elem, container.firstElementChild);

      };

      this.displayThisMedia = function () {
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            var v_domActionOverlay = undefined;
            if (action.jAttr.triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
              v_domActionOverlay = document.createElement("div");
              v_domActionOverlay.id = action.jAttr.id;
              v_domActionOverlay.style.position = "absolute";
              v_domActionOverlay.style.width = container.style.width;
              v_domActionOverlay.style.height = container.style.height;
              v_domActionOverlay.style.top = container.style.top;
              v_domActionOverlay.style.left = container.style.left;
              container.parentElement.appendChild(v_domActionOverlay);
              v_touchOverlayElements.push(v_domActionOverlay);
            }
            var act = action.jAttr;
            InteractiveManager.addEventAction(v_domActionOverlay, act.id, act.triggerType, act.triggerCode,
              act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
              act.targetId, act.widgetId);
          });
        }

        /* Make embedded video tags, hardware accelerated */
        // document.getElementsByTagName('iframe')[0].contentDocument.getElementsByTagName('iframe')[0].contentDocument.getElementsByTagName('video')[0].setAttribute('texture','')

        if (hastransIn) {
          duration -= parseInt(transInDuration);
          utils.hide(this.elem);
        }
        if (hastransOut) {
          duration -= parseInt(transOutDuration);
        }
        if (duration < 0) {
          duration = 0;
        }

        setTimeout(function () {
          if (container.childElementCount > 1) {
            container.removeChild(container.lastElementChild);
          }
        }, 0);
        if (hastransIn) {
          parent.elem.style.visibility = "visible"
          LayoutManager.transin(this.elem, container, transInType, transInDuration, transInDirection, function () {
            timeoutMediaInvoke(parent);
          });
        } else {
          timeoutMediaInvoke(parent);
        }
      };
      this.stopMediaTimeout = function () {
        if (parent.audiooverlay) {
          parent.audiooverlay.stop();
        }
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            InteractiveManager.removeEventAction(action.jAttr.id);
          });
          v_touchOverlayElements.forEach(function (elem) {
            container.parentElement.removeChild(elem);
          });
          v_touchOverlayElements = [];
        }
        //        if (scheduleid){
        //            Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
        //        }
        clearTimeout(parent.timeoutElement);
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      utils.httpGetAsync(this.uri, function (contents) {
        if (contents.indexOf("<!-- DURATION=") !== -1) {
          var parts = contents.split("<!-- DURATION=");
          var htmlduration = parseInt(parts[parts.length - 1]);
          if (htmlduration > 0) {
            duration = htmlduration * 1000;
          }
        }
        prepareHtml(function () {
          parent.displayThisMedia(parent);
        });

      }, function (errorCb) {
        console.error(errorCb);
        setTimeout(function () {
          recurClearHTMLErrors(parent.elem);
        }, 0);
      });
      /**
       * Pause the html widget. The element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          //Pause and hide video element
          container.removeChild(parent.elem);
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
            v_touchOverlayElements.forEach(function (elem) {
              container.parentElement.removeChild(elem);
            });
            v_touchOverlayElements = [];
          }
          if (isFullscreen) {
            restoreSize();
          }
          //Calculate remain duration
          timePlayed += (new Date()).valueOf() - timeStarted;
          duration -= (new Date()).valueOf() - timeStarted;
          clearTimeout(parent.timeoutElement);
          parent.timeoutElement = null;
          isPaused = true;
        }
      };

      /**
       * Resume the html widget. The element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          parent.elem = prepareHtml(function (v, dur) {
            parent.elem = v;
            if (container.childElementCount > 1) {
              container.removeChild(container.lastElementChild);
            }
            if (media.media.action) {
              media.media.action.forEach(function (action) {
                var v_domActionOverlay = undefined;
                if (action.jAttr.triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
                  v_domActionOverlay = document.createElement("div");
                  v_domActionOverlay.id = action.jAttr.id;
                  v_domActionOverlay.style.position = "absolute";
                  v_domActionOverlay.style.width = container.style.width;
                  v_domActionOverlay.style.height = container.style.height;
                  v_domActionOverlay.style.top = container.style.top;
                  v_domActionOverlay.style.left = container.style.left;
                  container.parentElement.appendChild(v_domActionOverlay);
                  v_touchOverlayElements.push(v_domActionOverlay);
                }
                var act = action.jAttr;
                InteractiveManager.addEventAction(v_domActionOverlay, act.id, act.triggerType, act.triggerCode,
                  act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
                  act.targetId, act.widgetId);
              });
            }
            parent.timeoutMedia();
          });
          isPaused = false;
        }
      };
    } catch (e) {
      console.error(e);
    }
  };
}());
  