/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */

/* global LayoutManager, Logger, utils, LibraryManager, WebOSManager, ERRORS */

(function () {
  "use strict";

  /**
   * Display a html package widget in layout
   * @param {object} media Media options
   * @param {HTMLElement} container The parent (HTML) element.
   * @param {funtion} callback Callback function to call on end.
   * @param {string} layoutid The layout id
   * @param {string} scheduleid The schedule id
   * @param {string} scp "Main" if is is a normal layout or "overlay" if it is an overlay layout.
   * @returns {LayoutManager.htmlpackage} An html package widget.
   */
  window.LayoutManager.htmlpackage = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);

      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }

      this.isready = true;
      this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
      //this.uri = "http://127.0.0.1:9080/" + media.media.options[0].uri[0].jValue;
      this.uri = this.uri.substr(0, this.uri.length - 4);


      if (media.media.options[0].hasOwnProperty('nominatedFile')) {
        this.uri += "/" + decodeURIComponent(media.media.options[0].nominatedFile[0].jValue);
      } else {
        this.uri += "/index.html";
      }

      this.statsStartTime = utils.getISODate();
      this.duration = parseInt(media.duration) * 1000;
      var duration = this.duration;
      var isPaused = false;
      var timeStarted = 0;
      var timePlayed = 0;

      var parent = this;
      this.id = media.media.options[0].uri[0].jValue.substr(0, media.media.options[0].uri[0].jValue.length - 4);

      var v_touchOverlayElements = [];


      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        parent.dispatchEvent(widgetendevt);
      };


      var preparehtmlpackage = function (cbReady) {
        var ifrm = document.createElement("iframe");

        ifrm.onload = function (r1) {
          cbReady(ifrm);
        };
        parent.elem = ifrm;

        ifrm.src = parent.uri;
        ifrm.style.position = "absolute";
        ifrm.style.left = 0;
        ifrm.style.top = 0;
        ifrm.style.width = "100%";
        ifrm.style.height = "100%";
        ifrm.style.overflow = "hidden";
        ifrm.style.border = "none";
        if (hastransIn) {
          parent.elem.style.visibility = "hidden"
        }

        container.insertBefore(parent.elem, container.firstElementChild);

      };

      this.timeoutElement = null;
      this.removeMedia = function () {
        container.removeChild(parent.elem);
      };

      var hastransIn = false;
      var hastransOut = false;
      var transInDuration = 1000;
      var transOutDuration = 1000;
      var transInType = null;
      var transOutType = null;
      var transInDirection = "S";
      var transOutDirection = "S";

      if (media.media.options[0].hasOwnProperty("transIn")) {
        hastransIn = true;
        transInType = media.media.options[0].transIn[0].jValue;
        if (media.media.options[0].transInDirection) {
          transInDirection = media.media.options[0].transInDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transInDuration")) {
          transInDuration = media.media.options[0].transInDuration[0].jValue;
        }
      }
      if (media.media.options[0].hasOwnProperty("transOut")) {
        hastransOut = true;
        transOutType = media.media.options[0].transOut[0].jValue;
        if (media.media.options[0].transOutDirection) {
          transOutDirection = media.media.options[0].transOutDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transOutDuration")) {
          transOutDuration = media.media.options[0].transOutDuration[0].jValue;
        }
      }


      var widgetExpired = function () {
        if (hastransOut) {
          LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            if (scheduleid && media.enableStat !== "0") {
              Logger.logStat({
                fromdt: parent.statsStartTime,
                todt: utils.getISODate(),
                scheduleid: scheduleid,
                layoutid: layoutid,
                mediaid: media.id
              });
            }
//            callback();
            widgetEnded();
          });
        } else {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
//          callback();
          widgetEnded();
        }
      };

      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();

        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      /**
       * Display the media on the layout.
       * @returns {undefined}
       */
      this.displayThisMedia = function () {
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            var v_domActionOverlay = undefined;
            if (action.jAttr.triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
              v_domActionOverlay = document.createElement("div");
              v_domActionOverlay.id = action.jAttr.id;
              v_domActionOverlay.style.position = "absolute";
              v_domActionOverlay.style.width = container.style.width;
              v_domActionOverlay.style.height = container.style.height;
              v_domActionOverlay.style.top = container.style.top;
              v_domActionOverlay.style.left = container.style.left;
              container.parentElement.appendChild(v_domActionOverlay);
              v_touchOverlayElements.push(v_domActionOverlay);
            }
            var act = action.jAttr;
            InteractiveManager.addEventAction(v_domActionOverlay, act.id, act.triggerType, act.triggerCode,
              act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
              act.targetId, act.widgetId);
          });
        }
        if (hastransIn) {
          duration -= parseInt(transInDuration);
          utils.hide(parent.elem);
        }
        if (hastransOut) {
          duration -= parseInt(transOutDuration);
        }
        if (duration < 0) {
          duration = 0;
        }
        if (container.childElementCount > 1) {
          container.removeChild(container.lastElementChild);
        }
        if (hastransIn) {
          parent.elem.style.visibility = "visible"
          LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function () {
            timeoutMediaInvoke(parent);
          });
        } else {
          timeoutMediaInvoke(parent);
        }
      };

      this.stopMediaTimeout = function () {
        if (parent.audiooverlay) {
          parent.audiooverlay.stop();
        }
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            InteractiveManager.removeEventAction(action.jAttr.id);
          });
          v_touchOverlayElements.forEach(function (elem) {
            container.parentElement.removeChild(elem);
          });
          v_touchOverlayElements = [];
        }
        clearTimeout(parent.timeoutElement);
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };


      preparehtmlpackage(function () {
        parent.displayThisMedia();
      });

      /**
       * Pause the htmlpackage widget. The htmlpackage element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          container.removeChild(parent.elem);
          //Calculate remain duration
          timePlayed += (new Date()).valueOf() - timeStarted;
          duration -= (new Date()).valueOf() - timeStarted;
          clearTimeout(parent.timeoutElement);
          parent.timeoutElement = null;
          isPaused = true;
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
            v_touchOverlayElements.forEach(function (elem) {
              container.parentElement.removeChild(elem);
            });
            v_touchOverlayElements = [];
          }
        }
      };

      /**
       * Resume the htmlpackage widget. The htmlpackage element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          parent.elem = preparehtmlpackage(function (v, dur) {
            parent.elem = v;
            if (media.media.action) {
              media.media.action.forEach(function (action) {
                var v_domActionOverlay = undefined;
                if (action.jAttr.triggerType === InteractiveManager.TRIGGERTYPE_TOUCH) {
                  v_domActionOverlay = document.createElement("div");
                  v_domActionOverlay.id = action.jAttr.id;
                  v_domActionOverlay.style.position = "absolute";
                  v_domActionOverlay.style.width = container.style.width;
                  v_domActionOverlay.style.height = container.style.height;
                  v_domActionOverlay.style.top = container.style.top;
                  v_domActionOverlay.style.left = container.style.left;
                  container.parentElement.appendChild(v_domActionOverlay);
                  v_touchOverlayElements.push(v_domActionOverlay);
                }
                var act = action.jAttr;
                InteractiveManager.addEventAction(v_domActionOverlay, act.id, act.triggerType, act.triggerCode,
                  act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
                  act.targetId, act.widgetId);
              });
            }
            timeoutMediaInvoke(parent);
          });
          isPaused = false;
        }
      };
    } catch (e) {

      Logger.cmsFault(undefined, undefined,
        {
          code: ERRORS.FaultCodes.RemoteResourceFailed,
          reason: "Problem displaying html package with id " + media.media.options[0].uri[0].jValue + " on layout with Id " + layoutid + ". Error:" + {
            name: e.name,
            message: e.message
          },
          method: "factory",
          thread: "LayoutManager.htmlpackage",
          date: new Date(),
          expires: new Date((new Date()).valueOf() + WebOSManager.Config.getOptions().collectInterval * 1000),
          onlyOnce: true
        }
      );
      widgetEnded();
//      callback();
    }
  };
}());
