/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */

/* global LayoutManager, Logger, utils */

(function () {
  "use strict";

  window.LayoutManager.image = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);

      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }

      this.statsStartTime = utils.getISODate();
      this.hashcode = Math.random();
      this.isready = false;
      this.align = null;
      this.valign = null;
      this.scaleType = null;
      if (media.media.options[0].align !== undefined) {
        this.align = media.media.options[0].align[0].jValue;
      } else {
        this.align = 'center';
      }
      if (media.media.options[0].valign !== undefined) {
        this.valign = media.media.options[0].valign[0].jValue;
      } else {
        this.valign = 'middle';
      }
      if (media.media.options[0].scaleType !== undefined) {
        this.scaleType = media.media.options[0].scaleType[0].jValue;
      } else {
        this.scaleType = 'center';
      }

      this.duration = parseInt(media.duration);
      var duration = this.duration * 1000;
      var timeStarted = 0;
      var timePlayed = 0;
      var parent = this;
      var isPaused = false;
      var isFullscreen = false;

      if (media.isbase64 === 1) {
        this.uri = media.media.options[0].uri[0].jValue;
      } else {
        this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;
      }

      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        parent.dispatchEvent(widgetendevt);
      };


      this.timeoutElement = null;
      this.removeMedia = function () {
        container.removeChild(parent.elem);
      };

      this.displayThisMedia = function () {
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            var act = action.jAttr;
            InteractiveManager.addEventAction(parent.elem, act.id, act.triggerType, act.triggerCode,
              act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
              act.targetId, act.widgetId);
          });
        }
        if (hastransIn) {
          utils.hide(parent.elem);
        }
        //container.innerHTML = null;
        container.insertBefore(parent.elem, container.firstElementChild);
        if (container.childElementCount > 1) {
          container.removeChild(container.lastElementChild);
        }


        //    container.appendChild(this.elem);
        if (hastransIn) {
          parent.elem.style.visibility = "visible"
          LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function () {
            timeoutMediaInvoke(parent);
          });
        } else {
          timeoutMediaInvoke(parent);
        }
      };


      var prepareImage = function (cbReady) {
        var img = document.createElement("img");
        parent.elem = img;
        //this.elem.src = utils.downscaleImage(this.uri, container.clientWidth,"image/png",1);
        //        utils.downscaleImage(parent.uri, container.clientWidth, "image/png", 1, function (data) {
        //          img.src = data;
        //        });
        img.src = parent.uri;
        img.setAttribute("scaleType", parent.scaleType);
        img.setAttribute("containerWidth", container.clientWidth);
        img.setAttribute("containerHeight", container.clientHeight);
        img.setAttribute("alignElement", parent.align);
        img.setAttribute("valignElement", parent.valign);
        if (hastransIn) {
          parent.elem.style.visibility = "hidden"
        }
        img.onerror = function () {
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
          }
          parent.timeoutElement = setTimeout(function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            //            callback();
            widgetEnded();
          }, 5000, layoutid, scp);
        };

        img.onload = function () {

          if (img.getAttribute('scaleType') === "stretch") {
            img.style.width = "100%";
            img.style.height = "100%";
          } else if (img.getAttribute('scaleType') === "fit") {
            container.style.overflow = "hidden";
            img.style.objectFit = "cover";
            img.style.width = "100%";
            img.style.height = "100%";
          } else {
            var containerwidth = parseInt(img.getAttribute('containerWidth'));
            var containerheight = parseInt(img.getAttribute('containerHeight'));
            var widthRatio = containerwidth / img.width;
            var heightRatio = containerheight / img.height;
            var resizeRatio = Math.min(widthRatio, heightRatio);
            var newWidth = Math.ceil(resizeRatio * img.width);
            var newHeight = Math.ceil(resizeRatio * img.height);
            img.style.width = newWidth + "px";
            img.style.height = newHeight + "px";
            if (img.getAttribute('valignElement') === 'middle') {
              img.style.position = 'absolute';
              if (img.getAttribute('alignElement') === 'left') {
                img.style.left = 0;
                img.style.top = ((parseInt(container.style.height) - parseInt(img.style.height)) / 2) + "px";
              } else if (img.getAttribute('alignElement') === 'right') {
                img.style.left = (parseInt(container.style.width) - parseInt(img.style.width)) + "px";
                img.style.top = ((parseInt(container.style.height) - parseInt(img.style.height)) / 2) + "px";
              } else {
                img.style.left = ((parseInt(container.style.width) - parseInt(img.style.width)) / 2) + "px";
                img.style.top = ((parseInt(container.style.height) - parseInt(img.style.height)) / 2) + "px";
              }
            } else if (img.getAttribute('valignElement') === 'top') {
              if (img.getAttribute('alignElement') === 'left') {
                img.style.left = 0;
                img.style.top = 0;
              } else if (img.getAttribute('alignElement') === 'right') {
                img.style.left = (parseInt(container.style.width) - parseInt(img.style.width)) + "px";
                img.style.top = 0;
              } else {
                img.style.left = ((parseInt(container.style.width) - parseInt(img.style.width)) / 2) + "px";
                img.style.top = 0;
              }
            } else if (img.getAttribute('valignElement') === 'bottom') {
              if (img.getAttribute('alignElement') === 'left') {
                img.style.left = 0;
                img.style.top = (parseInt(container.style.height) - parseInt(img.style.height)) + "px";
              } else if (img.getAttribute('alignElement') === 'right') {
                img.style.right = 0;
                img.style.top = (parseInt(container.style.height) - parseInt(img.style.height)) + "px";
              } else {
                img.style.left = ((parseInt(container.style.width) - parseInt(img.style.width)) / 2) + "px";
                img.style.top = (parseInt(container.style.height) - parseInt(img.style.height)) + "px";
              }
            }
          }
          cbReady(img);
        };
      };

      var hastransIn = false;
      var hastransOut = false;
      var transInDuration = 1000;
      var transOutDuration = 1000;
      var transInType = null;
      var transOutType = null;
      var transInDirection = "S";
      var transOutDirection = "S";
      if (media.media.options[0].hasOwnProperty("transIn")) {
        hastransIn = true;
        transInType = media.media.options[0].transIn[0].jValue;
        if (media.media.options[0].transInDirection) {
          transInDirection = media.media.options[0].transInDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transInDuration")) {
          transInDuration = media.media.options[0].transInDuration[0].jValue;
        }
      }
      if (media.media.options[0].hasOwnProperty("transOut")) {
        hastransOut = true;
        transOutType = media.media.options[0].transOut[0].jValue;
        if (media.media.options[0].transOutDirection) {
          transOutDirection = media.media.options[0].transOutDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transOutDuration")) {
          transOutDuration = media.media.options[0].transOutDuration[0].jValue;
        }
      }
      if (hastransIn) {
        duration -= parseInt(transInDuration);
      }
      if (hastransOut) {
        duration -= parseInt(transOutDuration);
      }
      if (duration < 0) {
        duration = 0;
      }

      var widgetExpired = function () {
        if (hastransOut) {
          LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            if (scheduleid && media.enableStat !== "0") {
              Logger.logStat({
                fromdt: parent.statsStartTime,
                todt: utils.getISODate(),
                scheduleid: scheduleid,
                layoutid: layoutid,
                mediaid: media.id
              });
            }
            //            callback();
            widgetEnded();
          });
        } else {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
          //          callback();
          widgetEnded();
        }
      };

      //var tag = "r: " + container.getAttribute("id") + " m: " + media.id + " image";
      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      this.stopMediaTimeout = function () {
        if (parent.audiooverlay) {
          parent.audiooverlay.stop();
        }
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            InteractiveManager.removeEventAction(action.jAttr.id);
          });
        }
        clearTimeout(parent.timeoutElement);
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Pause the image widget. The image element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          //Pause and hide video element
          container.removeChild(parent.elem);
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
          }
          if (isFullscreen) {
            restoreSize();
          }
          //Calculate remain duration
          timePlayed += (new Date()).valueOf() - timeStarted;
          duration -= (new Date()).valueOf() - timeStarted;
          clearTimeout(parent.timeoutElement);
          parent.timeoutElement = null;
          isPaused = true;
        }
      };

      /**
       * Resume the image widget. The image element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          parent.elem = prepareImage(function (v, dur) {
            parent.elem = v;
            container.insertBefore(parent.elem, container.firstElementChild);
            if (container.childElementCount > 1) {
              container.removeChild(container.lastElementChild);
            }
            parent.timeoutMedia();
          });
          isPaused = false;
        }
      };

      prepareImage(function () {
        parent.displayThisMedia(parent);
      });

    } catch (e) {
      console.error(e);
    }
  };
}());
  