/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */

/* global LayoutManager, Logger, utils */
(function () {
  "use strict";
  window.LayoutManager.localvideo = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);
      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }

      var isPaused = false;
      var runOnce = true;
      var isFullscreen = false;
      var previousRegionSize = {};
      //XLG-118: Video Full Screen Option
      if (media.media.options[0].hasOwnProperty("showFullScreen")) {
        if (media.media.options[0].showFullScreen[0].jValue[0] === "1") {
          isFullscreen = true;
          previousRegionSize = {
            width: container.style.width,
            height: container.style.height,
            top: container.style.top,
            left: container.style.left,
            zIndex: container.style.zIndex
          };
        }
      }
      var restoreSize = function () {
        container.style.width = previousRegionSize.width;
        container.style.height = previousRegionSize.height;
        container.style.top = previousRegionSize.top;
        container.style.left = previousRegionSize.left;
        container.style.zIndex = previousRegionSize.zIndex;
      };

      var setFullScreen = function () {
        container.style.width = "100%";
        container.style.height = "100%";
        container.style.top = "0";
        container.style.left = "0";
        container.style.zIndex = 99999999;
      };

      this.statsStartTime = utils.getISODate();
      this.isready = false;
      this.uri = decodeURIComponent(media.media.options[0].uri[0].jValue);
      var re = /(?:\.([^.]+))?$/;
      var mediatype = this.uri.substr(0, this.uri.indexOf(':'));
      var ext = re.exec(media.media.options[0].uri[0].jValue)[1];
      this.duration = parseInt(media.duration) * 1000;
      var parent = this;
      var duration = this.duration;
      var timeStarted = 0;
      var timePlayed = 0;
      var isLiveStream = false;
      var source = document.createElement("source");
      if ((mediatype === "rtsp") || (mediatype === "udp") || (mediatype === "rtp")) {
        isLiveStream = true;
        var mediacodec = "mp4";
        var options = {
          htmlMediaOption: {
            useUMSMediaInfo: true
          },
          mediaTransportType: mediatype.toUpperCase()
        };

        source.setAttribute('src', this.uri);
        //      if ("mpegts" === mediacodec) {
        //        source.setAttribute('type', 'video/' + mediacodec + ';mediaOption=' + escape(JSON.stringify(options)));
        //      } else if (("mp4" === mediacodec) || ("mp2t" === mediacodec)) {
        source.setAttribute('type', 'video/' + mediacodec + ';mediaOption=' + escape(JSON.stringify(options)));
        //      }

      } else {
        source.setAttribute('src', this.uri);
      }

      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        parent.dispatchEvent(widgetendevt);
      };


      var prepareLocalVideo = function (cbReady) {
        var v = document.createElement("video");
        parent.elem = v;
        parent.elem.setAttribute('autoplay', '');
        parent.elem.setAttribute('texture', '');
        parent.elem.setAttribute('preload', 'metadata');
        //if (media.media.options[0].loop !== undefined) { if (media.media.options[0].loop[0].jValue === '1') { this.elem.setAttribute('loop',''); } }
        parent.elem.setAttribute('loop', '');
        if (media.media.options[0].mute !== undefined) {
          if (media.media.options[0].mute[0].jValue === '1') {
            parent.elem.muted=true;
          }
        }
        parent.elem.style.left = "0";
        parent.elem.style.top = "0";
        parent.elem.style.width = "100%";
        parent.elem.style.height = "100%";
        if (media.media.options[0].scaleType !== undefined) {
          if (media.media.options[0].scaleType[0].jValue === "aspect") {
            parent.elem.style.objectFit = "contain";
          } else if (media.media.options[0].scaleType[0].jValue === "stretch") {
            parent.elem.style.objectFit = "fill";
          } else {
            parent.elem.style.objectFit = "contain";
          }
        } else {
          parent.elem.style.objectFit = "contain";
        }

        parent.elem.appendChild(source);
        parent.elem.onerror = function (e) {
          parent.isready = true;
          console.error("Video Error.", e);
        };
        parent.elem.addEventListener("canplay", function (r) {
          if (runOnce) {
            runOnce = false;
            parent.isready = true;
            cbReady(v);
          }
        });
        // parent.elem.oncanplay = function (r) {
        // if (runOnce) {
        //   runOnce = false;
        //   parent.isready = true;
        //   cbReady(v);
        // }
        // };
        parent.elem.load();
      };

      var widgetExpired = function () {
        if (hastransOut) {
          LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            if (scheduleid && media.enableStat !== "0") {
              Logger.logStat({
                fromdt: parent.statsStartTime,
                todt: utils.getISODate(),
                scheduleid: scheduleid,
                layoutid: layoutid,
                mediaid: media.id
              });
            }
            if (isFullscreen) {
              restoreSize();
            }

            widgetEnded();
          });
        } else {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }

          if (isFullscreen) {
            restoreSize();
          }
          //            callback();
          widgetEnded();
        }
      };

      this.timeoutElement = null;
      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      this.removeMedia = function () {
        container.removeChild(parent.elem);
      };
      var hastransIn = false;
      var hastransOut = false;
      var transInDuration = 1000;
      var transOutDuration = 1000;
      var transInType = null;
      var transOutType = null;
      var transInDirection = "S";
      var transOutDirection = "S";
      if (media.media.options[0].hasOwnProperty("transIn")) {
        hastransIn = true;
        transInType = media.media.options[0].transIn[0].jValue;
        if (media.media.options[0].transInDirection) {
          transInDirection = media.media.options[0].transInDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transInDuration")) {
          transInDuration = media.media.options[0].transInDuration[0].jValue;
        }
      }
      if (media.media.options[0].hasOwnProperty("transOut")) {
        hastransOut = true;
        transOutType = media.media.options[0].transOut[0].jValue;
        if (media.media.options[0].transOutDirection) {
          transOutDirection = media.media.options[0].transOutDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transOutDuration")) {
          transOutDuration = media.media.options[0].transOutDuration[0].jValue;
        }
      }
      this.displayThisMedia = function () {
        if (!isPaused) {
          //container.innerHTML = null;
          if (isFullscreen) {
            setFullScreen();
          }
          container.appendChild(parent.elem);
          if (container.childElementCount > 1) {
            container.removeChild(container.firstChild);
          }
          timeoutMediaInvoke(parent);
        }
      };


      /**
       * Check if remote video is available.
       * @param {function} cbDone Callback function after it sorts out availability. True if video is fine else false.
       */
      var checkVideoAvailability = function (cbDone) {
        var xhr = new XMLHttpRequest();
        xhr.open('HEAD', parent.uri, false);
        var xhrerror = false;
        try {
          xhr.send();
        } catch (err) {
          //        console.warn(err);
          xhrerror = true;
        }
        var ret = !((xhr.status >= 400) & (xhr.status <= 404) || (xhrerror));
        cbDone(ret);
      };

      //this.elem.load();
      this.stopMediaTimeout = function () {
        //      console.debug("Timeout for " + media.id + " cleared!");
        if (parent.audiooverlay) {
          parent.audiooverlay.stop();
        }
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            InteractiveManager.removeEventAction(action.jAttr.id);
          });
        }
        clearTimeout(parent.timeoutElement);
        if (isFullscreen) {
          restoreSize();
        }
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Pause the localvideo widget. The video element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          if (container.contains(parent.elem)) {
            container.removeChild(parent.elem);
            if (isFullscreen) {
              restoreSize();
            }
            // Remove oncanplay event
            parent.elem.oncanplay = undefined;
            //Calculate remain duration
            timePlayed += (new Date()).valueOf() - timeStarted;
            duration -= (new Date()).valueOf() - timeStarted;
            clearTimeout(parent.timeoutElement);
            parent.timeoutElement = null;
          }
          isPaused = true;
        }
      };

      /**
       * Resume the localvideo widget. The video element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          isPaused = false;
          runOnce = true;
          parent.elem = prepareLocalVideo(function (v, dur) {
            parent.elem = v;
            v.currentTime = timePlayed / 1000;
            //placeIntoContainer(v);
            if (media.media.action) {
              media.media.action.forEach(function (action) {
                var act = action.jAttr;
                InteractiveManager.addEventAction(parent.elem, act.id, act.triggerType, act.triggerCode,
                  act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
                  act.targetId, act.widgetId);
              });
            }

            parent.displayThisMedia(parent);
          });
        }
      };
      if (hastransIn) {
        parent.elem.style.visibility = "hidden"
      }
      if (isLiveStream) {
        prepareLocalVideo(function (ret) {
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              var act = action.jAttr;
              InteractiveManager.addEventAction(parent.elem, act.id, act.triggerType, act.triggerCode,
                act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
                act.targetId, act.widgetId);
            });
          }
          //        parent.elem.load();
          parent.elem.play();
          if (hastransIn) {
            parent.elem.style.visibility = "visible"
            LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function () {
              parent.displayThisMedia(parent);
            });
          } else {
            parent.displayThisMedia(parent);
          }
        });
        parent.elem.removeEventListener("canplay", LayoutManager.localvideo)
      } else {
        checkVideoAvailability(function (isAvailable) {
          if (isAvailable) {
            prepareLocalVideo(function (ret) {
              if (media.media.action) {
                media.media.action.forEach(function (action) {
                  var act = action.jAttr;
                  InteractiveManager.addEventAction(parent.elem, act.id, act.triggerType, act.triggerCode,
                    act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
                    act.targetId, act.widgetId);
                });
              }
              if (duration === 0) {
                duration = parent.elem.children[0].duration * 1000;
              }
              if (hastransIn) {
                duration -= parseInt(transInDuration);
              }
              if (hastransOut) {
                duration -= parseInt(transOutDuration);
              }
              if (duration < 0) {
                duration = 0;
              }
              if (hastransIn) {
                LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function () {
                  parent.displayThisMedia(parent);
                });
              } else {
                parent.displayThisMedia(parent);
              }
            });
          } else {
            Logger.cmsLog({
              category: "audit",
              message: "Could not play local video: Widget ID " + media.id + " stored as " + media.media.options[0].uri[0].jValue + " on Layout ID " + layoutid + ", in Region ID " + container.id + ". ",
              layoutID: layoutid,
              mediaID: media.id,
              method: "checkVideoAvailability",
              thread: "LayoutManager.localvideo",
              onlyOnce: true
            });
            //            setTimeout(callback, 0);
            setTimeout(widgetEnded, 0);
          }
        });
      }
    } catch (e) {
      console.error(e);
    }
  };
}());
  