/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */

/* global LayoutManager, ScheduleManager, utils, Logger */

(function () {
  "use strict";


  /**
   * Create a shellcommand. Shell commands can be regular
   * @param {type} media
   * @param {type} container If container is "screen" then this is an action command
   * @param {type} callback
   * @param {type} layoutid
   * @param {type} scheduleid
   * @param {type} scp
   * @returns {LayoutManager.shellcommand}
   */
  LayoutManager.shellcommand = function (media, container, callback, layoutid, scheduleid, scp) {
    try {
      EventEmitter.call(this);


      this.statsStartTime = utils.getISODate();
      var parent = this;

      var commandType = "storedCommand";
      var useGlobalCommand = false;

      // TODO: Check what will happen if useGlobalCommand is true
      if (media.media.options[0].commandType) {
        commandType = media.media.options[0].commandType[0].jValue;
      }

      // TODO: Check what will happen if useGlobalCommand is true
      if (media.media.options[0].useGlobalCommand) {
        useGlobalCommand = media.media.options[0].useGlobalCommand[0].jValue;
      }


      var command = undefined;

      if (commandType === "createCommand") {
        if (useGlobalCommand === "0") {
          command = decodeURIComponent(media.media.options[0].webosCommand[0].jValue);
        } else {
          command = decodeURIComponent(media.media.options[0].globalCommand[0].jValue);
        }
      } else {
        command = media.media.options[0].commandCode[0].jValue;
      }

      this.duration = parseInt(media.duration) * 1000;
      var duration = this.duration;

      if (commandType === "createCommand") {
        console.info('Shellcommand: Run a shell command action');

        if (command.startsWith('http')) { //execute http command
          var param = command.split('|');
          if (param.length === 4) {
            utils.executeHTTPCommand(param[1], param[2], param[3]);
          } else {
            console.error("Incorect number of Shell command HTTP parameters.");
          }
        }
      } else {
        utils.empty(container);
        ScheduleManager.Commands.commandAction(command);
      }

      this.timeoutElement = null;
      this.timeoutMedia = function () {
        parent.timeoutElement = setTimeout(function () {
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
          callback();
        }, duration, layoutid, scp);
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      timeoutMediaInvoke(parent);
      this.stopMediaTimeout = function () {
        clearTimeout(parent.timeoutElement);
      };
    } catch (e) {
      console.error(e);
    }
  };

}());
