/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -  Elias Papadakis <ep@intelligentmedia.gr>
 */

/* global LayoutManager, Logger, utils, LibraryManager, WebOSManager, MediaError */
(function () {
  "use strict";

  var videoPlaybackError = [
    "MEDIA_ERR_ABORTED",
    "MEDIA_ERR_NETWORK",
    "MEDIA_ERR_DECODE",
    "MEDIA_ERR_SRC_NOT_SUPPORTED"
  ];
  window.LayoutManager.video = function (media, container, callback, layoutid, scheduleid, scp) {
    try {

      EventEmitter.call(this);

      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }

      this.isready = false;
      this.usePreload = false;
      this.uri = "file:///opt/usr/home/owner/content/Downloads/" + media.media.options[0].uri[0].jValue;

      var isPaused = false;
      var v = document.createElement("video");

      var isFullscreen = false;
      var previousRegionSize = {};
      //XLG-118: Video Full Screen Option
      if (media.media.options[0].hasOwnProperty("showFullScreen")) {
        if (media.media.options[0].showFullScreen[0].jValue[0] === "1") {
          isFullscreen = true;
          previousRegionSize = {
            width: container.style.width,
            height: container.style.height,
            top: container.style.top,
            left: container.style.left,
            zIndex: container.style.zIndex
          };
        }
      }
      var restoreSize = function () {
        container.style.width = previousRegionSize.width;
        container.style.height = previousRegionSize.height;
        container.style.top = previousRegionSize.top;
        container.style.left = previousRegionSize.left;
        container.style.zIndex = previousRegionSize.zIndex;
      };
      var setFullScreen = function () {
        container.style.width = "100%";
        container.style.height = "100%";
        container.style.top = "0";
        container.style.left = "0";
        container.style.zIndex = 99999999;
      };

      this.statsStartTime = utils.getISODate();


      this.duration = parseInt(media.duration) * 1000;
      var parent = this;
      var duration = this.duration;
      var timeStarted = 0;
      var timePlayed = 0;

      this.timeoutElement = null;

      //Event emmiters
      var widgetStarted = function () {
        var widgetstartedevt = new CustomEvent('widgetStarted', {
          detail: {}
        });
        parent.dispatchEvent(widgetstartedevt);
      };
      var widgetEnded = function () {
        var widgetendevt = new CustomEvent('widgetEnded', {
          detail: {}
        });
        //v.removeEventListener("canplay", function() {cbReady(v, v.duration)});
        v.pause();
        v.removeAttribute('src'); // empty source
        //   v.setAttribute('src', "");
        v.load();
        parent.dispatchEvent(widgetendevt);
      };

      var prepareVideo = function (cbReady) {
        console.debug('Prepare video', media.media.options[0].uri[0].jValue, parent.myID);
        //var v = document.createElement("video");
        v.setAttribute('autoplay', '');
        v.setAttribute('texture', '');
        v.setAttribute('preload', 'metadata');
        v.setAttribute('src', parent.uri);
        v.load();

        v.onloadeddata = function () {
          //cbReady(v, v.duration);
        };

        v.addEventListener("canplay", function () {
          console.log("can play fired", v, parent.uri);
          cbReady(v, v.duration)
        });


        if ((media.media.options[0].loop !== undefined) && (media.media.options[0].loop[0].jValue === '1')) {
          v.setAttribute('loop', '');
        }
        if ((media.media.options[0].mute !== undefined) && (media.media.options[0].mute[0].jValue === '1')) {
          v.muted=true;
        }
        v.style.left = "0";
        v.style.top = "0";
        v.style.width = "100%";
        v.style.height = "100%";
        if (media.media.options[0].scaleType !== undefined) {
          if (media.media.options[0].scaleType[0].jValue === "aspect") {
            v.style.objectFit = "contain";
          } else if (media.media.options[0].scaleType[0].jValue === "stretch") {
            v.style.objectFit = "fill";
          } else {
            v.style.objectFit = "contain";
          }
        } else {
          v.style.objectFit = "contain";
        }

        v.onerror = function (err) {
          console.error(err);
          parent.isready = true;
          if (isFullscreen) {
            restoreSize();
          }
          //console.debug("Video Error.");
          LibraryManager.blacklisted_widgets.push(media.media.jAttr.fileId);
          var msg = "Could not play video: Widget ID " + media.id + " stored as " + media.media.options[0].uri[0].jValue + " on Layout ID " + layoutid + ", in Region ID " + container.id + ". ";
          if (err.currentTarget.error && err.currentTarget.error.constructor === MediaError) {
            msg += "Media Error : " + videoPlaybackError[err.currentTarget.error.code - 1];
          }
          Logger.cmsLog({
            category: "audit",
            message: msg,
            layoutID: layoutid,
            mediaID: media.id,
            method: "onerror",
            thread: "LayoutManager.video",
            onlyOnce: true
          });
          //          callback(); 
          widgetEnded();
        };
        v.onload = function () {
          //parent.isready = true; 
        };
        return v;
      };


      // Duration expired
      var widgetExpired = function () {
        //console.info("VDO: Video timeoutMedia fired!");
        if (hastransOut) {
          LayoutManager.transout(parent.elem, container, transOutType, transOutDuration, transOutDirection, function () {
            if (parent.audiooverlay) {
              parent.audiooverlay.stop();
            }
            //container.innerHTML = null;
            //parent = null;
            if (scheduleid && media.enableStat !== "0") {
              Logger.logStat({
                fromdt: parent.statsStartTime,
                todt: utils.getISODate(),
                scheduleid: scheduleid,
                layoutid: layoutid,
                mediaid: media.id
              });
            }
            if (isFullscreen) {
              restoreSize();
            }
            //                callback(); 
            widgetEnded();
          });
        } else {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          //container.innerHTML = null;
          //parent = null;
          if (scheduleid && media.enableStat !== "0") {
            Logger.logStat({
              fromdt: parent.statsStartTime,
              todt: utils.getISODate(),
              scheduleid: scheduleid,
              layoutid: layoutid,
              mediaid: media.id
            });
          }
          if (isFullscreen) {
            restoreSize();
          }

          //callback();
          widgetEnded();
        }
      };

      this.timeoutMedia = function () {
        timeStarted = (new Date()).valueOf();
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
        if (media.media.action) {
          media.media.action.forEach(function (action) {
            var act = action.jAttr;
            InteractiveManager.addEventAction(parent.elem, act.id, act.triggerType, act.triggerCode,
              act.actionType, act.layoutCode, act.source, act.sourceId, act.target,
              act.targetId, act.widgetId);

          });
        }
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      this.removeMedia = function () {
        //console.info("VDO: Remove video element!");
        container.removeChild(this.elem);
      };
      var hastransIn = false;
      var hastransOut = false;
      var transInDuration = 1000;
      var transOutDuration = 1000;
      var transInType = null;
      var transOutType = null;
      var transInDirection = "S";
      var transOutDirection = "S";
      if (media.media.options[0].hasOwnProperty("transIn")) {
        hastransIn = true;
        transInType = media.media.options[0].transIn[0].jValue;
        if (media.media.options[0].transInDirection) {
          transInDirection = media.media.options[0].transInDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transInDuration")) {
          transInDuration = media.media.options[0].transInDuration[0].jValue;
        }
      }
      if (media.media.options[0].hasOwnProperty("transOut")) {
        hastransOut = true;
        transOutType = media.media.options[0].transOut[0].jValue;
        if (media.media.options[0].transOutDirection) {
          transOutDirection = media.media.options[0].transOutDirection[0].jValue;
        }
        if (media.media.options[0].hasOwnProperty("transOutDuration")) {
          transOutDuration = media.media.options[0].transOutDuration[0].jValue;
        }
      }

      this.displayThisMedia = function () {
        //console.info("VDO: Display video element!");
        //if (parent.usePreload){
        if (container.childElementCount > 1) {
          parent.elem.onplay = function () {
            container.removeChild(container.lastElementChild);
          };
        }
        if (isFullscreen) {
          setFullScreen();
        }
        timeoutMediaInvoke(parent);
      };

      var placeIntoContainer = function (v) {
        //    // Issue #36: Consecutive videos are not shown due to firmware problem.
        if (!container) {
          return;
        }
        container.insertBefore(v, container.firstElementChild);
        console.log("Video placed into container", v);
        // if (container.children.length>0){
        // 	if (container.lastChild.tagName === 'VIDEO'){
        // 		//console.info("VDO: Previous media IS video. Don't use preload!");
        // 		container.innerHTML = null;
        // 		container.appendChild(v);
        // 	}else{
        // 		//console.info("VDO: Previous media not video. Use preload!");
        // 		parent.usePreload = true;
        // 		container.insertBefore(v, container.firstElementChild);
        // 	}
        // }else{
        // 	//console.info("VDO: No previous media. Append!");
        // 	container.appendChild(v);
        // }
      };
      this.elem = prepareVideo(function (v, dur) {
        parent.elem = v;
        if (duration === 0) {
          duration = parent.elem.duration * 1000;
        }
        if (hastransIn) {
          duration -= parseInt(transInDuration);
        }
        if (hastransOut) {
          duration -= parseInt(transOutDuration);
        }
        if (duration < 0) {
          duration = 0;
        }

        placeIntoContainer(v);

        if (hastransIn) {
          parent.elem.style.visibility = "visible"
          LayoutManager.transin(parent.elem, container, transInType, transInDuration, transInDirection, function () {
            parent.displayThisMedia();
          });
        } else {
          parent.displayThisMedia(parent);
        }

      });

      this.stopMediaTimeout = function () {
        //console.info("VDO: Clear video element timeout!", parent.timeoutElement);
        if (parent) {
          if (parent.audiooverlay) {
            parent.audiooverlay.stop();
          }
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
          }

          //            if (scheduleid){
          //                Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
          //            }
          clearTimeout(parent.timeoutElement);
          console.debug('Stop Media Timeout for element ', parent.timeoutElement);
          if (isFullscreen) {
            restoreSize();
          }

        }
        //parent = null;
      };

      /**
       * Expire media immediately
       */
      this.expireMedia = function () {
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = null;
        widgetExpired();
      };

      /**
       * Extend media duration
       * @param {Number} dur Duration to extend in seconds
       */
      this.extendDuration = function (dur) {
        duration += dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Set media duration
       * @param {Number} dur Duration to set in seconds
       */
      this.setDuration = function (dur) {
        duration = dur * 1000;
        clearTimeout(parent.timeoutElement);
        parent.timeoutElement = setTimeout(widgetExpired, duration, layoutid, scp);
      };

      /**
       * Pause the video widget. The video element will hide it self!
       * Duration will be retained too!
       */
      this.pause = function () {
        if (!isPaused) {
          //Pause and hide video element
          container.removeChild(parent.elem);
          if (isFullscreen) {
            restoreSize();
          }
          //Calculate remain duration
          timePlayed += (new Date()).valueOf() - timeStarted;
          duration -= (new Date()).valueOf() - timeStarted;
          clearTimeout(parent.timeoutElement);
          parent.timeoutElement = null;
          isPaused = true;
          if (media.media.action) {
            media.media.action.forEach(function (action) {
              InteractiveManager.removeEventAction(action.jAttr.id);
            });
          }
        }
      };

      /**
       * Resume the video widget. The video element will show it self!
       */
      this.resume = function () {
        if (isPaused) {
          parent.elem = prepareVideo(function (v, dur) {
            parent.elem = v;
            v.currentTime = timePlayed / 1000;
            if (isFullscreen) {
              setFullScreen();
            }
            placeIntoContainer(v);
            parent.displayThisMedia(parent);
          });
          isPaused = false;
        }
      };

      if (hastransIn) {
        utils.hide(this.elem);
      }

      // // Issue #36: Consecutive videos are not shown due to firmware problem.
      // if (!container){
      // 	return;
      // }
      // if (container.children.length>0){
      // 	if (container.lastChild.tagName === 'VIDEO'){
      // 		//console.info("VDO: Previous media IS video. Don't use preload!");
      // 		container.innerHTML = null;
      // 		container.appendChild(this.elem);
      // 	}else{
      // 		//console.info("VDO: Previous media not video. Use preload!");
      // 		this.usePreload = true;
      // 		container.insertBefore(this.elem,container.firstElementChild);
      // 	}
      // }else{
      // 	//console.info("VDO: No previous media. Append!");
      // 	container.appendChild(this.elem);
      // }

    } catch (e) {
      console.error(e);
    }
  };
}());
