/**
 * Copyright (C) Intelligent Media LTD - All Rights Reserved
 *
 * This source code is protected under international copyright law.  All rights
 * reserved and protected by the copyright holders.
 * This file is confidential and only available to authorized individuals with the
 * permission of the copyright holders.  If you encounter this file and do not have
 * permission, please contact the copyright holders and delete this file.
 *
 * Author : Apostolos Katranitsas <ak@intelligentmedia.gr>
 * Contributors :
 *   -
 */

/* global LayoutManager, DeviceManager, utils, Logger */
(function () {
  "use strict";

  window.LayoutManager.videoin = function (media, container, callback, layoutid, scheduleid, scp) {
    try {
      EventEmitter.call(this);
      if (media.media.audio !== undefined) {
        var audio = {
          uri: "file:///opt/usr/home/owner/content/Downloads/" + media.media.audio[0].uri[0].jValue,
          loop: media.media.audio[0].uri[0].jAttr.loop,
          volume: media.media.audio[0].uri[0].jAttr.volume
        };
        this.audiooverlay = new LayoutManager.audioOverlay(audio, container);
        this.audiooverlay.preload(function () {
          parent.audiooverlay.play();
        }, null);
      }
      var isFullscreen = false;
      var previousRegionSize = {};
      //XLG-118: Video Full Screen Option
      if (media.media.options[0].hasOwnProperty("showFullScreen")) {
        if (media.media.options[0].showFullScreen[0].jValue[0] === "1") {
          isFullscreen = true;
          previousRegionSize = {
            width: container.style.width,
            height: container.style.height,
            top: container.style.top,
            left: container.style.left,
            zIndex: container.style.zIndex
          };
          container.style.width = "100%";
          container.style.height = "100%";
          container.style.top = "0";
          container.style.left = "0";
          container.style.zIndex = 99999999;
        }
      }
      var restoreSize = function () {
        container.style.width = previousRegionSize.width;
        container.style.height = previousRegionSize.height;
        container.style.top = previousRegionSize.top;
        container.style.left = previousRegionSize.left;
        container.style.zIndex = previousRegionSize.zIndex;
      };


      var lastError;
      this.statsStartTime = utils.getISODate();
      var inputType = media.media.options[0].sourceId[0].jValue;
      this.timeoutElement = null;
      var parent = this;
      parent.duration = parseInt(media.duration) * 1000;
      parent.elem = null;
      container.innerHTML = null;
      this.timeoutMedia = function () {
        parent.timeoutElement = setTimeout(
          function () {
            if (isFullscreen) {
              restoreSize();
            }
            DeviceManager.Device.Inputs.hideExternalInput(
              function () {
                window.LayoutManager.videoin.lastError = {};
                if (scheduleid && media.enableStat !== "0") {
                  Logger.logStat({
                    fromdt: parent.statsStartTime,
                    todt: utils.getISODate(),
                    scheduleid: scheduleid,
                    layoutid: layoutid,
                    mediaid: media.id
                  });
                }
                callback();
              }, function (e) {
                console.error(e);
              }
            );
          },
          parent.duration,
          layoutid,
          scp
        );
      };
      var timeoutMediaInvoke = this.timeoutMedia;
      this.hideMedia = function () {
        DeviceManager.Device.Inputs.hideExternalInput(function () {
          console.log("External input hid succesfully");
        }, function (e) {
          console.log("An error occured when hiding external input", e);
        })
      };

      this.showMedia = function () {
        DeviceManager.Device.Inputs.playExternalInput(function () {
            console.log("External input showed succesfully");
          }, function (e) {
            console.log("An error occured when showing external input", e);
          },
          {divId: container.id, sourceName: inputType});
      };

      this.removeMedia = function () {
        container.removeChild(parent.elem);
      };
      this.stopMediaTimeout = function () {
        if (parent.audiooverlay) {
          parent.audiooverlay.stop();
        }
        DeviceManager.Device.Inputs.hideExternalInput(function (e) {
          console.log("videoIn region removed successfully")
        }, function () {
          console.log("videoIn region cannot be removed")
        });
        //console.info("VDOIN: Clear video in element timeout!", parent.timeoutElement);
        //			if (scheduleid){
        //	            Logger.logStat({fromdt: parent.statsStartTime, todt: utils.getISODate(), scheduleid: scheduleid, layoutid: layoutid, mediaid:media.id});
        //	        }
        if (parent) {
          clearTimeout(parent.timeoutElement);
          //console.debug('Stop Media Timeout for element ',parent.timeoutElement);
        }
        //parent = null;
        if (isFullscreen) {
          restoreSize();
        }
      };
      var videoinloaded = function () {
        parent.elem = container.firstChild;
        timeoutMediaInvoke(parent);
      };
      var videoloadfailed = function (cbObj) {
        lastError = window.LayoutManager.videoin.lastError;
        if (lastError.message !== cbObj.message ||
          lastError.layoutID !== layoutid ||
          lastError.mediaID !== media.id) {
          window.LayoutManager.videoin.lastError.message = cbObj.message;
          window.LayoutManager.videoin.lastError.layoutID = layoutid;
          window.LayoutManager.videoin.lastError.mediaID = media.id;

          Logger.cmsLog({
            category: "error",
            message: cbObj.message,
            thread: "LayoutManager.videoin",
            method: "videoloadfailed",
            layoutID: layoutid,
            mediaID: media.id,
            onlyOnce: true
          });
        }
        callback();
      };
      if (LayoutManager.shouldDisplayExternalInput) {
        DeviceManager.Device.Inputs.playExternalInput(videoinloaded, videoloadfailed, {
          divId: container.id,
          sourceName: inputType
        });
      }

      /**
       * Holds the last error from underling layer to prevent floading the CMS.
       * Must be defined under the window.LayoutManager.videoin definition!
       */
      window.LayoutManager.videoin.lastError = {};

    } catch (e) {
      console.error(e);
    }
  };

}());
