/* global utils, DeviceManager, defined, b64toBA, hex2b64, byte2Hex, rc4 */
//jshint -W007, strict:false
(function () {

  /**
   * Various utilities
   * @classdesc Various utilities
   * @type utils
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W057
  window.utils = new function _utils() {

    /**
     * Clears the monitor cache.
     * @param {function} cbSuccess Callback function if clear cache succeeds.
     * @param {function} cbFailure Callback function if fail to perform the command.
     * @returns {undefined} undefined
     * @version 1.8.1
     * @since 1.8.1
     * @depricated Not Tizen compatible
     */
    //jshint -W098
    this.clearCache = function (cbSuccess, cbFailure) {
      if (cbSuccess) {
        cbSuccess();
      }
      return;
    };
    //jshint +W098

    /**
     * Enable application debug mode
     * @param {function} cbSuccess Callback function if debug mode is enabled.
     * @param {type} cbFailure Callback function if fail to enable debug mode.
     * @returns {undefined} undefined;
     * @version 1.8.1
     * @since 1.8.1
     * @depricated Not Tizen compatible
     */
    this.enableDebugMode = function (cbSuccess, cbFailure) {
      utils.clearCache(cbSuccess, cbFailure);
      return;
    };

    /**
     * Loads a page with ajax and puts it into a specific html element.
     * @param {String} source The URL to load the page from.
     * @param {String} destination The HTML element to put the contents of the loaded page.
     * @param {String} cbFunction Call the cbFunction of source page after load.
     * @param {Object | Array} cbArgument Argument(s) to pass to the cbFunction of source page.
     * @returns None
     * Scripts inside page can automaticaly run from function initrun() after load.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.loadPagetoTarget = function (source, destination, cbFunction, cbArgument) {
      this.httpGetAsync(source,
        function (response) {
          var element = document.getElementById(destination);
          element.innerHTML = response;
          if (defined(cbFunction)) {
            var objlist = element.getElementsByTagName("script");
            for (var i = 0; i < objlist.length; i++) {
              //jshint -W061
              eval(objlist[i].innerHTML);
              var pfunction = eval(cbFunction);
              //jshint +W061
              if (defined(pfunction)) {
                pfunction(cbArgument);
              }
            }
          }
        },
        function (cbError) {
          console.debug("httpGetAsync error", cbError);
        });

      return;
    };


    /**
     * Asynchronus request a web resource
     * @param {type} theUrl The Url to request from.
     * @param {type} callback Callback function to call when request complete.
     * @param {type} errorCb Callback function to call when error occurs.
     * @returns None
     * @version 1.8.1
     * @since 1.8.1
     */
    this.httpGetAsync = function (theUrl, callback, errorCb) {
      //Tizen is always in local mode.
      var appInLocalMode = true;

      var xmlHttp = new XMLHttpRequest();
      xmlHttp.onreadystatechange = function () {
        if (xmlHttp.readyState === 4) {
          if (appInLocalMode) {
            callback(xmlHttp.responseText);
          } else if (xmlHttp.status === 200) {
            callback(xmlHttp.responseText);
          }
        }
      };

      xmlHttp.onerror = function (errObj) {
        if (errorCb) {
          errorCb(errObj);
        }
      };

      //theUrl = "/mnt/lg/appstore/scap/contents/"+theUrl;

      xmlHttp.open("GET", theUrl, true); // true for asynchronous 
      xmlHttp.send(null);
    };

    /**
     * Get the current time stamp!
     * @returns {Number} The current time stamp!
     * @version 1.8.1
     * @since 1.8.1
     */
    this.getTimeStamp = function () {
      //jshint -W016
      return Date.now() / 1000 | 0;
      //jshint +W016
    };

    /**
     * Get the local date in form 'dd/MM/YYYY hh:mm:ss'.
     * @returns {String} Returns the local date.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.getLocalDate = function () {
      var d = new Date();
      var hr = d.getHours();
      if (hr < 10) {
        hr = "0" + hr;
      }
      var min = d.getMinutes();
      if (min < 10) {
        min = "0" + min;
      }
      var sec = d.getSeconds();
      if (sec < 10) {
        sec = "0" + sec;
      }
      var date = d.getDate();
      if (date < 10) {
        date = "0" + date;
      }
      var month = d.getMonth() + 1;
      if (month < 10) {
        month = "0" + month;
      }
      var year = d.getFullYear();

      return date + "/" + month + "/" + year + " " + hr + ":" + min + ":" + sec;
    };

    /**
     * Get the ISO date in form 'YYYY-MM-dd hh:mm:ss'.
     * @param {Date|Number|undefined} dt Date to format. If date is number then it will be considered as timestamp. If undefined then it will format the current time.
     * @returns {String} Returns the ISO date.
     * @version 2.0.200
     * @since 1.8.1
     */
    this.getISODate = function (dt) {
      if (!dt) {
        dt = new Date();
      } else if (Number.isInteger(dt)) {
        dt = new Date(dt);
      }
      var datestring = dt.getFullYear() + "-" + ("0" + (dt.getMonth() + 1)).slice(-2) + "-" + ("0" + dt.getDate()).slice(-2)
        + " " + ("0" + dt.getHours()).slice(-2) + ":" + ("0" + dt.getMinutes()).slice(-2) + ":" + ("0" + dt.getSeconds()).slice(-2);
      return datestring;
    };

    /**
     * Parses a date as string.
     * @param {String} string String representation of Date.
     * @returns {Number} The time stamp of the parsed date.
     * @version 2.0.202
     * @since 1.8.1
     */
    this.parseDate = function (string) {
      var d = Date.parse(string);
      d = Number.parseInt(d / 1000);
      return d;
    };

    /**
     * Check if date is within a specific range
     * @param {String|Date} start Date time or just time of range start
     * @param {String|Date} end Date time or just time of range end
     * @returns {Boolean} True if it is in range else false.
     * @throws {ERRORS.ParameterError} Throws parameter error if start or end cannot be parsed to date
     * @throws {ERRORS.ParameterNotDefined} Throws parameter not defined error if start or end is not defined
     * @version 1.8.17
     * @since 1.8.17
     */
    Date.prototype.inDateTimeRange = function (start, end) {

      if (start === end) {
        return true;
      }

      /**
       * Parse date time
       * @param {type} v Date time to parse
       * @returns {Number} Timestamp
       */
      var ts = function (v) {
        var slashes = (v.split("/").length - 1);
        var semicolon = (v.split(":").length - 1);
        var space = (v.split(" ").length - 1);

        var datePart = "";
        var timePart = "";

        var parseTime = function () {
          var t = "";
          if (space > 0) {
            t = v.split(v)[1];
          } else {
            t = v;
          }
          switch (semicolon) {
            case 0: //if number provided assume it is hour, set minutes and seconds to 0
              if (isNaN(t)) {
                throw new Error("Invalid format");
              }
              return t + ":00:00";
              break;
            case 1: //Assume that hour and minute is provided, set seconds to 0
              return t + ":00";
              break;
            case 2: //Full time provided
              return t;
              break;
            default: //Everything else is error
              throw new Error("Invalid format");
              break;
          }
        };

        var parseDate = function () {
          var dt = new Date();
          var d = "";
          if (space > 0) {
            d = v.split(v)[0];
          } else {
            return (dt.getMonth() + 1) + "/" + dt.getDate() + "/" + dt.getFullYear();
          }
          switch (slashes) {
            case 0: //if number provided assume it is year, set month and day to current date
              return (dt.getMonth() + 1) + "/" + dt.getDate() + "/" + d;
              break;
            case 1: //Assume that year and month is provided, set day to current date
              //Tricky date will come in form mm/yy[yy] but has to be set to mm/dd/yy[yy]
              var dsplit = d.split("/");
              return dsplit[0] + "/" + dt.getDate() + "/" + dsplit[1];
              break;
            case 2: //Full date provided
              //Tricky date will come in form dd/mm/yy[yy] but has to be set to mm/dd/yy[yy]
              var dsplit = d.split("/");
              return dsplit[1] + "/" + dsplit[0] + "/" + dsplit[2];
              break;
            default: //Everything else is error
              throw new Error("Invalid format");
              break;
          }
        };

        if (typeof v === "string") {

          if ((slashes + semicolon) === 0) {
            throw new Error("Not a date");
          }

          if (space > 1) {
            throw new Error("Invalid format");
          }
          try {
            datePart = parseDate();
          } catch (ed) {
            //Do nothing as it might be time.
          }
          try {
            timePart = parseTime();
          } catch (et) {
            if (datePart === "") {
              throw new Error("Not a date");
            }
          }

          return (new Date(datePart + " " + timePart) * 1);

        } else { //Date is provided, just return timestamp.
          return v * 1;
        }
      };


      if (start === undefined) {
        throw ERRORS.ParameterNotDefined("Range start not defined");
      }
      if (end === undefined) {
        throw ERRORS.ParameterNotDefined("Range end not defined");
      }


      var s;
      var e;
      try {
        s = ts(start);
      } catch (e) {
        throw ERRORS.ParameterError("Error in start range: " + e.message);
      }

      try {
        e = ts(end);
      } catch (e) {
        throw ERRORS.ParameterError("Error in end range: " + e.message);
      }

      if (s > e) {
        return Date.parse(this) <= e || Date.parse(this) >= s;
      }

      if ((Date.parse(this) >= s) && (Date.parse(this) <= e)) {
        return true;
      }

      return false;
    };


    /**
     * Various conversion functions.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.conversions = new function _conversions() {
      /**
       * Convert a string to byte array
       * @param {String} data String to convert to byte array.
       * @returns {Array} Byte array of string.
       */
      this.str2BA = function (data) {
        var ar = [];

        for (var i = 0; i < data.length; ++i) {
          var code = data.charCodeAt(i);
          //jshint -W016
          var cUpByte = code / 256 >>> 0;
          var cDownByte = code & 0xff;
          //jshint +W016

          if (cUpByte > 0) {
            ar.push(cUpByte);
          }
          ar.push(cDownByte);
        }
        return ar;
      };


      /**
       * Convert a hex string to byte array
       * @param {String} data Hex string to convert into byte array.
       * @returns {Array} Byte array of hex string.
       */
      this.hex2BA = function (data) {
        var chars = [];
        var j = 0;
        while (j < data.length) {
          var v = parseInt("0x" + (data.charAt(j) + data.charAt(j + 1))) % 255;
          j = j + 2;
          chars.push(v);
        }

        return chars;
      };

//			var str2Hex = function(data){
//				var strHex = [];
//
//				for (var i = 0; i < data.length; ++i) {
//					var code = data.charCodeAt(i);
//					var cUpByte = code / 256 >>> 0;
//					var cDownByte = code & 0xff;
//
//					if (cUpByte>0)
//					{
//						strHex.push(byte2Hex(cUpByte));
//					}
//					strHex.push(byte2Hex(cDownByte));
//				}
//				return strHex;
//			};

//			var str2BA = function(data) {
//				var ar = [];
//
//				for (var i = 0; i < data.length; ++i) {
//					var code = data.charCodeAt(i);
//					var cUpByte = code / 256 >>> 0;
//					var cDownByte = code & 0xff;
//
//					if (cUpByte>0)
//					{
//						ar.push(cUpByte);
//					}
//					ar.push(cDownByte);
//				}
//				return ar;
//			};

//			var ba2String = function (array) {
//				var chars = [];
//				for(var i = 0, n = array.length; i < n;) {
//					chars.push(((array[i++] & 0xff) << 8) | (array[i++] & 0xff));
//				}
//				return String.fromCharCode.apply(null, chars);
//			};        

    }();

    /*
     * Equivelant functions to JQUERY
     */

    /**
     * Selects an element. (jquery $(el))
     * @param {Object | String} el The element could be the element it self or element id or element class (only the first will be returned).
     * @returns {Object} The element from selection.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.elementSelector = function elementSelector(el) {
      if (typeof el === 'string') {
        var e = document.querySelector(el);
        return e;
      } else {
        return el;
      }
    };

    /**
     * Hide an element. (jquery $(el).hide())
     * @param {Object | String} el The element to hide.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.hide = function hide(el) {
      var e = utils.elementSelector(el);
      if (e) {
        e.style.display = 'none';
      }
    };

    /**
     * Show an element. (jquery $(el).show())
     * @param {Object | String} el The element to show.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.show = function show(el) {
      var e = utils.elementSelector(el);
      if (e) {
        e.style.display = 'block';
      }
    };

    /**
     * Remove element's content. (jquery $(el).empty())
     * @param {Object | String} el The element to empty.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.empty = function empty(el) {
      var e = utils.elementSelector(el);
      if (e) {
        e.innerHTML = '';
      }
    };

    /**
     * Remove element from DOM. (jquery $(el).remove())
     * @param {Object | String} el The element to remove.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.remove = function remove(el) {
      var e = utils.elementSelector(el);
      if (e) {
        e.parentNode.removeChild(e);
      }
    };

//		/**
//		 * *** WARNING *** It is not implemented! *** WARNING ***
//		 * Remove all event handlers from element. (jquery $(el).off())
//		 * @param {Object | String} el The element from which event handlers will be removed.
//		 * @version 1.8.1
//		 * @since 1.8.1
//		 */
//		this.off = function off(el){
//			var e = utils.elementSelector(el);
//			if (e){
//				for (var k in getEventListeners(e)) {
//					console.log(k);
//				}
//			}
//		};


    var fadeAnimation = function fadeAnimation(animStr, el, callback) {
      var cb = function () {
        el.removeEventListener("webkitAnimationEnd", cb);
        el.removeEventListener("animationend", cb);
        if (callback) {
          callback();
        }
      };

      el.addEventListener("webkitAnimationEnd", cb);
      el.addEventListener("animationend", cb);
      el.style.animation = animStr;
    };

    /**
     * Fade in effect. (jquery $(el).fadeIn(duration, cb))
     * @param {Object} el The element to fade in.
     * @param {Number} duration The duration of the effect.
     * @param {Function} callback Callback when effect complete.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.fadeIn = function fadeIn(el, duration, callback) {
      var e = utils.elementSelector(el);

      var animCssStr = "kfSimpleFadeIn " + duration + "ms linear 0s 1 normal";
      if (e) {
        fadeAnimation(animCssStr, e, callback);
        this.show(e);
      }

    };

    /**
     * Fade out effect. (jquery $(el).fadeOut(duration, cb))
     * @param {Object} el The element to fade out.
     * @param {Number} duration The duration of the effect.
     * @param {Function} callback Callback when effect complete.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.fadeOut = function fadeOut(el, duration, callback) {
      var e = utils.elementSelector(el);

      var animCssStr = "kfSimpleFadeOut " + duration + "ms linear 0s 1 normal";
      if (e) {
        this.show(e);
        fadeAnimation(animCssStr, e, function () {
          utils.hide(e);
          callback();
        });
      }
    };


    /**
     * Animate transition with hardware acceleration. (jquery $(el).animate(duration, cb))
     * @param {Object} el The element to animate.
     * @param {Object} prop The properties to change.
     * @param {Number} duration The duration of the effect.
     * @param {Function} callback Callback when effect complete.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.animate = function animate(el, prop, duration, callback) {
      var e = utils.elementSelector(el);
      if (e) {

        var pr = '';
        var left = parseInt(e.style.left);
        var top = parseInt(e.style.top);
        var finalLeft;
        var finalTop;

        for (var p in prop) {
          if (p === 'left') {
            finalLeft = prop[p];
            pr += 'translateX(' + (parseInt(prop[p]) - left) + 'px) ';
          } else if (p === 'right') {
            finalLeft = prop[p];
            pr += 'translateX(' + (parseInt(prop[p]) - left) + 'px) ';
          } else if (p === 'top') {
            finalTop = prop[p];
            pr += 'translateY(' + (parseInt(prop[p]) - top) + 'px) ';
          } else if (p === 'bottom') {
            finalTop = prop[p];
            pr += 'translateY(' + (parseInt(prop[p]) - top) + 'px) ';
          }
        }


        var cb = function () {
          e.removeEventListener("transitionend", cb);
          e.removeEventListener("webkitTransitionEnd", cb);
          e.style.left = finalLeft;
          e.style.top = finalTop;
          e.style.transition = '';
          e.style.webkitTransition = '';
          e.style.transform = '';
          e.style.webkitTransform = '';

//					if (callback){
//					callback();
//					}
        };

        e.addEventListener("transitionend", cb);
        e.addEventListener("webkitTransitionEnd", cb);

        // Give some time to DOM to settle down!
        setTimeout(function () {
          //e.style.webkitTransition = "-webkit-transform "+ duration +" ease-in-out";
          //e.style.transition = "transform "+ duration +" ease-in-out"
          e.style.transition = "transform " + duration + "ms ease-in-out";
          e.style.webkitTransform = "scale3d(1,1,1) " + pr;

        }, 100);

        setTimeout(function () {
          if (callback) {
            callback();
          }
        }, duration);

      }
    };

    /**
     * Get element width. If width is % it will be calculated.
     * @param {Object} el The element to get width from;
     * @returns {Number} The width of the element.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.getElementWidth = function (el) {
      var e = utils.elementSelector(el);
      if (e.constructor === Window) {
        var o = DeviceManager.Device.getOrientation();
        if (o === '1' || o === '9') {
          return e.screen.height;
        }

        return e.screen.width;
      }
      if (e) {
        var wp;
        if (e.style.width.indexOf('%') > -1) {
          wp = parseFloat(e.style.width);
          if (e.parentNode !== null) {
            return utils.getElementWidth(e.parentNode) * wp / 100;
          }
        } else {
          return parseFloat(e.style.width);
        }

      }
    };

    /**
     * Get element height. If height is % it will be calculated.
     * @param {Object} el The element to get height from;
     * @returns {Number} The height of the element.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.getElementHeight = function (el) {
      var e = utils.elementSelector(el);
      if (e.constructor === Window) {
        var o = DeviceManager.Device.getOrientation();
        if (o === '1' || o === '9') {
          return e.screen.width;
        }
        return e.screen.height;
      }

      if (e) {
        var wp;
        if (e.style.height.indexOf('%') > -1) {
          wp = parseFloat(e.style.height);
          if (e.parentNode !== null) {
            return utils.getElementHeight(e.parentNode) * wp / 100;
          }
        } else {
          return parseFloat(e.style.height);
        }
      }
    };

    /**
     * Replace Xibo scaler based on jQuery with this non-jQuery version.
     * @param {Object | String} el The element to scale its children.
     * @param {Object} scalerOptions Options to scaler.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.scaler = function (el, scalerOptions) {
      var e = utils.elementSelector(el);
      var width;
      var height;
      var newWidth;
      var newHeight;

      // Default options
      var defaults = {
        "originalWidth": 0,
        "originalHeight": 0,
        "previewWidth": 0,
        "previewHeight": 0,
        "scaleOverride": 0,
        "widgetDesignWidth": 0,
        "widgetDesignHeight": 0,
        "widgetDesignPadding": 0,
        "itemsPerPage": 0
      };

      var options = defaults;
      for (var o in scalerOptions) {
        if (options.hasOwnProperty(o)) {
          options[o] = scalerOptions[o];
        }
      }

      // Region original width and height
      var originalWidth = options.originalWidth;
      var originalHeight = options.originalHeight;

      if (options.previewWidth === 0 || options.previewHeight === 0) {
//				width = jQueryWebOs(window).width();
//				height = jQueryWebOs(window).height();
        width = utils.getElementWidth(window);
        height = utils.getElementHeight(window);
      } else {
        width = options.previewWidth;
        height = options.previewHeight;
      }

      // Calculate the ratio to apply as a scale transform
      var ratio = Math.min(width / options.originalWidth, height / options.originalHeight);

      // The layout designer might provide a scale override, which we need to use because we cannot tell whether
      // the region has been resized or not. If it has been resized in the designer, but not saved, the original
      // width and height will be incorrect (the old saved values). Scale override provides the correct
      // scale in that case.
      if (options.scaleOverride !== 0) {
        ratio = options.scaleOverride;
      }

      newWidth = width / ratio;
      newHeight = height / ratio;

      // The widget is scaled according to the layout dimensions

      // Does the widget have an original design width/height
      // if so, we need to further scale the widget        
      if (options.widgetDesignWidth > 0 && options.widgetDesignHeight > 0) {

        if (options.itemsPerPage > 0) {
          if (newWidth > newHeight) {
            //Landscape or square size plus padding
            options.widgetDesignWidth = (options.itemsPerPage * options.widgetDesignWidth) + (options.widgetDesignPadding * (options.itemsPerPage - 1));
            //options.widgetDesignHeight = options.widgetDesignHeight;
          } else {
            //Portrait size plus padding
            options.widgetDesignHeight = (options.itemsPerPage * options.widgetDesignHeight) + (options.widgetDesignPadding * (options.itemsPerPage - 1));
            //options.widgetDesignWidth = options.widgetDesignWidth;
          }
        }

        // Calculate the ratio between the new
        var widgetRatio = Math.min(newWidth / options.widgetDesignWidth, newHeight / options.widgetDesignHeight);

        ratio = ratio * widgetRatio;
        newWidth = width / ratio;
        newHeight = height / ratio;
      }

//			for (var i = 0, max = e.children.length; i < max; i++) {
//				e.children[i].style.width = newWidth+"px";
//				e.children[i].style.height = newHeight+"px";
//
//				e.children[i].style.transform = "scale(" + ratio + ")";
//				e.children[i].style.transformOrigin = "0 0";
//			}
      e.parentElement.style.width = originalWidth * ratio;
      e.parentElement.style.height = originalHeight * ratio;
      e.style.width = originalWidth + "px";
      e.style.height = originalHeight + "px";
      e.style.transform = "scale(" + ratio + ")";
      e.style.transformOrigin = "0 0";
    };

    /**
     * Set background image to an element.
     * @param {Object | String} el The element to set the backgroung image.
     * @param {String} url The url of the image.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.setBackground = function (el, url) {
      var e = utils.elementSelector(el);
      if (e) {
        var canvas = document.createElement('canvas');
        canvas.id = e.id + '_background';
        canvas.setAttribute('width', el.style.width);
        canvas.setAttribute('height', el.style.height);

        el.appendChild(canvas);
        var ctx = canvas.getContext("2d");
        var img = new Image();

        img.onload = function () {

          // set size proportional to image
          // step 1 - resize to 50%
          var oc = document.createElement('canvas'),
            octx = oc.getContext('2d');

          oc.width = utils.getElementWidth(e);
          oc.height = utils.getElementHeight(e);
          octx.drawImage(img, 0, 0, oc.width, oc.height);

          // step 2
          octx.drawImage(oc, 0, 0, oc.width, oc.height);

          // step 3, resize to final size
          ctx.drawImage(oc,
            0, 0, oc.width, oc.height,
            0, 0, canvas.width, canvas.height);

          img = oc = ctx = octx = null;
        };
        img.src = url;
      }
    };

    /**
     * Remove background image from an element
     * @param {Object | String} el The element to remove background image from.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.removeBackground = function (el) {
      var e = utils.elementSelector(el);
      if (e) {
        var c = document.getElementById(e.id + '_background');
        if (c) {
          e.removeChild(c);
        }
      }
    };

    // Take an image URL, downscale it to the given size (width,height), and return a new image URL.
    /**
     * Take an image URL, downscale it to the given size (width,height), and return a new image URL.
     * @param {String} dataUrl The url to get the image
     * @param {Number} newWidth The resized width of the new image.
     * @param {String} imageType The new image type. Default is 'image/jpeg'.
     * @param {Number} compression The compression ratio of the new image. Default is 1.
     * @param {Function} cb Callback function when resize completes.
     * @returns {utils.utilsutils.downscaleImage.newDataUrl}
     * @version 1.8.1
     * @since 1.8.1
     */
    this.downscaleImage = function downscaleImage(dataUrl, newWidth, imageType, compression, cb) {
      var image, oldWidth, oldHeight, newHeight, canvas, ctx, newDataUrl;

      // Provide default values
      imageType = imageType || "image/jpeg";
      compression = compression || 1;

      // Create a temporary image so that we can compute the height of the downscaled image.
      image = new Image();
      image.setAttribute('crossOrigin', 'anonymous');
      image.onload = function () {
        oldWidth = image.width;
        oldHeight = image.height;
        newHeight = Math.floor(oldHeight / oldWidth * newWidth);


        // Create a temporary canvas to draw the downscaled image on.
        canvas = document.createElement("canvas");
        canvas.width = newWidth;
        canvas.height = newHeight;

        // Draw the downscaled image on the canvas and return the new data URL.
        ctx = canvas.getContext("2d");
        ctx.drawImage(image, 0, 0, newWidth, newHeight);
        newDataUrl = canvas.toDataURL(imageType, compression);
        //console.log("DownscaleImage: Old size:["+oldWidth+","+oldHeight+"], new:["+newWidth+","+newHeight+"]");
        if (cb) {
          cb(newDataUrl);
        }
      };
      image.src = dataUrl;
    };

    /**
     * Define a constant value to an object
     * @param {type} obj The object to define the constant.
     * @param {type} name The name of the contanst.
     * @param {type} value The value of the constant.
     * @version 1.8.1
     * @since 1.8.1
     */
    this.const = function (obj, name, value) {
      Object.defineProperty(obj, name, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: false
      });
    };
    /**
     * Execute a HTTP Command
     * @param {type} url
     * @param {type} requestType
     * @param {type} paramText
     * @returns {undefined}
     * @version 3.302.0
     * @since 3.302.0
     */
    this.executeHTTPCommand = function (url, requestType, paramText) {
      var req = new XMLHttpRequest();
      req.withCredentials = true;
      var params, headers;
      var body = "";

      try {
        params = JSON.parse(paramText);

        headers = JSON.parse(params.headers);

        if (requestType === "application/x-www-form-urlencoded") {
          body = params.body;
        } else if (requestType === "application/json") {
          body = JSON.parse(params.body);
        } else { // Plain Text
          body = params.body;
        }
      } catch (e) {
        console.error("Unable to parse Shell command HTTP parameters.", e);
        return;
      }

      req.addEventListener("load", function () {
        console.log('Request returned back', req.status);
      });

      req.open(params.method, url);

      req.setRequestHeader('Content-Type', requestType);
      if (headers) {
        var headerKeyArray = Object.keys(headers);
        for (var i = 0, max = headerKeyArray.length; i < max; i++) {
          req.setRequestHeader(headerKeyArray[i], headers[headerKeyArray[i]]);
        }
      }
      req.send(body);
    };


    /**
     * Create a range of statistiscs
     * @param {function} cb Callback function after creation of statistics.<br>
     * cb returns the number of created statistics.
     * @param {Date} startDate The date and time statistics starts.
     * @param {Date} endDate The date and time statistics ends.
     * @param {Number} duration The duration of each statistic in sec.
     * @param {String|Number} mediaid The media id (if type is "media")
     * @param {String|Number} layoutid The layout id (if type is "media")
     * @param {String|Number} scheduleid The schedule id
     * @param {String} type Type of stat. Can be "media"|"layout".
     * @returns {undefined}
     */
    this.createSampleStats = function (cb, startDate, endDate, duration, mediaid, layoutid, scheduleid) {

      var sdt = startDate;
      var edt;
      var statEndt;
      var count = 0;
      var preStat = [];
      var calcDuration = duration * 1000;

      /* For displaying progress */
      //    var displCount = 0;

      while (true) {
        edt = new Date(sdt * 1 + (calcDuration));
        if (!statEndt) {
          statEndt = utils.getISODate(edt).substr(0, 10);
        }

        //console.log(formatDate(sdt)+'  -  '+formatDate(edt));
        preStat.push({
          fromdt: utils.getISODate(sdt),
          todt: utils.getISODate(edt),
          mediaid: mediaid,
          layoutid: layoutid,
          scheduleid: scheduleid
        });

        sdt = edt;
        var nextEdt = new Date(edt * 1 + (calcDuration));
        if (statEndt !== utils.getISODate(nextEdt).substr(0, 10)) {
          statEndt = undefined;
        }

        if (edt >= endDate) {
          break;
        }
      }


      function int_addStat() {
        Logger.logStat(preStat[count], function () {
          count++;
          /* For displaying progress */
          //displCount = (count/preStat.length*1000)|0;
          //if ((displCount%100)===0){
          //console.log("Stats created ",(displCount/10)+"%");
          //}
          if ((count % 1000) === 0) {
            console.log("Stats created ", ((count / preStat.length * 100) | 0) + "%");
          }
          if (count < (preStat.length)) {
            //          setTimeout(function(){
            int_addStat();
            //          },0);
          } else {
            cb(preStat.length);
          }
        }, function () {
          console.error('Unable to add stat', count, preStat[count]);
        });
      };
      int_addStat();
    };
  }();

  /**
   * @classdesc Check if an object is defined.
   * @param {object} obj The object to check.
   * @returns {Boolean} True if it is defined else false.
   * @version 1.8.1
   * @since 1.8.1
   */
  this.defined = function (obj) {
    return (typeof obj !== 'undefined') && obj !== null;
  };

  /**
   * RC4 symmetric encryption/decryption .
   * @param {Array} key Byte array of key
   * @param {Array} str Byte array of encrypted/decrypted data
   * @param {"string"|"hex"|"array"} returnAs Return value as string (default) or hex string or as byte array.
   * @returns {String | Array} String or Byte array of decrypted/ecrypted data
   * @version 1.8.1
   * @since 1.8.1
   */
  this.rc4 = function (key, str, returnAs) {
    if (key === null || str === null) {
      return null;
    }
    var s = [], j = 0, k, x, res = '', z = [];
    for (var i = 0; i < 256; i++) {
      s[i] = i;
    }
    for (i = 0; i < 256; i++) {
      j = (j + s[i] + key[i % key.length]) % 256;
      x = s[i];
      s[i] = s[j];
      s[j] = x;
    }
    i = 0;
    j = 0;

    var myStr = '';

    for (var y = 0; y < str.length; y++) {
      i = (i + 1) % 256;
      j = (j + s[i]) % 256;
      x = s[i];
      s[i] = s[j];
      s[j] = x;
      k = s[(s[i] + s[j]) % 256];
      //res += String.fromCharCode(str[y] ^ k);
      //jshint -W016
      res += byte2Hex(str[y] ^ k);
      z.push(str[y] ^ k);
      myStr += String.fromCharCode(str[y] ^ k);
      //jshint +W016
    }

    if (defined(returnAs)) {

      if (returnAs === "hex") {
        return res;
      } else if (returnAs === "array") {
        return z;
      }
    }
    return myStr;
  };


  /*
   * Secure storage functions
   */

  /**
   * The rc4 salt.
   */
  var rc4crypt = [112, 142, 174, 242, 141, 124, 216, 124, 106, 47, 151, 150, 130, 93, 56, 144, 97, 215, 119, 172, 43, 38, 213, 69, 30, 103, 161, 211, 203, 47, 97, 183, 123, 38, 220, 203, 213, 93, 194, 123];

  /**
   * Save a key,value pair in localstorage encrypted.
   * @param {String} key The key.
   * @param {String} value The value
   * @returns None
   * @version 1.8.1
   * @since 1.8.1
   */
  //jshint -W103
  localStorage.__proto__.SecureSave = function (key, value) {
    var cvtValue;
    if (typeof value === "string") {
      cvtValue = value;
    } else if (typeof value === "number") {
      cvtValue = value.toString();
    } else { /* Probably object */
      cvtValue = JSON.stringify(value);
    }

    /* Not used */
    var barr = [];
    for (var i = 0, max = cvtValue.length; i < max; i++) {
      barr.push(cvtValue.charCodeAt(i));
    }
    /* Not used */

    /* Working*/
    //var ba = rc4(rc4crypt, cvtValue, "hex");
    /* Working*/
    var ba = rc4(rc4crypt, barr, "hex");

    var b64ba = hex2b64(ba);
    localStorage[key] = b64ba;
  };

  /**
   * Get a previously encrypted value from key.
   * @param {type} key The key.
   * @returns {undefined|Object|String|Number} The value.
   * @version 1.8.1
   * @since 1.8.1
   */
  localStorage.__proto__.SecureLoad = function (key) {
    if (!localStorage.hasOwnProperty(key)) {
      return Window.undefined;
    }


    var b64ba = b64toBA(localStorage[key]);

    var ba = rc4(rc4crypt, b64ba, "array");


    var hex2str = "";
    for (var i = 0, max = ba.length; i < max; i++) {
      hex2str += String.fromCharCode(ba[i]);
    }


    var value = JSON.parse(hex2str);
    if (value === null) {
      value = Number.parseInt(ba);
    }

    if (value === null) {
      value = ba.toString();
    }


    return value;
  };
  //jshint +W103


}());